package favi;

import java.awt.*;
import javax.swing.text.*;
import jGDK.F0Automaton;

public class F0Document extends DefaultStyledDocument 
{
	public static final String REGULAR	= "0";
	public static final String FORMAT	= "1";
	public static final String NUMBER	= "2";
	public static final String WORD		= "3";
	public static final String VARIABLE = "4";
	public static final String QUOTE	= "5";
	public static final String COMMAND	= "6";
	public static final String OPERATOR = "7";
	public static final String COMMENT	= "8";

    public F0Document() 
	{
		StyleContext styleContext = StyleContext.getDefaultStyleContext();
		Style def = styleContext.getStyle(StyleContext.DEFAULT_STYLE);

		Style regular = addStyle(REGULAR, def);
		StyleConstants.setFontFamily(def, "monospaced");
		StyleConstants.setFontSize(regular,12); 
    
		Style format = addStyle(FORMAT,regular);
		StyleConstants.setBold(format, true);
		StyleConstants.setForeground(format, new Color(0, 120, 0));

		Style number = addStyle(NUMBER,regular);
		StyleConstants.setBold(number, true);
		StyleConstants.setForeground(number, Color.black);

		Style variable = addStyle(VARIABLE,regular);
		StyleConstants.setBold(variable, true);
		StyleConstants.setForeground(variable, Color.blue);

		Style quote = addStyle(QUOTE,regular);
		StyleConstants.setForeground(quote, Color.magenta);

		Style comment = addStyle(COMMENT,regular);
		StyleConstants.setForeground(comment, new Color(0, 160, 0));
		StyleConstants.setItalic(comment, true);

		Style command = addStyle(COMMAND,regular);
		StyleConstants.setBold(command, true);
		StyleConstants.setForeground(command, Color.red);
		
		Style operator = addStyle(OPERATOR,regular);
		StyleConstants.setBold(operator, true);
		StyleConstants.setForeground(operator, Color.black);
	}

	public void insertString(int offs, String str, AttributeSet a) 
		throws BadLocationException 
	{
		super.insertString(offs, str, getStyle(REGULAR));
		Element root = getDefaultRootElement();
		int firstParagraph = root.getElementIndex(offs);
		int lastParagraph = root.getElementIndex(offs+str.length());
		for(int i=0;i<1+lastParagraph-firstParagraph;i++)
			scanParagraph(root.getElement(firstParagraph+i));
	}

	public void remove(int offs, int len) 
		throws BadLocationException
	{
        super.remove(offs, len);
		Element root = getDefaultRootElement();
		int firstParagraph = root.getElementIndex(offs);
		int lastParagraph = root.getElementIndex(offs+len);
		for(int i=0;i<1+lastParagraph-firstParagraph;i++)
			scanParagraph(root.getElement(firstParagraph+i));
	}

	protected void scanParagraph(Element paragraph) 
		throws BadLocationException
	{
		int startPosition = paragraph.getStartOffset();
		int length = (paragraph.getEndOffset()<getLength() ? paragraph.getEndOffset() : getLength())-startPosition;
		String content = getText(startPosition,length);
		 
		char character;
		int start = 0 ;
		int end = 0;

		F0Automaton automaton = new F0Automaton(content);
	
		start = getNotWhitespaceChar(content, 0);
		if (start == -1) return;

		if (content.charAt(start)=='#') {
			setCharacterAttributes(startPosition+start, length-start, getStyle(COMMENT),true);
			return;
		}

		int type = automaton.nextToken();

		if (type == F0Automaton.ID_CLASS) {
			start = content.indexOf(':', start);
			setCharacterAttributes(startPosition, start+1, getStyle(COMMAND),true);
		} else {
			if (type == F0Automaton.ID_FORMAT){
				setCharacterAttributes(startPosition, length, getStyle(FORMAT),true);
			}
			else {
				setCharacterAttributes(startPosition, length, getStyle(REGULAR),true);
				return;
			}
		  }

		String variable;
		String argument;

		while (automaton.nextToken() != F0Automaton.ID_EOL && automaton.tokenMaster.type != F0Automaton.ID_EOF)
		{
			switch (automaton.tokenMaster.type)
			{
			case F0Automaton.ID_VARIABLE:
				variable = automaton.tokenMaster.data;
				start = content.indexOf(variable, start);
				setCharacterAttributes(startPosition+start, variable.length(), getStyle(VARIABLE), true);
				start+=variable.length();
				if (automaton.tokenSlave.type == F0Automaton.ID_QUOTE) {
					argument = automaton.tokenSlave.data;
					start = content.indexOf(argument, start);
					setCharacterAttributes(startPosition+start, argument.length(), getStyle(QUOTE), true);
					start+=argument.length();
					break;
				}
				argument = automaton.tokenSlave.data;
				start = content.indexOf(argument, start);
				setCharacterAttributes(startPosition+start, argument.length(),  getStyle(REGULAR), true);
				start+=argument.length();
				break;
			case F0Automaton.ID_WORD:
				argument = automaton.tokenMaster.data;
				start = content.indexOf(argument, start);
				setCharacterAttributes(startPosition+start, argument.length(),  getStyle(REGULAR), true);
				start+=argument.length();
				break;
			case F0Automaton.ID_QUOTE:
				argument = automaton.tokenMaster.data;
				start = content.indexOf(argument, start);
				setCharacterAttributes(startPosition+start, argument.length(), getStyle(QUOTE), true);
				start+=argument.length();
				break;
			default:
				setCharacterAttributes(startPosition+start, length-start,  getStyle(REGULAR), true);
				return;
			}
		}

	}

	private int getNotWhitespaceChar(String string, int currentPosition)
	{
		for (int i = currentPosition;  i < string.length(); i++) 
			if (!Character.isWhitespace(string.charAt(i))) 
				return i;
		return -1;
	}

/*
	private int getIdentifier(String string, int currentPosition)
	{
		for (int i = currentPosition;  i < string.length(); i++) 
			if (!Character.isJavaIdentifierPart(string.charAt(i))) 
				return i;

		return -1;
	}
	*/
}   



