package favi;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.undo.*;
import javax.swing.tree.*;
import javax.swing.border.*;

import jGDK.Framstick;
import jGDK.ErrManager;
import jGDK.I18N;

public class F0Viewer extends JFrame {
	DefaultTreeModel treeModel;
	DefaultMutableTreeNode mainNode;
	DefaultMutableTreeNode warnings;
	DefaultMutableTreeNode errors;
	DefaultMutableTreeNode hints;

	DefaultStyledDocument linijkaDoc;

	SimpleAttributeSet attrs;
	JTextPane textPane;
	Hashtable actions;

	F0Document doc;
	private Framstick framstick;
	ErrManager errManager;

	protected UndoAction undoAction;
	protected RedoAction redoAction;
	protected ChangeAction changeAction;
	protected Vector undoBuffer = new Vector();
	protected Vector redoBuffer = new Vector();

	CompoundEdit compoundEdit;

	boolean compiled = true;

	public F0Viewer(Framstick fram) {
		this.setTitle(I18N.getMessage("f0Genotype"));

		//Create the document for the text area.
		framstick = fram;
		doc = new F0Document();
		textPane = new JTextPane(doc);
		textPane.setMargin(new Insets(5, 5, 5, 5));
		textPane.setEditable(false);

		actions = new Hashtable();
		createActionTable(textPane);

		attrs = new SimpleAttributeSet();
		StyleConstants.setBold(attrs, true);
		StyleConstants.setFontFamily(attrs, "monospaced");
		StyleConstants.setFontSize(attrs, 12);
		StyleConstants.setForeground(attrs, new Color(130, 130, 130));


		JPanel noWrapPanel = new JPanel();
		noWrapPanel.setLayout(new BorderLayout());
		noWrapPanel.add(textPane);

		JScrollPane scrollPane = new JScrollPane(noWrapPanel);
		scrollPane.setBorder(new EmptyBorder(new Insets(0, 0, 0, 0)));

		mainNode = new DefaultMutableTreeNode("Messages");
		treeModel = new DefaultTreeModel(mainNode);
		errors = new DefaultMutableTreeNode("Errors");
		warnings = new DefaultMutableTreeNode("Warnings");
		hints = new DefaultMutableTreeNode("Hints");

		mainNode.add(errors);
		mainNode.add(warnings);
		mainNode.add(hints);

		linijkaDoc = new DefaultStyledDocument();
		JTextPane linijka = new JTextPane(linijkaDoc);
		linijka.setMargin(new Insets(5, 5, 5, 5));
		linijka.setCaretPosition(0);
		linijka.setEditable(false);

		scrollPane.setRowHeaderView(linijka);

		//Add the components to the frame.
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BorderLayout());
		contentPane.setPreferredSize(new Dimension(500, 400));
		contentPane.add(scrollPane, BorderLayout.CENTER);
		setContentPane(contentPane);

		//Start watching for undoable edits and caret changes.
		doc.addUndoableEditListener(new MyUndoableEditListener());
		doc.addDocumentListener(new MyDocumentListener());
		this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
	}

	private void createActionTable(JTextComponent textComponent) {
		Action[] actionsArray = textComponent.getActions();
		for (int i = 0; i < actionsArray.length; i++) {
			Action a = actionsArray[i];
			actions.put(a.getValue(Action.NAME), a);
		}
	}

	private Action getActionByName(String name) {
		return (Action) (actions.get(name));
	}

	protected class MyDocumentListener implements DocumentListener {
		int liczba_linii;

		MyDocumentListener() {
			liczba_linii = 0;
		}

		public void insertUpdate(DocumentEvent e) {
			displayEditInfo(e);
		}

		public void removeUpdate(DocumentEvent e) {
			displayEditInfo(e);
		}

		public void changedUpdate(DocumentEvent e) {
			displayEditInfo(e);
		}

		private void displayEditInfo(DocumentEvent e) {
			compiled = false;
			Document document = e.getDocument();
			Element rootElement = document.getDefaultRootElement();
			int axx = rootElement.getElementCount();
			if (liczba_linii < axx) {
				try {
					for (int i = 1; i <= (axx - liczba_linii); i++) {
						linijkaDoc.insertString(linijkaDoc.getLength(), String.valueOf(++liczba_linii) + "\n", attrs);
					}
				}
				catch (Exception ex) {
					;
				}
			}
		}
	}

	public void initDocument() {
		try {
			doc.remove(0, doc.getLength());
			doc.insertString(doc.getLength(), framstick.genotype, attrs);
		}
		catch (BadLocationException ble) {
			System.err.println("Couldn't insert initial text.");
		}
		textPane.setCaretPosition(0);
		ErrManager.msg.clear();
	}

	//This one listens for edits that can be undone.
	protected class MyUndoableEditListener implements UndoableEditListener {
		public void undoableEditHappened(UndoableEditEvent e) {
			//Remember the edit and update the menus.
			UndoableEdit sort = e.getEdit();
			if (sort.getPresentationName().equals("addition") || sort.getPresentationName().equals("deletion")) {
				if (compoundEdit == null) {
					compoundEdit = new CompoundEdit();
				}
				else {
					compoundEdit.end();
					compoundEdit = new CompoundEdit();
				}
				compoundEdit.addEdit(e.getEdit());
				undoBuffer.add(compoundEdit);
				redoBuffer.clear();
			}
			else
				compoundEdit.addEdit(e.getEdit());


		}
	}

	class ChangeAction extends AbstractAction {
		public ChangeAction() {
			super();
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			try {
				doc.remove(0, doc.getLength());
				doc.insertString(doc.getLength(), framstick.genotype, attrs);
			}
			catch (BadLocationException ble) {
				System.err.println("Couldn't insert initial text.");
			}
			textPane.setCaretPosition(0);
			ErrManager.msg.clear();
			initDocument(); //Jarzbi
			setEnabled(false);
			compiled = true;

		}

		protected void setAvailable() {
			setEnabled(true);
		}
	};

	class UndoAction extends AbstractAction {
		public UndoAction() {
			super();
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			CompoundEdit cEdit = (CompoundEdit)undoBuffer.remove(undoBuffer.size() - 1);
			if (cEdit.isInProgress())
				cEdit.end();
			redoBuffer.add(cEdit);
			cEdit.undo();
			updateUndoState();
			redoAction.updateRedoState();
		}

		protected void updateUndoState() {
			if (undoBuffer.size() > 0)
				setEnabled(true);
			else
				setEnabled(false);
		}
	}

	class RedoAction extends AbstractAction {
		public RedoAction() {
			super();
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			CompoundEdit cEdit = (CompoundEdit)redoBuffer.remove(redoBuffer.size() - 1);
			if (cEdit.isInProgress())
				cEdit.end();
			undoBuffer.add(cEdit);

			cEdit.redo();
			updateRedoState();
			undoAction.updateUndoState();
		}

		protected void updateRedoState() {
			if (redoBuffer.size() > 0)
				setEnabled(true);
			else
				setEnabled(false);
		}
	}

	
	public void internalFrameOpened(InternalFrameEvent e) {	
	}
	public void internalFrameClosing(InternalFrameEvent e) {
	}
	public void internalFrameClosed(InternalFrameEvent e) {
	}
	public void internalFrameIconified(InternalFrameEvent e) {
	}
	public void internalFrameDeiconified(InternalFrameEvent e) {
	}
	public void internalFrameActivated(InternalFrameEvent e) {
		if (!compiled) {
			Object[] options = { I18N.getMessage("Overwrite"), I18N.getMessage("Cancel")};
			int n =
				JOptionPane.showOptionDialog(
					this,
					"Framstick was changed. Do you want to update view?",
					"Framstick changed",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[1]);
			if (n == JOptionPane.YES_OPTION) {
				;
			}
			else if (n == JOptionPane.NO_OPTION) {
				return;
			}
			else if (n == JOptionPane.CANCEL_OPTION) {
				return;
			}
			else {
				return;
			}
			changeAction.actionPerformed(null);
		}
	}
	public void internalFrameDeactivated(InternalFrameEvent e) {	
	}
}