package fred;

import idx3d.idx3d_Math;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import jGDK.Framstick;
import jGDK.GenotypeParser;
import jGDK.I18N;
import jGDK.Library;

/**
 * <p>BodyFrame class</p>
 * <p>This is container class for Body panel named <CODE>Tazo</CODE>. This class gives all functionality
 * to edit the framstick body.</p>
 * @author Damian Jarzebski
 * @version 2.0
 */

public class BodyFrame extends JInternalFrame implements MouseListener {
	public Action jTBZoomOut_Action = null;
	public Action jTBZoomIn_Action = null;

	private JToggleButton jTBAddJ = null;

	public Action jTBRedo_Action = null;
	public Action jTBUndo_Action = null;
	public Action jTBDelete_Action = null;
	public Action jTBDefault_Action = null;

	Framstick framstick = null;

	private JToolBar jTB = new JToolBar();
	public NeuroPanel neuroPanel = null;

	JToggleButton jTBRotation = new JToggleButton();
	
	private JToggleButton jTBAddP = null;
	private JToggleButton jTBTranslateP = null; //Marcin new 

	//Marcin
	private JPopupMenu jTBMenuRotation = null;
	private JPopupMenu jTBMenuTranslateP = null;
	private JPopupMenu jTBMenuAddP = null;

	public Tazo2 tazo = null;
	/**
	 * BodyFrame Constructor.
	 * @param framstick reference to the editing framstick.
	 */
	public BodyFrame(Framstick framstick) {
		this.framstick = framstick;
		tazo = new Tazo2(framstick);
		try {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * setting up the ToolBar buttons and connected to them actions.
	 */
	public void setToolBarButtons() {
		jTB.setName(I18N.getMessage("BodyToolbar"));
		//"Zoom In" Button
		Action action;
		jTBZoomIn_Action = new AbstractAction("Zoom In", new ImageIcon(ClassLoader.getSystemResource("images/ZoomIn24.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.zoomIn();
			}
		};
		JButton button = null;
		button = jTB.add(jTBZoomIn_Action);
		button.setText(""); //an icon-only button
		button.setToolTipText(I18N.getMessage("ZoomIn"));
		button.setMargin(new Insets(1, 1, 1, 1));

		//"Zoom Out" Button
		jTBZoomOut_Action = new AbstractAction("Zoom Out", new ImageIcon(ClassLoader.getSystemResource("images/ZoomOut24.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.zoomOut();
			}
		};
		button = jTB.add(jTBZoomOut_Action);
		button.setText(""); //an icon-only button
		button.setToolTipText(I18N.getMessage("ZoomOut"));
		button.setMargin(new Insets(1, 1, 1, 1));

		JToggleButton jTBDetails = new JToggleButton();
		JToggleButton jTBBackground = new JToggleButton();
		JToggleButton jTBBox = new JToggleButton();
		JToggleButton jTBWires = new JToggleButton();

		//"Default" Button
		jTBDefault_Action = new AbstractAction("Default", new ImageIcon(ClassLoader.getSystemResource("images/Default.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.defaultCameraPos();
				tazo.repaint();
			}
		};
		button = jTB.add(jTBDefault_Action);
		button.setText(""); //an icon-only button
		button.setToolTipText(I18N.getMessage("DispDefView"));
		button.setMargin(new Insets(1, 1, 1, 1));

		//"Rotation" Button niestety by Marcin
		action = new AbstractAction("Rotation", new ImageIcon(ClassLoader.getSystemResource("images/Rotate.gif"))) {
			public void actionPerformed(ActionEvent e) {
				if (!jTBRotation.isSelected()) {
					tazo.setAutorotation("S");
				}
				else
					jTBMenuRotation.show(jTBRotation, jTB.getX(), jTB.getY() + jTBRotation.getSize().height);
			}
		};
		jTBRotation.setAction(action);
		jTB.add(jTBRotation);
		jTBRotation.setText(""); //an icon-only button
		jTBRotation.setToolTipText(I18N.getMessage("RotModel"));
		jTBRotation.setMargin(new Insets(1, 1, 1, 1));
		//menu Marcin
		jTBMenuRotation = new JPopupMenu("Rotation");
		//dodanie elementu menu
		action = new AbstractAction(I18N.getMessage("XAxis")) {
			public void actionPerformed(ActionEvent e) {
				tazo.setAutorotation("X");
			}
		};
		JMenuItem menuItem;
		menuItem = jTBMenuRotation.add(action);
		menuItem.setIcon(null);
		//dodanie elementu menu
		action = new AbstractAction(I18N.getMessage("YAxis")) {
			public void actionPerformed(ActionEvent e) {
				tazo.setAutorotation("Y");
			}
		};
		menuItem = jTBMenuRotation.add(action);
		menuItem.setIcon(null);
		//dodanie elementu menu
		action = new AbstractAction(I18N.getMessage("ZAxis")) {
			public void actionPerformed(ActionEvent e) {
				tazo.setAutorotation("Z");
			}
		};
		menuItem = jTBMenuRotation.add(action);
		menuItem.setIcon(null);
		//dodanie elementu menu
		action = new AbstractAction(I18N.getMessage("Random")) {
			public void actionPerformed(ActionEvent e) {
				tazo.setAutorotation("R");
			}
		};
		menuItem = jTBMenuRotation.add(action);
		menuItem.setIcon(null);

		//"Delete" Button
		jTBDelete_Action = new AbstractAction(I18N.getMessage("Delete"), new ImageIcon(ClassLoader.getSystemResource("images/Delete24.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.DeleteElem();
				tazo.repaint();
			}
		};
		button = jTB.add(jTBDelete_Action);
		button.setText(""); //an icon-only button
		button.setToolTipText(I18N.getMessage("DelSelElem"));
		button.setMargin(new Insets(1, 1, 1, 1));

		//"Undo" Button
		jTBUndo_Action = new AbstractAction(I18N.getMessage("Undo"), new ImageIcon(ClassLoader.getSystemResource("images/Undo24.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.Undo();
				tazo.repaint();
			}
		};
		/*		jTBUndo = jTB.add(jTBUndo_Action);
				jTBUndo.setText(""); //an icon-only button
				jTBUndo.setToolTipText("Undo");
				jTBUndo.setMargin(new Insets(1, 1, 1, 1));
		*/
		//"Redo" Button
		jTBRedo_Action = new AbstractAction(I18N.getMessage("Redo"), new ImageIcon(ClassLoader.getSystemResource("images/Redo24.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.Redo();
				tazo.repaint();
			}
		};
		/*		jTBRedo = jTB.add(jTBRedo_Action);
				jTBRedo.setText(""); //an icon-only button
				jTBRedo.setToolTipText("Redo");
				jTBRedo.setMargin(new Insets(1, 1, 1, 1));
		*/
		jTBMenuAddP = new JPopupMenu(I18N.getMessage("AddPart"));
		//"Add part - front" Menu
		action = new AbstractAction(I18N.getMessage("front")+"(XY)") {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = true; //tryb dodawania parta
				tazo.translating = false; //trial
				tazo.addingJoint = false; //wiec nie mozna dodac jointa
				tazo.autorotation = false;
				//wylaczamy rotacje by nie bylo przeksztalcen
				tazo.partAddingFlag = 0; //oznaczamy plaszczyzne flaga
				tazo.prepareScene();
				//scene trzeba odtworzyc dla zachowania wspolrzednych
				tazo.createLights(); //razem ze swiatlami
				tazo.scene.defaultCamera.setPos(0, 0, -tazo.view); //i kamera
				tazo.redrawScene(); //jeszcze przebudowanie nowki
				tazo.description1 = I18N.getMessage("WhereAddP");
				tazo.description2 = ""; //i zapytanie o pozycje nowego parta
				tazo.repaint(); //przerysowanie panelu
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		menuItem = jTBMenuAddP.add(action);
		menuItem.setIcon(null);
		//"Add part - side" Menu
		action = new AbstractAction(I18N.getMessage("side")+"(ZY)") {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = true; //wszystko jak powyzej 
				tazo.translating = false;
				tazo.addingJoint = false;
				tazo.autorotation = false;
				tazo.partAddingFlag = 1; //tylko odpowiednio flaga inna
				tazo.prepareScene();
				tazo.createLights();
				tazo.scene.defaultCamera.setPos(0, 0, -tazo.view);
				tazo.scene.rotate(0, idx3d_Math.pi / 2, 0);
				//i obrot sceny dla zmiany widoku
				tazo.redrawScene();
				tazo.description1 = I18N.getMessage("WhereAddP");
				tazo.description2 = "";
				tazo.repaint();
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		menuItem = jTBMenuAddP.add(action);
		menuItem.setIcon(null);
		//"Add part - plan" Menu
		action = new AbstractAction(I18N.getMessage("plan")+"(XZ)") {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = true; //wszystko j.w.
				tazo.translating = false;
				tazo.addingJoint = false;
				tazo.autorotation = false;
				tazo.partAddingFlag = 2;
				tazo.prepareScene();
				tazo.createLights();
				tazo.scene.defaultCamera.setPos(0, 0, -tazo.view);
				tazo.scene.rotate(idx3d_Math.pi / 2, 0, 0);
				tazo.redrawScene();
				tazo.description1 = I18N.getMessage("WhereAddP");
				tazo.description2 = "";
				tazo.repaint();
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		menuItem = jTBMenuAddP.add(action);
		menuItem.setIcon(null);
		jTB.add(jTBMenuAddP);
		jTB.add(jTBMenuRotation);
		//"Add joint" Button
		action = new AbstractAction("Add Joint", new ImageIcon(ClassLoader.getSystemResource("images/AddJ.gif"))) {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = false;
				tazo.translating = false;
				tazo.AddJoint();
				tazo.repaint();
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		jTBAddJ = new JToggleButton();
		jTBAddJ.setAction(action);
		jTB.add(jTBAddJ);
		jTBAddJ.setText(""); //an icon-only button
		jTBAddJ.setToolTipText(I18N.getMessage("AddJoint"));
		jTBAddJ.setMargin(new Insets(1, 1, 1, 1));
		//"Add part..." Button
		action = new AbstractAction("Add Part...", new ImageIcon(ClassLoader.getSystemResource("images/Add24.gif"))) {
			public void actionPerformed(ActionEvent e) {
				jTBMenuAddP.show(jTBAddP, jTB.getX(), jTB.getY() + jTBAddP.getSize().height);
			}
		};
		jTBAddP = new JToggleButton();
		jTBAddP.setAction(action);
		jTB.add(jTBAddP);
		jTBAddP.setText(""); //an icon-only button
		jTBAddP.setToolTipText(I18N.getMessage("AddPart"));
		jTBAddP.setMargin(new Insets(1, 1, 1, 1));

		//Marcin tutaj szarzowal
		jTBMenuTranslateP = new JPopupMenu(I18N.getMessage("Translate"));

		//"Translate - front" Menu
		action = new AbstractAction(I18N.getMessage("front")+"(XY)") {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = false; //tryb dodawania parta
				tazo.addingJoint = false; //wiec nie mozna dodac jointa
				tazo.autorotation = false;
				//trial
				boolean sth_is_marked = false;
				for(int i=0; i<framstick.getPartCount(); i++)
					if(framstick.elementStrongSelected("p",i)) sth_is_marked = true;
				if(!sth_is_marked) 
				{
					tazo.description1 = I18N.getMessage("NoMarkedP");
					tazo.description2 = "";
					tazo.repaint();
					jTBRotation.setSelected(tazo.autorotation);
					return;
				}
				//trial ends
				//wylaczamy rotacje by nie bylo przeksztalcen
				tazo.translating = true;
				tazo.translatingFlag = 0; //oznaczamy plaszczyzne flaga
				tazo.prepareScene();
				//scene trzeba odtworzyc dla zachowania wspolrzednych
				tazo.createLights(); //razem ze swiatlami
				tazo.scene.defaultCamera.setPos(0, 0, -tazo.view); //i kamera
				tazo.redrawScene(); //jeszcze przebudowanie nowki
				tazo.description1 = I18N.getMessage("DragDirection");
				tazo.description2 = ""; //i zapytanie o pozycje nowego parta
				tazo.repaint(); //przerysowanie panelu
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		menuItem = jTBMenuTranslateP.add(action);
		menuItem.setIcon(null);
		//"Translate - side" Menu
		action = new AbstractAction(I18N.getMessage("side")+"(ZY)") {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = false; //wszystko jak powyzej 
				tazo.addingJoint = false;
				tazo.autorotation = false;
								//trial
				boolean sth_is_marked = false;
				for(int i=0; i<framstick.getPartCount(); i++)
					if(framstick.elementStrongSelected("p",i)) sth_is_marked = true;
				if(!sth_is_marked) 
				{
					tazo.description1 = I18N.getMessage("NoMarkedP");
					tazo.description2 = "";
					tazo.repaint();
					jTBRotation.setSelected(tazo.autorotation);
					return;
				}
				//trial ends
				tazo.translating = true;
				tazo.translatingFlag = 1; //tylko odpowiednio flaga inna
				tazo.prepareScene();
				tazo.createLights();
				tazo.scene.defaultCamera.setPos(0, 0, -tazo.view);
				tazo.scene.rotate(0, idx3d_Math.pi / 2, 0);
				//i obrot sceny dla zmiany widoku
				tazo.redrawScene();
				tazo.description1 = I18N.getMessage("DragDirection");
				tazo.description2 = "";
				tazo.repaint();
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		menuItem = jTBMenuTranslateP.add(action);
		menuItem.setIcon(null);
		//"Translate - plan" Menu
		action = new AbstractAction(I18N.getMessage("plan")+"(XZ)") {
			public void actionPerformed(ActionEvent e) {
				tazo.addingPart = false; //wszystko j.w.
				tazo.addingJoint = false;
				tazo.autorotation = false;
				//trial
				boolean sth_is_marked = false;
				for(int i=0; i<framstick.getPartCount(); i++)
					if(framstick.elementStrongSelected("p",i)) sth_is_marked = true;
				if(!sth_is_marked) 
				{
					tazo.description1 = I18N.getMessage("NoMarkedP");
					tazo.description2 = "";
					tazo.repaint();
					jTBRotation.setSelected(tazo.autorotation);
					return;
				}
				//trial ends
				tazo.translating = true;
				tazo.translatingFlag = 2;
				tazo.prepareScene();
				tazo.createLights();
				tazo.scene.defaultCamera.setPos(0, 0, -tazo.view);
				tazo.scene.rotate(idx3d_Math.pi / 2, 0, 0);
				tazo.redrawScene();
				tazo.description1 = I18N.getMessage("DragDirection");
				tazo.description2 = "";
				tazo.repaint();
				jTBRotation.setSelected(tazo.autorotation);
			}
		};
		menuItem = jTBMenuTranslateP.add(action);
		menuItem.setIcon(null);
		jTB.add(jTBMenuTranslateP);
		//"Translate" Button
		action = new AbstractAction("translate", new ImageIcon(ClassLoader.getSystemResource("images/Translate.gif"))) {
			public void actionPerformed(ActionEvent e) {
				jTBMenuTranslateP.show(jTBTranslateP, jTB.getX(), jTB.getY() + jTBTranslateP.getSize().height);
			}
		};
		jTBTranslateP = new JToggleButton();
		jTBTranslateP.setAction(action);
		jTB.add(jTBTranslateP);
		jTBTranslateP.setText(""); //an icon-only button
		jTBTranslateP.setToolTipText(I18N.getMessage("Translate"));
		jTBTranslateP.setMargin(new Insets(1, 1, 1, 1));
		jTB.addSeparator();
	}
	/**
	 * Initialization metod
	 * @throws Exception
	 */
	private void jbInit() throws Exception {
		this.setIconifiable(false);
		this.setMaximizable(true);
		this.setResizable(true);
		this.setClosable(true);
		this.setTitle(I18N.getMessage("Body"));
		this.setPreferredSize(new Dimension(450, 400));
		this.setSize(new Dimension(450, 400));

		setToolBarButtons();

		//rest of the elements
		this.getContentPane().add(jTB, BorderLayout.NORTH);
		this.getContentPane().add(tazo, BorderLayout.CENTER);
		neuroPanel = new NeuroPanel(framstick);
		neuroPanel.setPreferredSize(new Dimension(50, 200));
		this.getContentPane().add(neuroPanel, BorderLayout.WEST);
		neuroPanel.setVisible(false);
		tazo.addMouseListener(this);
		tazo.setVisible(true);
		this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
	}
	/**
	 * Creates WeakReference to Prop object with the apropriate parameters.
	 * @return WeakReference to Prop class object.
	 * @see WeakReference
	 */
	public WeakReference showProp() {
		if (tazo.markedObject == null)
			return null;
		int numer = -1;
		if (tazo.markedObject.startsWith("Part")) {
			numer = Integer.parseInt(tazo.markedObject.substring(4, tazo.markedObject.length() - 4));
		}
		else if (tazo.markedObject.startsWith("Joint")) {
			numer = Integer.parseInt(tazo.markedObject.substring(5, tazo.markedObject.length() - 2));
		}
		else if (tazo.markedObject.startsWith("Neuron")) {
			numer = Integer.parseInt(tazo.markedObject.substring(6, tazo.markedObject.length() - 2));
		}
		else
			return null;
		Prop p = new Prop(framstick, tazo.markedObject.substring(0, 1).toLowerCase(), numer);
		p.setLocation(this.getLocation().x + 100, this.getLocation().y + 50);
		p.pack();
		return new WeakReference(p);
	}
	/**
	 * doing nothing, but needed becouse of interface implemented.
	 */
	public void mouseClicked(MouseEvent e) {}
	/**
	 * doing nothing, but needed becouse of interface implemented.
	 */
	public void mousePressed(MouseEvent e) {}
	/**
	 * Deselecting pressed button on the toolbar.
	 */
	public void mouseReleased(MouseEvent e) {
		jTBAddP.setSelected(false);
		jTBAddJ.setSelected(false);
		jTBTranslateP.setSelected(false);
	}
	/**
	 * doing nothing, but needed becouse of interface implemented.
	 */
	public void mouseEntered(MouseEvent e) {}
	/**
	 * doing nothing, but needed becouse of interface implemented.
	 */
	public void mouseExited(MouseEvent e) {}
	/**
	 * Main function - running the BodyFrame without MainFrame
	 * @param args
	 * @throws PropertyVetoException
	 */
	public static void main(String[] args) throws PropertyVetoException {
		I18N in = new I18N("","");
		Library library = null;
		//jGDK-----------
		try {
			File f = new File("model.def");
			if (!f.exists()) {
				System.out.println("Main Frame" + "The config file don't exist: ");
				return;
			}
			if (!f.canRead()) {
				System.out.println("Main Frame: " + "There is no access to the config file: ");
				return;
			}
			InputStream input = new FileInputStream(f);
			int n = input.available();
			byte buf[] = new byte[n];
			if ((n = input.read(buf)) >= 0) {
				System.out.println("Main Frame" + "The config file has been read");
			}
			input.close();
			library = new Library(new String(buf));
		}
		catch (IOException ex) {}
		Framstick.library = library;
		GenotypeParser.library = library;
		//jGDK-----------END
		Framstick fr = new Framstick();
		fr.rebuildModel();
		BodyFrame bodyFrame1 = new BodyFrame(fr);
		JFrame f = new JFrame("Fred - Main Frame");
		f.setSize(bodyFrame1.getSize().width + 10, bodyFrame1.getSize().height + 30);
		f.getContentPane().setLayout(null);
		f.setLocation(50, 20);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		f.getContentPane().add(bodyFrame1);
		System.gc();
		bodyFrame1.setVisible(true);
		bodyFrame1.setMaximum(true);
	}
	//////////////////////////////////////////////////////
	//////////////////////////////////////////////////////
	protected void finalize() throws Throwable {
		System.out.println("Finalizing body window");
		this.getContentPane().removeAll();
		jTBZoomOut_Action = null;
		jTBZoomIn_Action = null;

		jTBAddJ = null;

		jTBRedo_Action = null;
		jTBUndo_Action = null;
		jTBDelete_Action = null;
		jTBDefault_Action = null;

		framstick = null;

		jTB = null;
		neuroPanel = null;

		jTBRotation = null;
	
		jTBAddP = null;
		jTBTranslateP = null; //Marcin new 

		//Marcin
		jTBMenuRotation = null;
		jTBMenuTranslateP = null;
		jTBMenuAddP = null;
		
		tazo.removeMouseListener(this);
		tazo = null;
		super.finalize();
	}
	boolean wiresSet = false;
	/**
	 * Select/deselects wires wiev.
	 */
	public void setWires() {
		tazo.setWires();
		wiresSet=!wiresSet;
	}
	boolean boxSet = true;
	/**
	 * Select/deselects the box visible.
	 */
	public void setBox() {
		tazo.boxVisible();
		boxSet=!boxSet;
	}
	boolean backSet = false;
	/**
	 * Select/deselects the background(clouds) visible
	 */
	public void setBackground() {
		tazo.setBackground();
		backSet=!backSet;
	}
	boolean detSet = false;
	/**
	 * Select/deselects the details visible (green parts etc.).
	 */
	public void setDetails() {
		tazo.setDetails();
		detSet=!detSet;
	}
}