package fred;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyVetoException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.ref.WeakReference;
import java.net.URL;

import javax.swing.*;

import jGDK.ErrManager;
import jGDK.Framstick;
import jGDK.I18N;
import jGDK.Library;
import jGDK.Record;

import favi.Symbol;

/**
 * <p>BrainFrame class</p>
 * <p>This is container class for Symbol, Names and Neuro panels. This class gives all functionality
 * to edit the framstick brain - with the help of <CODE>Prop</CODE> class. This class uses the instance
 *  of MyLayout to set size of the panels during resize.</P>
 * @author Damian Jarzebski
 * @version 2.0
 */
public class BrainFrame extends JInternalFrame implements ActionListener
{
  JLayeredPane layeredPane = new JLayeredPane();
  private boolean[] networkSet =
  {false, false, false  };
  private Framstick framstick = null;
  public Action jAnimate_Action;
  private Action jAddN_Action;
  private Action jAddC_Action;
  /**
   * Action of deleting neuron
   */
  public Action jDelN_Action;
  public Action jUndo_Action;
  public Action jRedo_Action;
  public Action jZoomIn_Action;
  public Action jZoomOut_Action;
  public Action jDefault_Action;
  
  public int view = 0;
  
  public Neuro neuro = null;
  public Symbol symbol = null;
  public Names names = null;
  
  JToolBar jToolBar1 = new JToolBar();
  JButton zoomin = null;
  JButton zoomout = null;
  JButton deflt = null;
  JToggleButton network = new JToggleButton();
  JToggleButton animate = new JToggleButton();
  JButton AddN = null;
  JButton AddC = null;
  JButton DelN = null;
  JButton Undo = null;
  JButton Redo = null;
  String[] Opcje =
  { I18N.getMessage("Animated"), I18N.getMessage("Symbols"), I18N.getMessage("Names")};
  JComboBox jComboBox1 = new JComboBox(Opcje);
  
  
  //Marcin
  private JPopupMenu jTBMenuAddN = null;
  
  /**
   * BrainFrame Constructor - loads all 3 panels.
   * @param framstick
   */
  public BrainFrame(Framstick framstick)
  {
    this.framstick = framstick;
    //    System.out.println("Loading animated panel...");
    neuro = new Neuro(framstick);
    //    System.out.println("Loading symbols panel...");
    symbol = new Symbol(framstick);
    //    System.out.println("Loading names panel...");
    names = new Names(framstick);
    //    System.out.println("Initing...");
    jbInit();
  }
  //================================ MAIN ======================================
  /**
   * Function Main - running the BrainFrame without the MainFrame
   * @param args
   * @throws PropertyVetoException
   */
  public static void main(String[] args) throws PropertyVetoException
  {
    I18N in = new I18N("", "");
    ErrManager err = new ErrManager();
    Library library = null;
    //jGDK-----------
    URL u = ClassLoader.getSystemResource("model.def");
    char buf[] = null;
    try
    {
      InputStream input = u.openStream();
      int n = input.available();
      BufferedReader br = new BufferedReader(new InputStreamReader(input));
      buf = new char[n];
      System.out.println("do odczytania:"+n);
      int r=0;
      if ((r = br.read(buf,0,n)) >= 0)
      {
        System.out.println("odczytano:"+r);
        System.out.println("The config file has been read");
      }
      br.close();
      input.close();
    }
    catch (IOException e1)
    {
      System.out.println("IOError:"+e1.getLocalizedMessage());
    }
    String s = new String(buf);
    //System.out.println("File lenght="+buf.length+"\nstr len="+s.length()+"string:\n"+s+"buf:\n"+buf.toString());
    library = new Library(s);
    Framstick.library = library;
    //jGDK-----------END
    Framstick fr = new Framstick();
    fr.genotype = "//0\np:\np:x=2\nn:p=0\nn:p=1\n";
    //fr.rebuildModel();
    BrainFrame brainFrame1 = new BrainFrame(fr);
    JFrame f = new JFrame("Fred - Main Frame");
    WindowListener l = new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        System.exit(0);
      }
    };
    f.addWindowListener(l);
    f.setSize(brainFrame1.getSize().width + 10, brainFrame1.getSize().height + 30);
    f.getContentPane().setLayout(null);
    f.setLocation(50, 50);
    f.setVisible(true);
    f.getContentPane().add(brainFrame1);
    brainFrame1.setVisible(true);
    brainFrame1.setMaximum(true);
  }
  //	======================== end of MAIN ==============================================
  /**
   * Function setting the ToolBar buttons and actions.
   */
  public void setToolBarButtons()
  {
    jToolBar1.setName(I18N.getMessage("BrainToolbar"));
    jComboBox1.addActionListener(this);
    jComboBox1.setActionCommand("combo");
    jZoomIn_Action = new AbstractAction("Zoom In", new ImageIcon(ClassLoader.getSystemResource("images/ZoomIn24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (view == 0)
          neuro.zoomIn();
        else if (view == 1)
          symbol.zoomIn();
        else
          names.zoomIn();
        layeredPane.repaint();
      }
    };
    zoomin = jToolBar1.add(jZoomIn_Action);
    zoomin.setToolTipText(I18N.getMessage("ZoomIn"));
    zoomin.setMargin(new Insets(1, 1, 1, 1));
    zoomin.setText("");
    
    jZoomOut_Action = new AbstractAction("Zoom Out", new ImageIcon(ClassLoader.getSystemResource("images/ZoomOut24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (view == 0)
          neuro.zoomOut();
        else if (view == 1)
          symbol.zoomOut();
        else
          names.zoomOut();
        layeredPane.repaint();
      }
    };
    zoomout = jToolBar1.add(jZoomOut_Action);
    zoomout.setToolTipText(I18N.getMessage("ZoomOut"));
    zoomout.setMargin(new Insets(1, 1, 1, 1));
    zoomout.setText("");
    
    jDefault_Action = new AbstractAction("Default", new ImageIcon(ClassLoader.getSystemResource("images/Default.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (view == 0)
        {
          neuro.defaultCameraPos();
          neuro.repaint();
        }
        else if (view == 1)
        {
          symbol.setZoomValue();
          symbol.repaint();
        }
        else
        {
          names.setZoomValue();
          names.repaint();
        };
      }
    };
    deflt = jToolBar1.add(jDefault_Action);
    deflt.setToolTipText(I18N.getMessage("DefaultView"));
    deflt.setMargin(new Insets(1, 1, 1, 1));
    deflt.setText("");
    
    network.setToolTipText(I18N.getMessage("ChangeNAligm"));
    network.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Network.gif")));
    network.setMargin(new Insets(1, 1, 1, 1));
    network.setText("");
    network.setActionCommand("network");
    network.addActionListener(this);
    
    //"Animate" Button
    jAnimate_Action = new AbstractAction("Animate", new ImageIcon(ClassLoader.getSystemResource("images/Rotate.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        neuro.setAnimation();
        neuro.repaint();
      }
    };
    animate.setAction(jAnimate_Action);
    jToolBar1.add(animate);
    animate.setText(""); //an icon-only button
    animate.setToolTipText(I18N.getMessage("AnimNeurons"));
    animate.setMargin(new Insets(1, 1, 1, 1));
    
    jAddN_Action = new AbstractAction("Add Neuron", new ImageIcon(ClassLoader.getSystemResource("images/Add24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        jTBMenuAddN.show(AddN, jToolBar1.getX(), jToolBar1.getY() + AddN.getSize().height);
      }
    };
    AddN = jToolBar1.add(jAddN_Action);
    AddN.setText(""); //an icon-only button
    AddN.setToolTipText(I18N.getMessage("AddNeuron"));
    AddN.setMargin(new Insets(1, 1, 1, 1));
    
    jTBMenuAddN = new JPopupMenu(I18N.getMessage("AddNeuron"));
    
    //not optimized, but works for all known neuron types:
    Record[] neurons=Framstick.library.getAllRecords("neuro");
    for(int i=0;i<neurons.length;i++)
    {
      final String key = (String) neurons[i].info.get("id");
      String name = (String) neurons[i].info.get("name");
      String neu=key+(name!=null ? " - "+name : "");
      Action a=new AbstractAction(I18N.getMessage("Add")+" "+neu)
      {
        public void actionPerformed(ActionEvent e)
        {
          if (jComboBox1.getSelectedIndex()==1) symbol.symbolAddNeuron(key); else names.symbolAddNeuron(key);
          layeredPane.repaint();
        }
      };
      jTBMenuAddN.add(a);
    }
    
    //*** code below - unbelievable: this fragment was repeated 10 times for each
    //    specific neuron type, with dedicated variables (all removed). Who wrote that code!????
    /*jTBMenuAddNtypeN_Action = new AbstractAction(I18N.getMessage("Add")+" N")
    {
      public void actionPerformed(ActionEvent e)
      {
        if (jComboBox1.getSelectedIndex() == 1)
          symbol.symbolAddNeuron("N");
        else
          names.symbolAddNeuron("N");
        layeredPane.repaint();
      }
    };
    jTBMenuAddNtypeN = jTBMenuAddN.add(jTBMenuAddNtypeN_Action);
    jTBMenuAddNtypeN.setIcon(null);*/
    //etc.
    
    jToolBar1.add(jTBMenuAddN);
    
    //"Del Neuron" Button
    jDelN_Action = new AbstractAction(I18N.getMessage("DelNeuron"), new ImageIcon(ClassLoader.getSystemResource("images/Delete24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (jComboBox1.getSelectedIndex() == 1)
        {
          symbol.symbolDelNeuron();
          symbol.repaint();
        }
        else
        {
          names.symbolDelNeuron();
          names.repaint();
        }
      }
    };
    DelN = jToolBar1.add(jDelN_Action);
    DelN.setText(""); //an icon-only button
    DelN.setToolTipText(I18N.getMessage("DelNeuron"));
    DelN.setMargin(new Insets(1, 1, 1, 1));
    //"Add Connect" Button
    jAddC_Action = new AbstractAction("Add Connection", new ImageIcon(ClassLoader.getSystemResource("images/AddJ.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (jComboBox1.getSelectedIndex() == 1)
        {
          symbol.symbolAddConnect();
          symbol.repaint();
        }
        else
        {
          names.symbolAddConnect();
          names.repaint();
        }
      }
    };
    AddC = jToolBar1.add(jAddC_Action);
    AddC.setText(""); //an icon-only button
    AddC.setToolTipText(I18N.getMessage("AddConn"));
    AddC.setMargin(new Insets(1, 1, 1, 1));
    //"Undo" Button
    jUndo_Action = new AbstractAction(I18N.getMessage("Undo"), new ImageIcon(ClassLoader.getSystemResource("images/Undo24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (jComboBox1.getSelectedIndex() == 1)
        {
          symbol.symbolUndo();
          symbol.repaint();
        }
        else if (jComboBox1.getSelectedIndex() == 2)
        {
          names.symbolUndo();
          names.repaint();
        }
      }
    };
                /*		Undo = jToolBar1.add(jUndo_Action);
                                Undo.setText(""); //an icon-only button
                                Undo.setToolTipText("Undo");
                                Undo.setMargin(new Insets(1, 1, 1, 1));
                 */
    //"Redo" Button
    jRedo_Action = new AbstractAction(I18N.getMessage("Redo"), new ImageIcon(ClassLoader.getSystemResource("images/Redo24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (jComboBox1.getSelectedIndex() == 1)
        {
          symbol.symbolRedo();
          symbol.repaint();
        }
        else if (jComboBox1.getSelectedIndex() == 2)
        {
          names.symbolRedo();
          names.repaint();
        }
      }
    };
                /*		Redo = jToolBar1.add(jRedo_Action);
                                Redo.setText(""); //an icon-only button
                                Redo.setToolTipText("Redo");
                                Redo.setMargin(new Insets(1, 1, 1, 1));
                 */
 //       jComboBox1.setMaximumSize(new Dimension(79, 21));
    jComboBox1.setToolTipText(I18N.getMessage("SelectView"));
    //jToolBar1.add(zoomin, null);
    //jToolBar1.add(zoomout, null);
    //jToolBar1.add(deflt, null);
    jToolBar1.add(network, null);
    jToolBar1.add(jComboBox1, null);
  }
  /**
   * Initialization function
   * @throws Exception
   */
  private void jbInit()
  {
    view = 0;
    this.setIconifiable(false);
    this.setMaximizable(true);
    this.setResizable(true);
    this.setClosable(true);
    this.setTitle(I18N.getMessage("Brain"));
    this.setSize(new Dimension(440, 400));
    
    setToolBarButtons();
    
    this.getContentPane().add(jToolBar1, BorderLayout.NORTH);
    
    jAnimate_Action.setEnabled(true);
    jAddC_Action.setEnabled(false);
    jAddN_Action.setEnabled(false);
    jUndo_Action.setEnabled(false);
    jRedo_Action.setEnabled(false);
    jDelN_Action.setEnabled(false);
    
    layeredPane.setLayout(new MyLayout());
    layeredPane.add(symbol, "first");
    layeredPane.add(neuro, "second");
    layeredPane.add(names, "last");
    getContentPane().add(layeredPane, BorderLayout.CENTER);
    layeredPane.moveToFront(neuro);
    this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
  }
  ///////// end of INIT
  /**
   * Doing all action connected with all menus and buttons.
   */
  void jComboBox1_actionPerformed()
  {
    view = jComboBox1.getSelectedIndex();
    if (view == 0)
    {
      network.setSelected(networkSet[0]);
      if(jAnimate_Action!=null)jAnimate_Action.setEnabled(true);
      if(jAddC_Action!=null)jAddC_Action.setEnabled(false);
      if(jAddN_Action!=null)jAddN_Action.setEnabled(false);
      if(jUndo_Action!=null)jUndo_Action.setEnabled(false);
      if(jRedo_Action!=null)jRedo_Action.setEnabled(false);
      if(jDelN_Action!=null)jDelN_Action.setEnabled(false);
      layeredPane.moveToFront(neuro);
      neuro.defaultCameraPos();
      neuro.redrawScene();
      neuro.repaint();
    }
    else if (view == 1)
    {
      network.setSelected(networkSet[1]);
      if(jAnimate_Action!=null)jAnimate_Action.setEnabled(false);
      if(jAddC_Action!=null)jAddC_Action.setEnabled(true);
      if(jAddN_Action!=null)jAddN_Action.setEnabled(true);
      if(jUndo_Action!=null)jUndo_Action.setEnabled(true);
      if(jRedo_Action!=null)jRedo_Action.setEnabled(true);
      if(jDelN_Action!=null)jDelN_Action.setEnabled(true);
      layeredPane.moveToFront(symbol);
      symbol.setZoomValue();
      symbol.repaint();
    }
    else
    {
      network.setSelected(networkSet[2]);
      if(jAnimate_Action!=null)jAnimate_Action.setEnabled(false);
      if(jAddC_Action!=null)jAddC_Action.setEnabled(true);
      if(jAddN_Action!=null)jAddN_Action.setEnabled(true);
      if(jUndo_Action!=null)jUndo_Action.setEnabled(true);
      if(jRedo_Action!=null)jRedo_Action.setEnabled(true);
      if(jDelN_Action!=null)jDelN_Action.setEnabled(true);
      layeredPane.moveToFront(names);
      names.setZoomValue();
      names.repaint();
    }
    layeredPane.revalidate();
    layeredPane.repaint();
  }
  /**
   * Creates WeakReference to Prop object with the apropriate parameters.
   * @return WeakReference to Prop class object.
   * @see WeakReference
   */
  public WeakReference showProp()
  {
    if (view == 0)
    {
      if (neuro.markedNeuron == null)
        return null;
      int numer = -1;
      if (neuro.markedNeuron.startsWith("Neuron"))
      {
        numer = Integer.parseInt(neuro.markedNeuron.substring(6, neuro.markedNeuron.length() - 2));
      }
      else
        return null;
      Prop p = new Prop( framstick, "n",
      //neuro.markedNeuron.substring(0,1).toLowerCase(),
      numer);
      p.setLocation(this.getLocation().x + 100, this.getLocation().y + 50);
      p.pack();
      return new WeakReference(p);
    }
    else
    {
      System.out.println("Symbol - prop");
      if (view == 1)
      {
        if (symbol.markedNeuron == -1)
          return null;
        Prop p = new Prop(framstick, "n", symbol.markedNeuron);
        p.setLocation(this.getLocation().x + 100, this.getLocation().y + 50);
        //trial
        p.pack();
        return new WeakReference(p);
      }
      else
      {
        if (names.markedNeuron == -1)
          return null;
        Prop p = new Prop(framstick, "n", names.markedNeuron);
        p.setLocation(this.getLocation().x + 100, this.getLocation().y + 50);
        //trial
        p.pack();
        return new WeakReference(p);
      }
      
    }
  }
  //////////////////////////////
  void network_actionPerformed()
  {
    if (view == 0)
    {
      neuro.changePlacingMethod();//trial ???
      networkSet[0] = !networkSet[0];
    }
    else if (view == 1)
    {
      symbol.changePlacingMethod();
      networkSet[1] = !networkSet[1];
    }
    else
    {
      names.changePlacingMethod();
      networkSet[2] = !networkSet[2];
    }
    layeredPane.repaint();
  }
  /**
   * Sets visible one 3 panels.
   * @param view
   */
  public void setView(int view)
  {
    if ((view>=0) || (view<=2))
      jComboBox1.setSelectedIndex(view);
  }
  /**
   * adding MouseListener to all panels and the window.
   */
  public void addMouseListener(MouseListener ml)
  {
    if(ml!=null)
    {
      if(neuro!=null)	neuro.addMouseListener(ml);
      if(names!=null) names.addMouseListener(ml);
      if(symbol!=null) symbol.addMouseListener(ml);
      super.addMouseListener(ml);
    }
  }
  /**
   * Returning the parameters for the actual selected alement on panles.
   * @param i x coordinate of the element
   * @param j y coordinate of the element
   * @return 2 Strings: kind of element and his number
   */
  public String[] showProperties(int i, int j)
  {
    if (view == 0)
      return neuro.showProperties(i,j);
    else if (view == 1)
    {
      //trial
      return symbol.showProperties(i,j);
    }
    else //if(view == "Names")
    {
      //trial
      return names.showProperties(i,j);
    }
    //else return null;
  }
  /**
   * Doing all action connected with all menus and buttons.
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   */
  public void actionPerformed(ActionEvent e)
  {
    if(e.getActionCommand().equalsIgnoreCase("network"))
    {
      network_actionPerformed();
    }
    else if(e.getActionCommand().equalsIgnoreCase("combo"))
    {
      jComboBox1_actionPerformed();
    }
  }
  protected void finalize() throws Throwable
  {
    System.out.println("Finalizing brain window");
    this.getContentPane().removeAll();
    jComboBox1.removeActionListener(this);
    
    layeredPane = null;
    networkSet = null;
    framstick = null;
    jAnimate_Action = null;
    jAddN_Action = null;
    jAddC_Action = null;
    jDelN_Action = null;
    jUndo_Action = null;
    jRedo_Action = null;
    jZoomIn_Action = null;
    jZoomOut_Action = null;
    jDefault_Action = null;
    
    
    neuro = null;
    symbol = null;
    names = null;
    
    jToolBar1 = null;
    zoomin = null;
    zoomout = null;
    deflt = null;
    network.removeActionListener(this);
    network = null;
    animate = null;
    AddN = null;
    AddC = null;
    DelN = null;
    Undo = null;
    Redo = null;
    Opcje = null;
    jComboBox1 = null;
    
    jTBMenuAddN = null;
    super.finalize();
  }
}