package fred;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import jGDK.ErrManager;
import jGDK.Framstick;
import jGDK.GenFileParser;
import jGDK.GenotypeParser;
import jGDK.I18N;
import jGDK.Library;

import favi.Symbol;

/**
 * <p>Browser class</p>
 * <p>Dialog enabiling user to choose the framstick to edit from genotype pool stored in file.
 * This class using instance of Tazo and Symbol class to visualize framstick. Implements Runnable
 * interface to enable self closing if there is only one framstick genotype in file.</p>
 * @author Damian Jarzebski
 * @version 2.0
 */

public class Browser extends JDialog implements ListSelectionListener, ActionListener, Runnable
{
  String file = null;
  Framstick framstick = null;
  Framstick fram = null;
  
  JSplitPane jSplitPane1 = null;
  JSplitPane jSplitPane2 = null;
  JSplitPane jSplitPane3 = null;
  JList jList1 = null;
  JTextPane tp;
  
  Tazo2 LeftPanel = null;
  Symbol RightPanel = null;
  
  JPanel jBottomPanel = null;
  JButton jButtonCacel = null;
  JButton jButtonOk = null;
  
  boolean accepted = false;
  
  Vector wynik = null;
  Vector names = null;
  /**
   * Browser class constructor. The file parameter is the String contains *.gen file. f is the instance
   * of Framstick class that will be edit.
   * @param file
   * @param f
   */
  public Browser(String file, Framstick f)
  {
    this.file = file;
    this.framstick = f;
    this.setModal(true);
    this.setTitle(I18N.getMessage("GBrowser"));
    jbInit();
  }
  private void jbInit()
  {
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    LoadWindow lw = new LoadWindow(I18N.getMessage("Loading"),I18N.getMessage("ProcesFile"));
    lw.start();
    System.out.println(I18N.getMessage("ParseFile"));
    //    long time = System.currentTimeMillis();
    GenFileParser parser = new GenFileParser(file);
    //    System.out.println("(" + (System.currentTimeMillis() - time) + "ms)");
    if (parser.numberOfFramsticks < 1)
    {
      JOptionPane.showConfirmDialog(
      this,
      I18N.getMessage("NothingToLoad"),
      I18N.getMessage("Error"),
      JOptionPane.OK_OPTION);
      return;
    }
/*    else if (parser.numberOfFramsticks == 1) //although single, might be in an unsupported genotype format
    {
      wynik = parser.framsticksSet;
      fram = new Framstick();
      fram.author = ((Framstick)wynik.get(0)).author;
      fram.genotype = ((Framstick)wynik.get(0)).genotype;
      fram.info = ((Framstick)wynik.get(0)).info;
      fram.name = ((Framstick)wynik.get(0)).name;
      this.getContentPane().setLayout(new GridLayout(2, 1));
      jButtonOk = new JButton();
      jButtonOk.setOpaque(true);
      jButtonOk.setText("OK");
      jButtonOk.addActionListener(this);
      JLabel l = new JLabel(I18N.getMessage("OneLoaded", "One genotype loaded"));
      this.getContentPane().add(l);
      this.getContentPane().add(jButtonOk);
      this.setSize(200, 70);
      d = Toolkit.getDefaultToolkit().getScreenSize();
      d.width = (d.width - this.getSize().width) / 2;
      d.height = (d.height - this.getSize().height) / 2;
      this.setLocation(d.width, d.height);
      lw.destroy();
      this.start();
    }*/
    else //if (parser.numberOfFramsticks > 1)
    {
      wynik = parser.framsticksSet;
      names = new Vector();
      for (int i = 0; i < parser.numberOfFramsticks; i++)
      {
        names.add(((Framstick)wynik.get(i)).name);
      }
      
      fram = new Framstick();
      jList1 = new JList();
      jBottomPanel = new JPanel();
      jButtonCacel = new JButton();
      jButtonOk = new JButton();
      
      LeftPanel = new Tazo2(fram);
      RightPanel = new Symbol(fram);
      
      jList1.setMinimumSize(new Dimension(40, 60));
      jList1.setPreferredSize(new Dimension(d.width/3, d.height/3));
      jList1.setToolTipText(I18N.getMessage("PickToEdit"));
      jList1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      jList1.addListSelectionListener(this);
      JScrollPane scrollPane = new JScrollPane(jList1);
      
      tp=new JTextPane();
      tp.setEditable(false);
      tp.setBackground(getContentPane().getBackground());
      
      this.getContentPane().setLayout(new BorderLayout());
      jButtonCacel.setText(I18N.getMessage("Cancel"));
      jButtonCacel.addActionListener(this);
      jButtonOk.setOpaque(true);
      jButtonOk.setText("OK");
      jButtonOk.addActionListener(this);
      jBottomPanel.setBorder(BorderFactory.createLoweredBevelBorder());

      LeftPanel.setMinimumSize(new Dimension(50, 60));
      LeftPanel.setPreferredSize(new Dimension(d.width/3, d.height/3));
      RightPanel.setMinimumSize(new Dimension(50, 60));
      RightPanel.setPreferredSize(new Dimension(d.width/3, d.height/3));
      tp.setPreferredSize(new Dimension(d.width/3, d.height/3));
      
      jSplitPane1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,true,scrollPane,tp);
      jSplitPane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,true,LeftPanel,RightPanel);
      jSplitPane3 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true,jSplitPane1,jSplitPane2);

      jSplitPane2.setDividerSize(10);

      jSplitPane1.resetToPreferredSizes();
      jSplitPane2.resetToPreferredSizes();
      jSplitPane3.resetToPreferredSizes();
      
      this.getContentPane().add(jSplitPane3, BorderLayout.CENTER);

      jBottomPanel.add(jButtonOk, null);
      jBottomPanel.add(jButtonCacel, null);
      this.getContentPane().add(jBottomPanel, BorderLayout.SOUTH);
      
      lw.destroy();
      jList1.setListData(names);
      jList1.setSelectedIndex(0);
    }
    this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    pack();
    d = Toolkit.getDefaultToolkit().getScreenSize();
    d.width = (d.width - this.getPreferredSize().width) / 2;
    d.height = (d.height - this.getPreferredSize().height) / 2;
    this.setLocation(d.width, d.height);
  }

  private void onExit()
  {
    this.removeAll();
    file = null;
    framstick = null;
    fram = null;
    jSplitPane1 = null;
    if(jList1!=null)jList1.removeListSelectionListener(this);
    jList1 = null;
    jSplitPane2 = null;
    LeftPanel = null;
    RightPanel = null;
    jBottomPanel = null;
    jButtonCacel = null;
    jButtonOk = null;
    wynik = null;
    names = null;
    Runtime.getRuntime().gc();
  }
  /**
   * Main function
   * @param args
   */
  public static void main(String[] args)
  {
    String s = "";
    String dir = "";
    String filename = "";
    //jGDK - variables
    
    JFrame b = new JFrame("Main Frame");
    ErrManager err = new ErrManager();
    Library library = null;
    //jGDK-----------
    try
    {
      File f = new File("model.def");
      if (!f.exists())
      {
        System.out.println("Main Frame" + "The config file don't exist: " + filename);
        return;
      }
      if (!f.canRead())
      {
        System.out.println("Main Frame: " + "There is no access to the config file: " + filename);
        return;
      }
      InputStream input = new FileInputStream(f);
      int n = input.available();
      byte buf[] = new byte[n];
      if ((n = input.read(buf)) >= 0)
      {
        System.out.println("Main Frame" + "The config file has been read");
      }
      input.close();
      library = new Library(new String(buf));
    }
    catch (IOException ex)
    {}
    Framstick.library = library;
    GenotypeParser.library = library;
    //jGDK-----------END
    b.setSize(100, 100);
    b.setLocation(200, 50);
    b.setVisible(true);
    b.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    JFileChooser c = new JFileChooser();
    int rVal = c.showOpenDialog(b);
    if (rVal == JFileChooser.APPROVE_OPTION)
    {
      filename = c.getSelectedFile().getName();
      dir = c.getCurrentDirectory().toString();
      try
      {
        File f = new File(dir + "/" + filename);
        if (!f.exists())
        {
          
          return;
        }
        if (!f.canRead())
        {
          
          return;
        }
        InputStream input = new FileInputStream(f);
        int n = input.available();
        byte buf[] = new byte[n];
        while ((n = input.read(buf)) >= 0)
        {}
        input.close();
        s = new String(buf);
      }
      catch (IOException ex)
      {
        System.out.println("Error:" + ex);
      }
      b.setTitle("[" + dir + "/" + filename + "]");
      Browser browser = new Browser(s, new Framstick());
      browser.setSize(600, 500);
      Point loc = b.getLocation();
      browser.setLocation(200,50);
      //			browser.setModal(true);
      browser.pack();
      browser.show();
      b.setTitle(b.getTitle() + "[OK]");
      browser = null;
//      System.runFinalization();
//      System.gc();
    }
    if (rVal == JFileChooser.CANCEL_OPTION)
    {
      filename = "";
      
      b.setTitle("B");
      dir = "";
    }
  }
  /**
   * Refreshing the panels view of the framstick selected in list.
   * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
   */
  public void valueChanged(ListSelectionEvent e)
  {
    if ((!e.getValueIsAdjusting()) && (jList1.getMinSelectionIndex() >= 0))
    {
      int index = jList1.getSelectedIndex();
      String s = new String(this.getTitle());
      this.setTitle(s + " - " + I18N.getMessage("PleaseWait"));
      fram.author = ((Framstick)wynik.get(index)).author;
      fram.genotype = ((Framstick)wynik.get(index)).genotype;
      fram.info = ((Framstick)wynik.get(index)).info;
      fram.name = ((Framstick)wynik.get(index)).name;
      boolean ok=fram.rebuildModel();
      if (!ok)
	{// dzieki temu nie zostaje stare body&brain.
	 // nie ma fram.clear() a fram=new... byloby za bardzo ryzykowne
	 // na szczescie akceptuje pusty genotyp (co niestety nie zgadza sie
	 // z zachowaniem framsow)
	fram.genotype="//0\n";
	fram.rebuildModel();
	}
      jButtonOk.setEnabled(ok);
      String t="";
      if (ErrManager.msg.size()>0) t=ErrManager.msg+"\n\n";
      tp.setText(t+fram.info);
      ErrManager.msg.clear();
      fram.fireSignificantEditEvent();
      LeftPanel.defaultCameraPos();
      this.setTitle(s);
    }
  }
  /**
   * Reaction to "OK" or "Cancel" button pressed.
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   */
  public void actionPerformed(ActionEvent e)
  {
    if (e.getActionCommand().compareToIgnoreCase("OK") == 0)
    {
      framstick.author = fram.author;
      framstick.genotype = fram.genotype;
      framstick.info = fram.info;
      framstick.name = fram.name;
      framstick.rebuildModel();
      framstick.fireSignificantEditEvent();
      framstick.fireMarkChangedEvent();
      onExit();
      accepted = true;
      this.dispose();
    }
    else //cancel
    {
      fram = null;
      jSplitPane1 = null;
      jSplitPane2 = null;
      LeftPanel = null;
      RightPanel = null;
      onExit();
      accepted = false;
      this.dispose();
    }
  }
  private Thread clockThread = null;
  /**
   * Starts the thread to close the browser window.
   */
  public void start()
  {
    if (clockThread == null)
    {
      clockThread = new Thread(this, "Clock");
      clockThread.start();
    }
  }
  /**
   * Closing the browser for user.
   */
  public void run()
  {
    Thread myThread = Thread.currentThread();
    if (clockThread == myThread)
    {
      this.jButtonOk.doClick();
    }
  }
  protected void finalize() throws Throwable
  {
    System.out.println("Finalizing browser");
    this.getContentPane().removeAll();
    clockThread = null;
    file = null;
    framstick = null;
    fram = null;
    jSplitPane1 = null;
    jList1.removeListSelectionListener(this);
    jList1 = null;
    jSplitPane2 = null;
    LeftPanel = null;
    RightPanel = null;
    jBottomPanel = null;
    jButtonOk.removeActionListener(this);
    jButtonCacel.removeActionListener(this);
    jButtonCacel = null;
    jButtonOk = null;
    wynik = null;
    names = null;
    super.finalize();
  }
}