package fred;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.undo.*;
import javax.swing.tree.*;
import javax.swing.border.*;

import jGDK.Framstick;
import jGDK.ErrManager;
import jGDK.I18N;
import jGDK.Message;
import jGDK.FramstickListener;
import jGDK.FramstickEvent;
import jGDK.GenotypeParser;

import favi.F0Document;

public class F0Editor extends JInternalFrame implements FramstickListener, InternalFrameListener
{
  DefaultTreeModel treeModel;
  DefaultMutableTreeNode mainNode;
  DefaultMutableTreeNode warnings;
  DefaultMutableTreeNode errors;
  DefaultMutableTreeNode hints;
  
  DefaultStyledDocument linijkaDoc;
  
  SimpleAttributeSet attrs;
  JTextPane textPane;
  private Hashtable actions;
  
  private F0Document doc;
  private Framstick framstick;
  public ErrManager errManager;
  
  protected UndoAction undoAction;
  protected RedoAction redoAction;
  protected ChangeAction changeAction;
  
  protected Vector undoBuffer = new Vector();
  protected Vector redoBuffer = new Vector();
  
  CompoundEdit compoundEdit;
  MyUndoableEditListener myUndoListener;
  
  boolean compiled = true;
  
  public F0Editor(Framstick fram)
  {
    super("Editor", true, true, true, false);
    
    //Create the document for the text area.
    framstick = fram;
    doc = new F0Document();
    textPane = new JTextPane(doc);
    textPane.setMargin(new Insets(5, 5, 5, 5));
    
    actions = new Hashtable();
    createActionTable(textPane);
    
    attrs = new SimpleAttributeSet();
    StyleConstants.setBold(attrs, true);
    StyleConstants.setFontFamily(attrs, "monospaced");
    StyleConstants.setFontSize(attrs, 12);
    StyleConstants.setForeground(attrs, new Color(130, 130, 130));
    
    JToolBar toolBar = new JToolBar();
    toolBar.setFloatable(false);
    addButtons(toolBar);
    
    JPanel noWrapPanel = new JPanel();
    noWrapPanel.setLayout(new BorderLayout());
    noWrapPanel.add(textPane);
    
    JScrollPane scrollPane = new JScrollPane(noWrapPanel);
    scrollPane.setBorder(new EmptyBorder(new Insets(0, 0, 0, 0)));
    
    mainNode = new DefaultMutableTreeNode("Messages");
    treeModel = new DefaultTreeModel(mainNode);
    errors = new DefaultMutableTreeNode("Errors");
    warnings = new DefaultMutableTreeNode("Warnings");
    hints = new DefaultMutableTreeNode("Hints");
    
    mainNode.add(errors);
    mainNode.add(warnings);
    mainNode.add(hints);
    
    JTree tree = new JTree(treeModel);
    JScrollPane treeView = new JScrollPane(tree);
    
    linijkaDoc = new DefaultStyledDocument();
    JTextPane linijka = new JTextPane(linijkaDoc);
    linijka.setMargin(new Insets(5, 5, 5, 5));
    linijka.setCaretPosition(0);
    linijka.setEditable(false);
    
    scrollPane.setRowHeaderView(linijka);
    
    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true,scrollPane, treeView);
    splitPane.setDividerLocation(250);
    //Add the components to the frame.
    JPanel contentPane = new JPanel();
    contentPane.setLayout(new BorderLayout());
    contentPane.setPreferredSize(new Dimension(500, 400));
    contentPane.add(toolBar, BorderLayout.NORTH);
    contentPane.add(splitPane, BorderLayout.CENTER);
    setContentPane(contentPane);
    
    //Start watching for undoable edits and caret changes.
    myUndoListener = new MyUndoableEditListener();
    doc.addUndoableEditListener(myUndoListener);
    doc.addDocumentListener(new MyDocumentListener());
    framstick.addFramstickListener(this);
    this.addInternalFrameListener(this);
    this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
  }
  
  private void createActionTable(JTextComponent textComponent)
  {
    Action[] actionsArray = textComponent.getActions();
    for (int i = 0; i < actionsArray.length; i++)
    {
      Action a = actionsArray[i];
      actions.put(a.getValue(Action.NAME), a);
    }
  }
  
  public Action getActionByName(String name)
  {
    return (Action) (actions.get(name));
  }
  
  protected class MyDocumentListener implements DocumentListener
  {
    int liczba_linii;
    
    MyDocumentListener()
    {
      liczba_linii = 0;
    }
    
    public void insertUpdate(DocumentEvent e)
    {
      compiled = false;
      displayEditInfo(e);
    }
    
    public void removeUpdate(DocumentEvent e)
    {
      compiled = false;
      displayEditInfo(e);
    }
    
    public void changedUpdate(DocumentEvent e)
    {
      displayEditInfo(e);
    }
    
    private void displayEditInfo(DocumentEvent e)
    {
      Document document = e.getDocument();
      Element rootElement = document.getDefaultRootElement();
      int axx = rootElement.getElementCount();
      if (liczba_linii < axx)
      {
        try
        {
          for (int i = 1; i <= (axx - liczba_linii); i++)
          {
            linijkaDoc.insertString(linijkaDoc.getLength(), String.valueOf(++liczba_linii) + "\n", attrs);
          }
        }
        catch (Exception ex)
        {
          ;
        }
      }
    }
  }
  
  public void initDocument()
  {
    try
    {
      doc.remove(0, doc.getLength());
      doc.insertString(doc.getLength(), framstick.genotype, attrs);
    }
    catch (BadLocationException ble)
    {
      System.err.println("Couldn't insert initial text.");
    }
    textPane.setCaretPosition(0);
    ErrManager.msg.clear();
    compiled = true;
  }
  
  public void refreshDocument()
  {
    textPane.setCaretPosition(0);
    try
    {
      doc.remove(0, doc.getLength());
      doc.insertString(doc.getLength(), framstick.modelToString(), attrs);
      //System.out.println(framstick.modelToString());
    }
    catch (BadLocationException ble)
    {
      System.err.println("Couldn't insert initial text.");
    }
    textPane.setCaretPosition(0);
    undoBuffer.clear();
    redoBuffer.clear();
    undoAction.updateUndoState();
    redoAction.updateRedoState();
    compiled = true;
  }
  
  //This one listens for edits that can be undone.
  protected class MyUndoableEditListener implements UndoableEditListener
  {
    public void undoableEditHappened(UndoableEditEvent e)
    {
      //Remember the edit and update the menus.
      UndoableEdit sort = e.getEdit();
      if (sort.getPresentationName().equals("addition") || sort.getPresentationName().equals("deletion"))
      {
        if (compoundEdit == null)
        {
          compoundEdit = new CompoundEdit();
        }
        else
        {
          compoundEdit.end();
          compoundEdit = new CompoundEdit();
        }
        compoundEdit.addEdit(e.getEdit());
        undoBuffer.add(compoundEdit);
        if (undoBuffer.size() > 10) undoBuffer.removeElementAt(0);
        redoBuffer.clear();
      }
      else
        compoundEdit.addEdit(e.getEdit());
      
      undoAction.updateUndoState();
      redoAction.updateRedoState();
    }
  }
  
  protected void addButtons(JToolBar toolBar)
  {
    
    JButton button = null;
    
    button = new JButton();
    button.setText("");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Play24.gif")));
    button.setToolTipText("Rebuild model");
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ErrManager.msg.clear();
        try
        {
          ErrManager.msg.clear();
          warnings.removeAllChildren();
          errors.removeAllChildren();
          hints.removeAllChildren();
          
          treeModel.reload();
          
          framstick.genotype = doc.getText(0, doc.getLength());
          framstick.rebuildModel();
          
          for (int x = 0; x < ErrManager.msg.size(); x++)
          {
            Message message = (Message)ErrManager.msg.get(x);
            switch (message.status)
            {
              case ErrManager.ER_WARNING :
                warnings.add(new DefaultMutableTreeNode(message.description));
                break;
              case ErrManager.ER_ERROR :
                errors.add(new DefaultMutableTreeNode(message.description));
                break;
              case ErrManager.ER_HINT :
                hints.add(new DefaultMutableTreeNode(message.description));
            }
          }
        }
        catch (Exception ex)
        {
          ;
        }
        
        //Marcin
        //framstick.fireMarkChangedEvent();
        framstick.fireSignificantEditEvent();
        compiled = true;
      }
    });
    
    toolBar.add(button);
    
    //second button
    button = new JButton();
    button.setToolTipText("Check syntax");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Properties.gif")));
    button.setText("");
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        try
        {
          GenotypeParser parser = new GenotypeParser("class", doc.getText(0, doc.getLength()));
          parser.createModel();
          
          warnings.removeAllChildren();
          errors.removeAllChildren();
          hints.removeAllChildren();
          
          treeModel.reload();
          
          for (int x = 0; x < ErrManager.msg.size(); x++)
          {
            Message message = (Message)ErrManager.msg.get(x);
            switch (message.status)
            {
              case ErrManager.ER_WARNING :
                warnings.add(new DefaultMutableTreeNode(message.description));
                break;
              case ErrManager.ER_ERROR :
                errors.add(new DefaultMutableTreeNode(message.description));
                break;
              case ErrManager.ER_HINT :
                hints.add(new DefaultMutableTreeNode(message.description));
            }
          }
          
          ErrManager.msg.clear();
        }
        catch (Exception ex)
        {
          ;
        }
      }
    });
    toolBar.add(button);
    
    toolBar.addSeparator();
    
    //second button
    button = new JButton();
    button.setAction(getActionByName(DefaultEditorKit.copyAction));
    button.setToolTipText("Copy");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Copy24.gif")));
    button.setText("");
                /*        button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                               // displayResult("Action for second button");
                            }
                        });
                 */
    toolBar.add(button);
    
    //third button
    button = new JButton();
    button.setAction(getActionByName(DefaultEditorKit.cutAction));
    button.setToolTipText("Cut");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Cut24.gif")));
    button.setText("");
                /*      button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                              //  displayResult("Action for third button");
                            }
                 
                                });
                 */
    toolBar.add(button);
    
    button = new JButton();
    button.setAction(getActionByName(DefaultEditorKit.pasteAction));
    button.setToolTipText("Paste");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Paste24.gif")));
    button.setText("");
    
                /*
                                button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                              //  displayResult("Action for third button");
                            }
                        });
                 */
    toolBar.add(button);
    toolBar.addSeparator();
    
    undoAction = new UndoAction();
    redoAction = new RedoAction();
    changeAction = new ChangeAction();
    
    button = new JButton();
    button.setAction(undoAction);
    button.setToolTipText("Undo");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Undo24.gif")));
    button.setText("");
    toolBar.add(button);
    
    button = new JButton();
    button.setAction(redoAction);
    button.setToolTipText("Redo");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Redo24.gif")));
    button.setText("");
    toolBar.add(button);
    
    toolBar.addSeparator();
    
    button = new JButton();
    button.setAction(changeAction);
    button.setToolTipText("model change state");
    button.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/rbrs.gif")));
    toolBar.add(button);
  }
  
  class ChangeAction extends AbstractAction
  {
    public ChangeAction()
    {
      super();
      setEnabled(false);
    }
    
    public void actionPerformed(ActionEvent e)
    {
      refreshDocument();
      setEnabled(false);
    }
    
    protected void setAvailable()
    {
      setEnabled(true);
    }
  };
  
  class UndoAction extends AbstractAction
  {
    public UndoAction()
    {
      super();
      setEnabled(false);
    }
    
    public void actionPerformed(ActionEvent e)
    {
      CompoundEdit cEdit = (CompoundEdit)undoBuffer.remove(undoBuffer.size() - 1);
      if (cEdit.isInProgress())
        cEdit.end();
      redoBuffer.add(cEdit);
      cEdit.undo();
      updateUndoState();
      redoAction.updateRedoState();
    }
    
    protected void updateUndoState()
    {
      if (undoBuffer.size() > 0)
        setEnabled(true);
      else
        setEnabled(false);
    }
  }
  
  class RedoAction extends AbstractAction
  {
    public RedoAction()
    {
      super();
      setEnabled(false);
    }
    
    public void actionPerformed(ActionEvent e)
    {
      CompoundEdit cEdit = (CompoundEdit)redoBuffer.remove(redoBuffer.size() - 1);
      if (cEdit.isInProgress())
        cEdit.end();
      undoBuffer.add(cEdit);
      
      cEdit.redo();
      updateRedoState();
      undoAction.updateUndoState();
    }
    
    protected void updateRedoState()
    {
      if (redoBuffer.size() > 0)
        setEnabled(true);
      else
        setEnabled(false);
    }
  }
  
  public void significantEdit(FramstickEvent framEvent)
  {
    //		System.out.println("Editor: model was updated ! - significantEdit");
    if (compiled)
    {
      changeAction.setEnabled(true);
      refreshDocument();
    }
  }
  
  public void markChanged(FramstickEvent framEvent)
  {
    //		System.out.println("Editor: model was updated ! - markChange");
    if (compiled)
    {
      refreshDocument();
      changeAction.setEnabled(true);
    }
    
  }
  
  
  
  public void internalFrameOpened(InternalFrameEvent e)
  { ; }
  public void internalFrameDeactivated(InternalFrameEvent e)
  { ; }
  public void internalFrameClosing(InternalFrameEvent e)
  { ; }
  public void internalFrameClosed(InternalFrameEvent e)
  { ; }
  public void internalFrameIconified(InternalFrameEvent e)
  { ; }
  public void internalFrameDeiconified(InternalFrameEvent e)
  { ; }
  public void internalFrameActivated(InternalFrameEvent e)
  {
    if (!compiled)
    {
      Object[] options =
      { I18N.getMessage("Overwrite"), I18N.getMessage("Cancel")};
      int n =
      JOptionPane.showOptionDialog(
      this,
      I18N.getMessage("FramChangedUpdate"),
      I18N.getMessage("FramChanged"),
      JOptionPane.YES_NO_CANCEL_OPTION,
      JOptionPane.QUESTION_MESSAGE,
      null,
      options,
      options[1]);
      if (n == JOptionPane.YES_OPTION)
      {
        ;
      }
      else if (n == JOptionPane.NO_OPTION)
      {
        return;
      }
      else if (n == JOptionPane.CANCEL_OPTION)
      {
        return;
      }
      else
      {
        return;
      }
      changeAction.actionPerformed(null);
    }
  }
  
}