package fred;

import jGDK.ErrManager;
import jGDK.Framstick;
import jGDK.FramstickEvent;
import jGDK.FramstickListener;
import jGDK.GenotypeParser;
import jGDK.I18N;
import jGDK.Library;
import jGDK.Message;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.ref.WeakReference;
import java.net.*;
import java.util.Locale;
import java.util.Properties;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.text.DefaultEditorKit;

/**
 * <P>MainFrame class</P><BR>
 * <p>Main window class. This is basic, the most important class. This class is managing all windows
 * in program. It gives the basic application interface the user need. This class uses instance of
 * MyDesktop Layout to scale and move windows on the desktop of the application.</p>
 * @author Damian Jarzebski
 * @version 2.0
 */
public class MainFrame extends JFrame implements ActionListener, InternalFrameListener, FramstickListener, MouseListener
{
  
  private String country = null;
  private String language = null;
  private Properties settings = new Properties();
  private boolean fileChanged = false;
  private File lastfile = null;
  private final String settingsfilename = "settings.Fred";
  
  //all windows
  private Model model = null;
  private FramInfo framInfo = null;
  F0Editor editor = null;
  BodyFrame body = null;
  private WeakReference wBody = new WeakReference(body);
  Help help = null;
  BrainFrame brain = null;
  private WeakReference wBrain = new WeakReference(brain);
  static Messages messages = null;
  private WeakReference wProp = null;
  
  //vector to store windows references
  Vector brainV = new Vector();
  Vector propsV = new Vector();
  Vector bodyV = new Vector();
  
  private int activeWindow = 0;
  
  // Possible Look & Feels
  final private static String mac = "com.sun.java.swing.plaf.mac.MacLookAndFeel";
  final private static String metal = "javax.swing.plaf.metal.MetalLookAndFeel";
  final private static String motif = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
  final private static String windows = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
  
  // The current Look & Feel
  String currentLookAndFeel = metal;
  
  //jGDK - variables
  static ErrManager err = new ErrManager();
  Framstick framstick = new Framstick();
  String s_plik = new String();
  
  static String separator = System.getProperty("file.separator");
  
  JToolBar jToolBar = new JToolBar();
  
  //menu View
  JCheckBoxMenuItem panel = null;
  JCheckBoxMenuItem BWires = new JCheckBoxMenuItem();
  JCheckBoxMenuItem BBox = new JCheckBoxMenuItem();
  JCheckBoxMenuItem BBack = new JCheckBoxMenuItem();
  JCheckBoxMenuItem BDet = new JCheckBoxMenuItem();
  
  JPanel contentPane = null;
  JMenuBar jMenuBar1 = new JMenuBar();
  
  Action jCloseFile_Action = null;
  ImageIcon openIcon = null;
  ImageIcon newIcon = null;
  ImageIcon internetIcon = null;
  ImageIcon helpIcon = null;
  ImageIcon undoIcon = null;
  ImageIcon redoIcon = null;
  ImageIcon saveIcon = null;
  ImageIcon cutIcon = new ImageIcon(ClassLoader.getSystemResource("images/Cut24.gif"));
  ImageIcon copyIcon = new ImageIcon(ClassLoader.getSystemResource("images/Copy24.gif"));
  ImageIcon pasteIcon = new ImageIcon(ClassLoader.getSystemResource("images/Paste24.gif"));
  
  JLabel statusBar = new JLabel(" ");
  //Menu edit
  JMenu jMenuEdit = new JMenu();
  JMenuItem jMenuEditCut = new JMenuItem();
  JMenuItem jMenuEditCopy = new JMenuItem();
  JMenuItem jMenuEditPaste = new JMenuItem();
  
  JMenu jMenuWindow = new JMenu();
  JMenu jMenuView = new JMenu();
  JCheckBoxMenuItem jMenuWindowModel = new JCheckBoxMenuItem();
  JCheckBoxMenuItem jMenuWindowEditor = new JCheckBoxMenuItem();
  JCheckBoxMenuItem jMenuWindowBody = new JCheckBoxMenuItem();
  JCheckBoxMenuItem jMenuWindowBrain = new JCheckBoxMenuItem();
  JCheckBoxMenuItem jMenuWindowMessages = new JCheckBoxMenuItem();
  JMenu jMenuWindowLaF = new JMenu();
  JRadioButtonMenuItem jMenuWindowLaFmac = new JRadioButtonMenuItem();
  JRadioButtonMenuItem jMenuWindowLaFmetal = new JRadioButtonMenuItem();
  JRadioButtonMenuItem jMenuWindowLaFmotif = new JRadioButtonMenuItem();
  JRadioButtonMenuItem jMenuWindowLaFwindows = new JRadioButtonMenuItem();
  JMenuItem jMenuEditUndo = new JMenuItem();
  JMenuItem jMenuEditRedo = new JMenuItem();
  JMenuItem jMenuFileSave = new JMenuItem();
  JMenuItem jMenuFileSaveAs = new JMenuItem();
  JMenuItem info = new JMenuItem();
  
  JDesktopPane jDesktop = new JDesktopPane();
  //window toolbar
  JToolBar jTBWindow = new JToolBar();
  JToggleButton jTBWindowModel = new JToggleButton();
  JToggleButton jTBWindowBrain = new JToggleButton();
  JToggleButton jTBWindowBody = new JToggleButton();
  JToggleButton jTBWindowEditor = new JToggleButton();
  JToggleButton jTBWindowMessages = new JToggleButton();
  
  JCheckBoxMenuItem jMenuWindowWToolbar = new JCheckBoxMenuItem();
  JCheckBoxMenuItem jMenuWindowBToolbar = new JCheckBoxMenuItem();
  
  JButton jToolBarSaveFile = new JButton();
  JButton jTBUndo = new JButton("");
  JButton jTBRedo = new JButton("");
  
  String Title = "Fred - Framstics Editor 2.1";
  String filename, dir;
  
  /**
   * <P>MainFrame constructor.</P><BR>
   * Construct this class and loads imediately the <CODE> Browser</Code> window with
   *  the <CODE>file</CODE> specified in prameter
   * @param file - *.gen to load on start
   */
  public MainFrame(String file)
  {
    int i;
    i = file.lastIndexOf('\\');
    if (i <= 0)
      i = file.lastIndexOf('/');
    if (i > 0)
    {
      this.filename = new String(file.substring(i + 1));
      this.dir = new String(file.substring(0, i));
    }
    //*********************************************************
    System.out.println(dir + separator + filename);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    jbInit();
    Object[] atr =
    { filename };
    try
    {
      File f = new File(dir + separator + filename);
      if (!f.exists())
      {
        messages.append(
        I18N.getMessage("FredMain"),
        I18N.getMessage("FileDExist") + " " + filename,
        ErrManager.ER_ERROR);
        return;
      }
      if (!f.canRead())
      {
        messages.append("Main Frame", I18N.getMessage("NoAccessFile") + " " + filename, ErrManager.ER_ERROR);
        return;
      }
      lastfile = f;
      InputStream input = new FileInputStream(f);
      int n = input.available();
      byte buf[] = new byte[n];
      if ((n = input.read(buf)) >= 0)
      {
        messages.append(I18N.getMessage("Browser"), I18N.getFormated("FileRead", atr), ErrManager.ER_MESSAGE);
      }
      input.close();
      s_plik = new String(buf);
    }
    catch (IOException ex)
    {
      System.out.println(I18N.getMessage("LoadError") + ": " + ex);
    }
    this.setTitle(Title + " - " + ((dir != null) ? (dir + separator + filename) : ""));
    Browser browser = new Browser(this.s_plik, framstick);
    WeakReference wBrowser = new WeakReference(browser);
    browser = null;
    ((Browser)wBrowser.get()).setModal(true);
    ((Browser)wBrowser.get()).show();
    updateMessages();
    if (((Browser)wBrowser.get()).accepted)
    {
      atr[0] = framstick.name;
      messages.append(I18N.getMessage("Browser"), I18N.getFormated("FramLoaded", atr), ErrManager.ER_MESSAGE);
    }
    else
      messages.append(I18N.getMessage("Browser"), I18N.getMessage("GenLoadCancel"), ErrManager.ER_MESSAGE);
    this.setTitle(this.Title + " - " + ((dir != null) ? (dir + separator + filename) : "") + "(" + framstick.name + ")");
    enableMenu();
    wBrowser = null;
    System.gc();
    fileChanged = false;
  }
  /**
   * <P>MainFrame constructor.</P><BR>
   */
  public MainFrame()
  {
    this.setName("Fred");
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    jbInit();
    System.gc();
  }
  /**
   * Sets the current L&F on each module
   */
  public void updateLookAndFeel()
  {
    try
    {
      UIManager.setLookAndFeel(currentLookAndFeel);
      SwingUtilities.updateComponentTreeUI(this);
    }
    catch (Exception ex)
    {
      System.out.println(I18N.getMessage("FailedLoadingLF") + ": " + currentLookAndFeel);
      System.out.println(ex);
    }
  }
  /**
   * Checks if <CODE>laf</CODE> is available.
   * @param laf - L&F to check
   * @return true if is available.
   */
  protected boolean isAvailableLookAndFeel(String laf)
  {
    try
    {
      Class lnfClass = Class.forName(laf);
      LookAndFeel newLAF = (LookAndFeel) (lnfClass.newInstance());
      return newLAF.isSupportedLookAndFeel();
    }
    catch (Exception e)
    { // If ANYTHING weird happens, return false
      return false;
    }
  }
  /**
   * Creates L&F menu
   */
  private void setLaFMenu()
  {
    ButtonGroup LaFgroup = new ButtonGroup();
    //"metal" Button
    jMenuWindowLaFmetal.addActionListener(this);
    jMenuWindowLaFmetal.setIcon(null);
    jMenuWindowLaFmetal.setText("metal");
    jMenuWindowLaFmetal.setEnabled(isAvailableLookAndFeel(metal));
    LaFgroup.add(jMenuWindowLaFmetal);
    jMenuWindowLaF.add(jMenuWindowLaFmetal);
    //"mac" Button
    jMenuWindowLaFmac.addActionListener(this);
    jMenuWindowLaFmac.setIcon(null);
    jMenuWindowLaFmac.setText("mac");
    jMenuWindowLaFmac.setEnabled(isAvailableLookAndFeel(mac));
    jMenuWindowLaF.add(jMenuWindowLaFmac);
    LaFgroup.add(jMenuWindowLaFmac);
    //"motif" Button
    jMenuWindowLaFmotif.addActionListener(this);
    jMenuWindowLaFmotif.setIcon(null);
    jMenuWindowLaFmotif.setText("motif");
    jMenuWindowLaFmotif.setEnabled(isAvailableLookAndFeel(motif));
    jMenuWindowLaF.add(jMenuWindowLaFmotif);
    LaFgroup.add(jMenuWindowLaFmotif);
    //"windows" Button
    jMenuWindowLaFwindows.addActionListener(this);
    jMenuWindowLaFwindows.setIcon(null);
    jMenuWindowLaFwindows.setText("windows");
    jMenuWindowLaFwindows.setEnabled(isAvailableLookAndFeel(windows));
    jMenuWindowLaF.add(jMenuWindowLaFwindows);
    LaFgroup.add(jMenuWindowLaFwindows);
    jMenuWindowLaF.setText(I18N.getMessage("LFStyle"));
    jMenuView.add(jMenuWindowLaF);
  }
  /**
   * Initing all needed static classes from jGDK package.
   */
  private void initJGDK()
  {
    //jGDK-----------
    System.out.println(I18N.getMessage("InitJGDK"));
    Library library = null;
    try
    {
      library = new Library();
    }
    catch (Exception ex)
    {
      System.out.println(ex.getLocalizedMessage());
    }
/*
                JarURLConnection uc = null;
                try {
                        try {
                                uc = (JarURLConnection)url.openConnection();
                        }
                        catch (IOException e2) {
                                System.err.println(e2.getLocalizedMessage());
                        }
 
                        try {
                                InputStream input = uc.getInputStream();
                                int n = input.available();
                                char buf[] = new char[n];
                                BufferedReader br = new BufferedReader(new InputStreamReader(input));
                                System.out.println("do odczytania:" + n); //trial
                                int r = 0;
                                if ((r = br.read(buf, 0, n)) >= 0) {
                                        System.out.println("odczytano:" + r); //trial
                                        messages.append(
                                                I18N.getMessage("MainFrame", "Main frame"),
                                                I18N.getMessage("ReadConfig", "The config file has been read"),
                                                ErrManager.ER_MESSAGE);
                                        System.out.println(I18N.getMessage("ReadConfig", "The config file has been read"));
                                        String s = new String(buf);
                                }
                                br.close();
                                input.close();
                                library = new Library(new String(buf));
                        }
                        catch (IOException e) {
                                System.out.println(e.getLocalizedMessage());
                        }
                }
                catch (ClassCastException e2) {
                        try {
                                File f = new File("model.def");
                                if (!f.exists()) {
                                        messages.append(I18N.getMessage("MainFrame", "Main frame"), "The config file don't exist: " + f.getName(), ErrManager.ER_ERROR);
                                        System.out.println("The config file don't exist: " + f.getName());
                                        return;
                                }
                                if (!f.canRead()) {
                                        messages.append(I18N.getMessage("MainFrame", "Main frame"), "There is no access to the config file: " + f.getName(), ErrManager.ER_ERROR);
                                        System.out.println("There is no access to the config file: " + f.getName());
                                        return;
                                }
                                InputStream input = new FileInputStream(f);
                                int n = input.available();
                                byte buf[] = new byte[n];
                                if ((n = input.read(buf)) >= 0) {
                                        messages.append(
                                                I18N.getMessage("MainFrame", "Main frame"),
                                                I18N.getMessage("ReadConfig", "The config file has been read"),
                                                ErrManager.ER_MESSAGE);
                                        System.out.println(I18N.getMessage("ReadConfig", "The config file has been read"));
                                        String s = new String(buf);
                                }
                                input.close();
                                library = new Library(new String(buf));
                        }
                        catch (IOException ex) {}
 */
    Framstick.library = library;
    GenotypeParser.library = library;
    Tazo2.createTextures(library);
    framstick.rebuildModel();
    framstick.addFramstickListener(this);
  }
  
  private void makeFileMenu()
  {
    //    System.out.print("Making File menu...");
    JMenu jMenuFile = new JMenu();
    JMenuItem jMenuFileExit = new JMenuItem();
    openIcon = new ImageIcon(ClassLoader.getSystemResource("images/Open24.gif"));
    newIcon = new ImageIcon(ClassLoader.getSystemResource("images/New24.gif"));
    internetIcon = new ImageIcon(ClassLoader.getSystemResource("images/Search24.gif"));
    saveIcon = new ImageIcon(ClassLoader.getSystemResource("images/Save24.gif"));
    jMenuFile.setText(I18N.getMessage("File"));
    jMenuFile.setMnemonic(KeyEvent.VK_F);
    jMenuFileExit.setText(I18N.getMessage("Exit"));
    jMenuFileExit.setActionCommand("Exit");
    jMenuFileExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.ALT_MASK));
    jMenuFileExit.addActionListener(this);
    JMenuItem jMenuFileNew = new JMenuItem();
    JMenu jMenuFileOpen = new JMenu();
    JMenuItem jMenuFileOpenFile = new JMenuItem();
    JMenuItem jMenuFileOpenDB = new JMenuItem();
    jMenuFileNew.setText(I18N.getMessage("New"));
    jMenuFileNew.setActionCommand("New");
    jMenuFileNew.setIcon(newIcon);
    jMenuFileNew.addActionListener(this);
    jMenuFileNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
    jMenuFileOpen.setText(I18N.getMessage("OpenFrom"));
    jMenuFileOpenFile.setText(I18N.getMessage("file"));
    jMenuFileOpenFile.setActionCommand("file...");
    jMenuFileOpenFile.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
    jMenuFileOpenFile.addActionListener(this);
    jMenuFileOpenFile.setIcon(openIcon);
    jMenuFileOpen.setIcon(jMenuFileOpenFile.getIcon());
    jMenuFileOpenDB.setText(I18N.getMessage("InternetDB"));
    jMenuFileOpenDB.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
    jMenuFileOpenDB.addActionListener(this);
    jMenuFileOpenDB.setActionCommand("Internet");
    jMenuFileOpenDB.setIcon(internetIcon);
    jMenuFileSave.setEnabled(true);
    jMenuFileSave.setText(I18N.getMessage("Save"));
    jMenuFileSave.setActionCommand("Save");
    jMenuFileSave.setIcon(saveIcon);
    jMenuFileSave.addActionListener(this);
    jMenuFileSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    jMenuFileSaveAs.setEnabled(true);
    jMenuFileSaveAs.setText(I18N.getMessage("SaveAs"));
    jMenuFileSaveAs.setActionCommand("SaveAs");
    jMenuFileSaveAs.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/SaveAs24.gif")));
    jMenuFileSaveAs.addActionListener(this);
    
    jMenuFile.add(jMenuFileNew);
    jMenuFile.add(jMenuFileOpen);
    //Close File
    jCloseFile_Action = new AbstractAction("Close File", new ImageIcon(ClassLoader.getSystemResource("images/Stop24.gif")))
    {
      public void actionPerformed(ActionEvent e)
      {
        if (fileChanged)
        {
          int n =
          JOptionPane.showConfirmDialog(
          (Component)e.getSource(),
          I18N.getMessage("FileNSavedDo"),
          I18N.getMessage("Closing"),
          JOptionPane.YES_NO_CANCEL_OPTION);
          if (n == JOptionPane.YES_OPTION)
          {
            save();
          }
          else if (n == JOptionPane.NO_OPTION)
          {
            ;
          }
          else
          {
            return;
          }
        }
        jMenuWindow.setEnabled(false);
        
        jMenuEdit.setEnabled(false);
        jMenuView.setEnabled(false);
        jTBWindowBody.setEnabled(false);
        jTBWindowModel.setEnabled(false);
        jTBWindowBrain.setEnabled(false);
        jTBWindowEditor.setEnabled(false);
        jTBWindowMessages.setEnabled(false);
        jCloseFile_Action.setEnabled(false);
        jToolBarSaveFile.setEnabled(false);
        jMenuFileSave.setEnabled(false);
        jMenuFileSaveAs.setEnabled(false);
        info.setEnabled(false);
        if (bodyV.size() > 0)
          for (int i = 0;i<bodyV.size(); i++)
          {
            body = (BodyFrame) ((WeakReference)bodyV.remove(i)).get();
            if(body!=null)body.dispose();
          }
        body = null;
        if (model != null)
          model.dispose();
        jTBWindowModel.setSelected(false);
        jTBWindowBody.setSelected(false);
        if (brainV.size() > 0)
          for (int i = 0;i<brainV.size();i++)
          {
            brain = (BrainFrame) ((WeakReference)brainV.elementAt(i)).get();
            if(brain!=null)brain.dispose();
          }
        brain = null;
        jTBWindowBrain.setSelected(false);
        if (editor != null)
          editor.dispose();
        setTitle(Title);
        fileChanged = false;
        if (messages != null)
          messages.clear();
        Object[] atr =
        {(filename != null) ? filename : "" };
        messages.append(I18N.getMessage("MainFrame"), I18N.getFormated("FileClosed", atr), ErrManager.ER_MESSAGE);
        filename = "";
        dir = "";
        if (propsV.size() > 0)
          for (int i = propsV.size() - 1; i >= 0; i--)
          {
            wProp = (WeakReference)propsV.remove(i);
            if(wProp.get()!=null)
            {
              jDesktop.remove(((Prop)wProp.get()));
              if (!((Prop)wProp.get()).isClosed())
                ((Prop)wProp.get()).dispose();
              wProp = null;
            }
          }
        if (framInfo != null)
          framInfo.dispose();
        if (help != null)
        {
          help.dispose();
          help = null;
        }
        framstick = null;
        //        System.gc();
      }
    };
    JMenuItem jMenuFileClose = null;
    jMenuFileClose = jMenuFile.add(jCloseFile_Action);
    jMenuFileClose.setEnabled(true);
    jMenuFileClose.setText(I18N.getMessage("Close"));
    jMenuFileClose.setActionCommand("Close");
    jMenuFileClose.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.CTRL_MASK));
    jMenuFileClose.addActionListener(this);
    jMenuFile.add(jMenuFileSave);
    jMenuFile.add(jMenuFileSaveAs);
    jMenuFile.addSeparator();
    info = new JMenuItem(I18N.getMessage("FramInfo"));
    info.setActionCommand("Info");
    info.addActionListener(this);
    info.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Information24.gif")));
    info.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
    info.setMnemonic(KeyEvent.VK_I);
    jMenuFile.add(info);
    jMenuFile.addSeparator();
    jMenuFile.add(jMenuFileExit);
    jMenuFileOpen.add(jMenuFileOpenFile);
    jMenuFileOpen.add(jMenuFileOpenDB);
    
    jMenuBar1.add(jMenuFile);
    //    System.out.println("OK");
  }
  private void makeEditMenu()
  {
    //		System.out.print("Making Edit menu...");
    undoIcon = new ImageIcon(ClassLoader.getSystemResource("images/Undo24.gif"));
    redoIcon = new ImageIcon(ClassLoader.getSystemResource("images/Redo24.gif"));
    jMenuEdit.setText(I18N.getMessage("Edit"));
    jMenuEdit.setMnemonic(KeyEvent.VK_E);
    jMenuEditCut.setEnabled(false);
    jMenuEditCut.setText(I18N.getMessage("Cut"));
    jMenuEditCut.setActionCommand("Cut");
    jMenuEditCut.setIcon(cutIcon);
    jMenuEditCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
    jMenuEditCopy.setEnabled(false);
    jMenuEditCopy.setText(I18N.getMessage("Copy"));
    jMenuEditCopy.setActionCommand("Copy");
    jMenuEditCopy.setIcon(copyIcon);
    jMenuEditCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
    jMenuEditPaste.setEnabled(false);
    jMenuEditPaste.setText(I18N.getMessage("Paste"));
    jMenuEditPaste.setActionCommand("Paste");
    jMenuEditPaste.setIcon(pasteIcon);
    jMenuEditPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
    //jMenuEditUndo.setEnabled(false);
    jMenuEditUndo.setText(I18N.getMessage("Undo"));
    jMenuEditUndo.setActionCommand("Undo");
    jMenuEditUndo.addActionListener(this);
    jMenuEditUndo.setIcon(undoIcon);
    jMenuEditUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
    //jMenuEditRedo.setEnabled(false);
    jMenuEditRedo.setText(I18N.getMessage("Redo"));
    jMenuEditRedo.setActionCommand("Redo");
    jMenuEditRedo.setIcon(redoIcon);
    jMenuEditRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
    jMenuEdit.add(jMenuEditUndo);
    jMenuEdit.add(jMenuEditRedo);
    jMenuEdit.addSeparator();
    jMenuEdit.add(jMenuEditCut);
    jMenuEdit.add(jMenuEditCopy);
    jMenuEdit.add(jMenuEditPaste);
    
    jMenuBar1.add(jMenuEdit);
    //		System.out.println("OK");
  }
  private void makeWindowMenu()
  {
    //		System.out.print("Making Window Menu...");
    jMenuWindow.setText(I18N.getMessage("Window"));
    jMenuWindow.setMnemonic(KeyEvent.VK_W);
    jMenuWindowModel.setText(I18N.getMessage("Model"));
    jMenuWindowModel.setActionCommand("Model");
    jMenuWindowModel.addActionListener(this);
    jMenuWindowEditor.setText(I18N.getMessage("GEditor"));
    jMenuWindowEditor.addActionListener(this);
    jMenuWindowEditor.setActionCommand("Editor");
    jMenuWindowBody.setActionCommand("Body");
    jMenuWindowBody.setText(I18N.getMessage("Body"));
    jMenuWindowBody.setActionCommand("Body");
    jMenuWindowBody.addActionListener(this);
    jMenuWindowBrain.setText(I18N.getMessage("Brain"));
    jMenuWindowBrain.setActionCommand("Brain");
    jMenuWindowBrain.addActionListener(this);
    jMenuWindowMessages.setText(I18N.getMessage("Messages"));
    jMenuWindowMessages.addActionListener(this);
    jMenuWindowMessages.setActionCommand("Messages");
    jMenuWindowWToolbar.setSelected(true);
    jMenuWindowWToolbar.setText(I18N.getMessage("WToolbar"));
    jMenuWindowWToolbar.setActionCommand("Windows toolbar");
    jMenuWindowWToolbar.addActionListener(this);
    jMenuWindowBToolbar.setText(I18N.getMessage("BToolbar"));
    jMenuWindowBToolbar.setActionCommand("Basic toolbar");
    jMenuWindowBToolbar.setSelected(true);
    jMenuWindowBToolbar.addActionListener(this);
    jMenuWindow.add(jMenuWindowModel);
    jMenuWindow.add(jMenuWindowEditor);
    jMenuWindow.addSeparator();
    jMenuWindow.add(jMenuWindowBody);
    jMenuWindow.add(jMenuWindowBrain);
    jMenuWindow.addSeparator();
    jMenuWindow.add(jMenuWindowMessages);
    jMenuWindow.addSeparator();
    jMenuWindow.add(jMenuWindowWToolbar);
    jMenuWindow.add(jMenuWindowBToolbar);
    jMenuWindow.addSeparator();
    JMenu jMenuAddWindow = new JMenu(I18N.getMessage("Add"));
    JMenuItem jMenuAddBody = new JMenuItem(I18N.getMessage("BodyWindow"));
    jMenuAddBody.setActionCommand("AddBody");
    jMenuAddBody.addActionListener(this);
    jMenuAddWindow.add(jMenuAddBody);
    JMenuItem jMenuAddBrain = new JMenuItem(I18N.getMessage("BrainWindow"));
    jMenuAddBrain.setActionCommand("AddBrain");
    jMenuAddBrain.addActionListener(this);
    jMenuAddWindow.add(jMenuAddBrain);
    jMenuWindow.add(jMenuAddWindow);
/*    jMenuWindow.addSeparator();                        Memory usage report - uncomment to enable
    JMenuItem mItem = new JMenuItem(I18N.getMessage("MemUsage"));
    mItem.setAction(new AbstractAction(I18N.getMessage("MemUsage"))
    {
      public void actionPerformed(ActionEvent e)
      {
        MemoryUsage mu = new MemoryUsage(1000, false);
        mu.setLocation(getLocation());
        MemoryUsage.refresh();
      }
    });
    jMenuWindow.add(mItem);*/
    
    jMenuBar1.add(jMenuWindow);
    //		System.out.println("OK");
  }
  private void makeViewMenu()
  {
    //		System.out.print("Making View Menu...");
    jMenuView.setText(I18N.getMessage("View"));
    jMenuView.setMnemonic(KeyEvent.VK_V);
    JMenu jMBody = new JMenu(I18N.getMessage("BodyView"));
    jMenuView.add(jMBody);
    //BWires.setAction(body.jTBWires_Action);
    BWires.setText(I18N.getMessage("Wires"));
    BWires.setActionCommand("Wires");
    BWires.addActionListener(this);
    BWires.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/wires.gif")));
    jMBody.add(BWires);
    //BBox.setAction(body.jTBBox_Action);
    BBox.setText(I18N.getMessage("box"));
    BBox.setActionCommand("Box");
    BBox.addActionListener(this);
    BBox.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/box.gif")));
    jMBody.add(BBox);
    BBox.setSelected(true);
    //BBack.setAction(body.jTBBackground_Action);
    ImageIcon details = new ImageIcon(ClassLoader.getSystemResource("images/Details.gif"));
    BBack.setText(I18N.getMessage("background"));
    BBack.setActionCommand("Background");
    BBack.addActionListener(this);
    BBack.setIcon(details);
    jMBody.add(BBack);
    //BDet.setAction(body.jTBDetails_Action);
    BDet.setText(I18N.getMessage("details"));
    BDet.setActionCommand("Details");
    BDet.addActionListener(this);
    BDet.setIcon(details);
    jMBody.add(BDet);
    panel = new JCheckBoxMenuItem(I18N.getMessage("ShowPanel"));
    jMBody.add(panel);
    panel.addActionListener(this);
    panel.setActionCommand("panel");
    jMenuView.addSeparator();
    
    jMenuBar1.add(jMenuView);
    //		System.out.println("OK");
    
    setLaFMenu();
    jMenuView.addSeparator();
    jMenuView.add(new AbstractAction(I18N.getMessage("Language"))
    {
      public void actionPerformed(ActionEvent e)
      {
        languageSelect();
      }
    });
  }
  private void makeHelpMenu()
  {
    //		System.out.print("Making Help menu...");
    JMenu jMenuHelp = new JMenu();
    JMenuItem jMenuHelpAbout = new JMenuItem();
    helpIcon = new ImageIcon(ClassLoader.getSystemResource("images/Help24.gif"));
    jMenuHelp.setText(I18N.getMessage("Help"));
    jMenuHelp.setEnabled(true);
    jMenuHelp.setMnemonic(KeyEvent.VK_H);
    jMenuHelpAbout.setText(I18N.getMessage("About"));
    jMenuHelpAbout.setActionCommand("About...");
    jMenuHelpAbout.addActionListener(this);
    jMenuHelpAbout.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/About24.gif")));
    JMenuItem jMenuHelpHelp = new JMenuItem();
    jMenuHelpHelp.setText(I18N.getMessage("Help"));
    jMenuHelpHelp.setActionCommand("Help");
    jMenuHelpHelp.addActionListener(this);
    jMenuHelpHelp.setIcon(helpIcon);
    jMenuHelpHelp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
    jMenuHelp.add(jMenuHelpHelp);
    jMenuHelp.add(jMenuHelpAbout);
    
    jMenuBar1.add(jMenuHelp);
    //		System.out.println("OK");
  }
  private void makeWindowsToolbar()
  {
    //		System.out.print("Making Windows toolbar...");
    jTBWindow.setName(I18N.getMessage("WToolbar"));
    
    jTBWindowModel.setToolTipText(I18N.getMessage("Model"));
    jTBWindowModel.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/Model.gif")));
    jTBWindowModel.setMargin(new Insets(1, 1, 1, 1));
    jTBWindowModel.setText("");
    jTBWindowModel.addActionListener(this);
    jTBWindowBrain.setToolTipText(I18N.getMessage("Brain"));
    jTBWindowBrain.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/brain.gif")));
    jTBWindowBrain.setMargin(new Insets(1, 1, 1, 1));
    jTBWindowBrain.setText("");
    jTBWindowBrain.addActionListener(this);
    jTBWindowBody.setToolTipText(I18N.getMessage("Body"));
    jTBWindowBody.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/body.gif")));
    jTBWindowBody.setMargin(new Insets(1, 1, 1, 1));
    jTBWindowBody.setText("");
    jTBWindowBody.addActionListener(this);
    jTBWindow.setOrientation(JToolBar.HORIZONTAL);
    jTBWindowEditor.setToolTipText(I18N.getMessage("GEditor"));
    jTBWindowEditor.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/editor.GIF")));
    jTBWindowEditor.setMargin(new Insets(1, 1, 1, 1));
    jTBWindowEditor.addActionListener(this);
    jTBWindowMessages.setMargin(new Insets(2, 2, 2, 2));
    jTBWindowMessages.setToolTipText(I18N.getMessage("Messages"));
    jTBWindowMessages.setIcon(new ImageIcon(ClassLoader.getSystemResource("images/messages.gif")));
    jTBWindowMessages.setMargin(new Insets(1, 1, 1, 1));
    jTBWindowMessages.setText("");
    jTBWindowMessages.addActionListener(this);
    jTBWindowMessages.setActionCommand("Messages");
    jTBWindowEditor.setActionCommand("Editor");
    jTBWindowModel.setActionCommand("Model");
    jTBWindowBody.setActionCommand("Body");
    jTBWindowBrain.setActionCommand("Brain");
    jTBWindowMessages.setActionCommand("Messages");
    jTBWindow.add(jTBWindowEditor);
    jTBWindow.add(jTBWindowModel);
    jTBWindow.add(jTBWindowBody);
    jTBWindow.add(jTBWindowBrain);
    jTBWindow.add(jTBWindowMessages);
    //		System.out.println("OK");
  }
  private void makeMainToolbar()
  {
    //		System.out.print("Making Main toolbar...");
    JButton jToolBarOpenFile = new JButton();
    JButton jToolBarCloseFile = new JButton();
    JButton jToolBarHelp = new JButton();
    jToolBar.setName(I18N.getMessage("MToolbar"));
    jToolBarOpenFile.setIcon(openIcon);
    jToolBarOpenFile.addActionListener(this);
    jToolBarOpenFile.setToolTipText(I18N.getMessage("OpenFile") + "...");
    jToolBarOpenFile.setActionCommand("File...");
    jToolBarHelp.setIcon(helpIcon);
    jToolBarHelp.addActionListener(this);
    jToolBarHelp.setToolTipText(I18N.getMessage("Help"));
    jToolBarHelp.setActionCommand("Help");
    JButton jToolBarInternetOpen = new JButton();
    jToolBarInternetOpen.setToolTipText(I18N.getMessage("OpenInternet"));
    jToolBarInternetOpen.setIcon(internetIcon);
    jToolBarInternetOpen.addActionListener(this);
    jToolBarInternetOpen.setActionCommand("Internet");
    jToolBarSaveFile.setIcon(saveIcon);
    jToolBarSaveFile.addActionListener(this);
    jToolBarSaveFile.setToolTipText(I18N.getMessage("Save"));
    jToolBarSaveFile.setActionCommand("Save");
    JButton jToolBarNewFile = new JButton();
    jToolBarNewFile.setIcon(newIcon);
    jToolBarNewFile.addActionListener(this);
    jToolBarNewFile.setToolTipText(I18N.getMessage("New"));
    jToolBarNewFile.setActionCommand("New");
    
    jToolBar.add(jToolBarNewFile);
    jToolBar.add(jToolBarOpenFile);
    jToolBar.add(jToolBarInternetOpen, null);
    jToolBarCloseFile = jToolBar.add(jCloseFile_Action);
    jToolBar.add(jToolBarSaveFile, null);
    
    jToolBar.add(jToolBarHelp);
    jToolBarCloseFile.setText(""); //an icon-only button
    jToolBarCloseFile.setToolTipText(I18N.getMessage("Close"));
    //jTBUndo.setAction(body.jTBUndo_Action);
    //jTBUndo.setEnabled(false);
    jTBUndo.setActionCommand("Undo");
    jTBUndo.addActionListener(this);
    jTBUndo.setText("");
    jTBUndo.setToolTipText(I18N.getMessage("Undo"));
    jTBUndo.setIcon(undoIcon);
    //jTBRedo.setAction(body.jTBRedo_Action);
    //jTBRedo.setEnabled(false);
    jTBRedo.setActionCommand("Redo");
    jTBRedo.addActionListener(this);
    jTBRedo.setText("");
    jTBRedo.setToolTipText(I18N.getMessage("Redo"));
    jTBRedo.setIcon(redoIcon);
    jToolBar.addSeparator();
    jToolBar.add(jTBUndo);
    jToolBar.add(jTBRedo);
    jToolBar.addSeparator();
    jToolBar.add(jTBWindow);
    //		System.out.println("OK");
  }
  /**
   * Selecting new language from the dialog box. It checking if there is any new language file.
   */
  void languageSelect()
  {
    Vector possible = I18N.getAvailableLocales();
    String[] possibilities = new String[possible.size()];
    for (int i = 0; i < possibilities.length; i++)
      possibilities[i] = new String(((Locale)possible.elementAt(i)).getDisplayName());
    String s =
    (String)JOptionPane.showInputDialog(
    this,
    I18N.getMessage("QSelectLanguage"),
    I18N.getMessage("LanguageSelection"),
    JOptionPane.PLAIN_MESSAGE,
    null,
    possibilities,
    possibilities[0]);
    if (s != null)
    {
      s = s.trim();
      if (s.length() > 0)
      {
        int a = 0;
        while (!possibilities[a].equalsIgnoreCase(s))
          a++;
        I18N.setLocale((Locale)possible.elementAt(a));
        this.setLocale((Locale)possible.elementAt(a));
        language = ((Locale)possible.elementAt(a)).getLanguage();
        country = ((Locale)possible.elementAt(a)).getCountry();
        messages.append(
        I18N.getMessage("MainFrame"),
        I18N.getMessage("LanguageWasChanged") + ": " + s + ".",
        ErrManager.ER_HINT);
        return;
      }
    }
  }
  /**
   * Loads the internationalization class.
   */
  void loadI18N()
  {
    System.out.println("Loading internationalization...");
    language = settings.getProperty("fred.language", language);
    country = settings.getProperty("fred.country", country);
    I18N i18n = new I18N(language, country);
    this.setLocale(new Locale(language, country));
    Title = I18N.getMessage("FredTitle");
    this.setTitle(Title);
    System.out.println(I18N.getMessage("I18NLoaded"));
  }
  private void createBody()
  {
    body = new BodyFrame(framstick);
    wBody = new WeakReference(body);
    bodyV.addElement(wBody);
    jDesktop.add(body);
    body.addInternalFrameListener(this);
    body.registerKeyboardAction(this, "ZoomIN", KeyStroke.getKeyStroke('+'), JInternalFrame.WHEN_FOCUSED);
    body.registerKeyboardAction(this, "ZoomOUT", KeyStroke.getKeyStroke('-'), JInternalFrame.WHEN_FOCUSED);
    body.registerKeyboardAction(this, "Delete", KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), JInternalFrame.WHEN_FOCUSED);
    body.registerKeyboardAction(this, "Props", KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JInternalFrame.WHEN_FOCUSED);
    body.tazo.addMouseListener(this);
    body.setTitle(I18N.getMessage("Body"));
    body.setSize(
    Integer.parseInt(settings.getProperty("body.sizeX", String.valueOf(400))),
    Integer.parseInt(settings.getProperty("body.sizeY", String.valueOf(400))));
    body.setLocation(
    Integer.parseInt(settings.getProperty("body.locX", String.valueOf(0))),
    Integer.parseInt(settings.getProperty("body.locY", String.valueOf(0))));
    try
    {
      body.setMaximum(Boolean.valueOf(settings.getProperty("body.maximized", "false")).booleanValue());
    }
    catch (PropertyVetoException e)
    {}
    if (BWires.isSelected())
      body.setWires();
    if (!BBox.isSelected())
      body.setBox();
    if (BBack.isSelected())
      body.setBackground();
    if (BDet.isSelected())
      body.setDetails();
    //body = null;
  }
  private void createBrain()
  {
    brain = new BrainFrame(framstick);
    wBrain = new WeakReference(brain);
    brainV.addElement(wBrain);
    brain.registerKeyboardAction(this, "ZoomIN", KeyStroke.getKeyStroke('+'), JInternalFrame.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    brain.registerKeyboardAction(this, "ZoomOUT", KeyStroke.getKeyStroke('-'), JInternalFrame.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    brain.registerKeyboardAction(this, "Delete", KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), JInternalFrame.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    brain.registerKeyboardAction(this, "Props", KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JInternalFrame.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    brain.addMouseListener(this);
    jDesktop.add(brain);
    brain.addInternalFrameListener(this);
    brain.setTitle(I18N.getMessage("Brain"));
    brain.setSize(
    Integer.parseInt(settings.getProperty("brain.sizeX", String.valueOf(400))),
    Integer.parseInt(settings.getProperty("brain.sizeY", String.valueOf(400))));
    brain.setLocation(
    Integer.parseInt(settings.getProperty("brain.locX", String.valueOf(this.getLocation().x + this.getSize().width - brain.getSize().width - 50))),
    Integer.parseInt(settings.getProperty("brain.locY", String.valueOf(0))));
    brain.setView(Integer.parseInt(settings.getProperty("brain.view")));
    try
    {
      brain.setMaximum(Boolean.valueOf(settings.getProperty("brain.maximized", "false")).booleanValue());
    }
    catch (PropertyVetoException e)
    {}
    //brain = null;
  }
  private void createEditor()
  {
    //    System.out.println("Making Editor window...");
    editor = new F0Editor(framstick);
    editor.errManager = err;
    editor.pack();
    editor.initDocument();
    editor.refreshDocument();
    jDesktop.add(editor);
    editor.addInternalFrameListener(this);
    editor.setSize(
    Integer.parseInt(settings.getProperty("editor.sizeX", String.valueOf(400))),
    Integer.parseInt(settings.getProperty("editor.sizeY", String.valueOf(400))));
    editor.setLocation(
    Integer.parseInt(settings.getProperty("editor.locX", String.valueOf(0))),
    Integer.parseInt(settings.getProperty("editor.locY", String.valueOf(0))));
    try
    {
      editor.setMaximum(Boolean.valueOf(settings.getProperty("editor.maximized", "false")).booleanValue());
    }
    catch (PropertyVetoException e)
    {}
    editor.show();
    jMenuWindowEditor.setSelected(true);
    jTBWindowEditor.setSelected(true);
    editor.setTitle(I18N.getMessage("GEditor"));
    //    System.out.println("...editor window ready.");
  }
  private void createMessages()
  {
    //    System.out.print("Making Messages window...");
    messages = new Messages();
    messages.setPreferredSize(new Dimension(500, 47));
    jDesktop.add(messages);
    messages.setTitle(I18N.getMessage("Messages"));
    messages.setSize(
    Integer.parseInt(settings.getProperty("messages.sizeX", String.valueOf(this.getSize().width - jTBWindow.getSize().width - 50))),
    Integer.parseInt(settings.getProperty("messages.sizeY", String.valueOf(120))));
    messages.setLocation(
    Integer.parseInt(settings.getProperty("messages.locX", String.valueOf(jDesktop.getLocation().x + jTBWindow.getLocation().x))),
    Integer.parseInt(
    settings.getProperty("messages.locY", String.valueOf(jDesktop.getLocation().y + this.getSize().height - 2 * messages.getSize().height + 20))));
    try
    {
      messages.setMaximum(Boolean.valueOf(settings.getProperty("messages.maximized", "false")).booleanValue());
    }
    catch (PropertyVetoException e)
    {}
    messages.addInternalFrameListener(this);
    //   System.out.println("OK");
  }
  private void createModel()
  {
    //    System.out.println("Making Model window...");
    model = new Model(framstick);
    jDesktop.add(model);
    model.setSize(
    Integer.parseInt(settings.getProperty("model.sizeX", String.valueOf(400))),
    Integer.parseInt(settings.getProperty("model.sizeY", String.valueOf(400))));
    model.setLocation(
    Integer.parseInt(settings.getProperty("model.locX", String.valueOf(0))),
    Integer.parseInt(settings.getProperty("model.locY", String.valueOf(0))));
    try
    {
      model.setMaximum(Boolean.valueOf(settings.getProperty("model.maximized", "false")).booleanValue());
    }
    catch (PropertyVetoException e)
    {}
    model.show();
    jMenuWindowModel.setSelected(true);
    jTBWindowModel.setSelected(true);
    model.addInternalFrameListener(this);
    //    System.out.println("...model window ready.");
  }
  /**
   * Loads settings from file.
   */
  private void loadSettings()
  {
    System.out.println("Loading settings...");
    File f = new File(settingsfilename);
    FileInputStream in = null;
    try
    {
      in = new FileInputStream(settingsfilename);
      try
      {
        settings.load(in);
        in.close();
      }
      catch (IOException e1)
      {
        System.out.println("Error loading Fred settings: " + e1.getLocalizedMessage());
      }
    }
    catch (FileNotFoundException e)
    {
      System.out.println("Cannot find " + e.getLocalizedMessage());
      System.out.println("Trying to load from a JAR file...");
      URL url = ClassLoader.getSystemResource(settingsfilename);
      if (url != null)
      {
        JarURLConnection uc = null;
        try
        {
          try
          {
            uc = (JarURLConnection)url.openConnection();
          }
          catch (IOException e2)
          {
            System.err.println(e2.getLocalizedMessage());
          }
          
          try
          {
            settings.load(uc.getInputStream());
          }
          catch (IOException e1)
          {
            System.err.println("Error loading Fred settings: " + e1.getLocalizedMessage());
          }
        }
        catch (ClassCastException e2)
        {
          System.out.println("Error: NO SETTINGS FOUND");
        }
      }
    }
    
    if (settings.getProperty("fred.lookandfeel") == null) writeSettings(); //zainicjuj sensowne domyslne wartosci!
  }
  /******************************************************************************************
   **************************************** INITING ************************************************
   *******************************************************************************************
   * Component initialization.
   */
  private void jbInit()
  {
    System.out.println("---Initializing Fred---");
    loadSettings();
    loadI18N();
    LoadWindow lw = new LoadWindow(I18N.getMessage("FredStartTitle"), I18N.getMessage("FredStart"));
    lw.start();
    
    contentPane = (JPanel)this.getContentPane();
    contentPane.setLayout(new BorderLayout());
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setContentPane(contentPane);
    this.setSize(d);
    
    jDesktop.setBorder(null);
    jDesktop.setLayout(new MyDesktopLayout());
    createMessages();
    initJGDK();
    makeFileMenu();
    makeEditMenu();
    makeWindowMenu();
    makeViewMenu();
    makeHelpMenu();
    makeMainToolbar();
    makeWindowsToolbar();
    
    this.setJMenuBar(jMenuBar1);
    contentPane.add(jToolBar, BorderLayout.NORTH);
    contentPane.add(statusBar, BorderLayout.SOUTH);
    contentPane.add(jDesktop, BorderLayout.CENTER);
    statusBar.setVisible(true);
    
    this.setTitle(this.Title + " - " + ((dir != null) ? (dir + separator + filename) : "") + " (" + framstick.name + ")");
    
    //    System.out.println("Updating look and feel...");
    currentLookAndFeel = settings.getProperty("fred.lookandfeel", currentLookAndFeel);
    updateLookAndFeel();
    if (currentLookAndFeel.equalsIgnoreCase(metal))
      jMenuWindowLaFmetal.setSelected(true);
    else if (currentLookAndFeel.equalsIgnoreCase(windows))
      jMenuWindowLaFwindows.setSelected(true);
    else if (currentLookAndFeel.equalsIgnoreCase(mac))
      jMenuWindowLaFmac.setSelected(true);
    else if (currentLookAndFeel.equalsIgnoreCase(motif))
      jMenuWindowLaFmotif.setSelected(true);
    //    System.out.println("...Look and feel updated.");
    lastfile = new File(settings.getProperty("fred.lastfile", ""));
    
    //      System.out.println("Making Body window...");
    createBody();
    ((BodyFrame)wBody.get()).show();
    jMenuWindowBody.setSelected(true);
    jTBWindowBody.setSelected(true);
    //      System.out.println("...making Body window ended.");
    
    //      System.out.println("Making Brain window...");
    createBrain();
    ((BrainFrame)wBrain.get()).show();
    jMenuWindowBrain.setSelected(true);
    jTBWindowBrain.setSelected(true);
    //      System.out.println("...making Brain window ended.");
    
    messages.show();
    jMenuWindowMessages.setSelected(true);
    jTBWindowMessages.setSelected(true);
    
    if (Boolean.valueOf(settings.getProperty("fred.model.open")).booleanValue())
    {
      createModel();
    }
    
    if (Boolean.valueOf(settings.getProperty("fred.editor.open", Boolean.toString(false))).booleanValue())
    {
      createEditor();
    }
    
    if (Boolean.valueOf(settings.getProperty("fred.maintoolbar.open", Boolean.toString(jToolBar.isVisible()))).booleanValue())
    {
      jMenuWindowBToolbar.setSelected(true);
      jToolBar.setVisible(true);
    }
    else
      jToolBar.setVisible(false);
    if (Boolean.valueOf(settings.getProperty("fred.windowtoolbar.open", Boolean.toString(jTBWindow.isVisible()))).booleanValue())
    {
      jMenuWindowWToolbar.setSelected(true);
      jTBWindow.setVisible(true);
    }
    else
      jTBWindow.setVisible(false);
    
    if (wBrain.get() != null)
      ((BrainFrame)wBrain.get()).setView(Integer.parseInt(settings.getProperty("brain.view", "1")));
    
    int sizex = Integer.parseInt(settings.getProperty("fred.sizeX"));
    int sizey = Integer.parseInt(settings.getProperty("fred.sizeY"));
    this.setSize((sizex>100)? sizex : d.width, (sizey>100)? sizey : d.height);
    
    sizex = Integer.parseInt(settings.getProperty("fred.locX"));
    sizey = Integer.parseInt(settings.getProperty("fred.locY"));
    this.setLocation(sizex,sizey);
    lw.destroy();
    //    System.gc();
    System.out.println(I18N.getMessage("FredInitCompl"));
  }
  //////////////////////end of INIT
  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e)
  {
    if (e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      if (fileChanged)
      {
        int n = JOptionPane.showConfirmDialog( this,
        I18N.getMessage("NotSavedSave"),
        I18N.getMessage("Closing"),
        JOptionPane.YES_NO_CANCEL_OPTION);
        if (n == JOptionPane.YES_OPTION)  save();
        if (n == JOptionPane.CANCEL_OPTION)  return;
      }
      writeSettings();
      System.exit(0);
    } else super.processWindowEvent(e);
  }
  /**
   *
   */
  void enableMenu()
  {
    jMenuWindow.setEnabled(true);
    jMenuEdit.setEnabled(true);
    jMenuView.setEnabled(true);
    jTBWindowBody.setEnabled(true);
    jTBWindowBrain.setEnabled(true);
    jTBWindowEditor.setEnabled(true);
    jTBWindowModel.setEnabled(true);
    jTBWindowMessages.setEnabled(true);
    jCloseFile_Action.setEnabled(true);
    jToolBarSaveFile.setEnabled(true);
    jMenuFileSave.setEnabled(true);
    jMenuFileSaveAs.setEnabled(true);
    info.setEnabled(true);
  }
  /**
   * Opens "open file" dialog and reads file selected.
   */
  void jMenuFileOpenFile_actionPerformed()
  {
    if (framstick == null)
      framstick = new Framstick();
    if (fileChanged)
    {
      int n =
      JOptionPane.showConfirmDialog(
      this,
      I18N.getMessage("NotSavedSave"),
      I18N.getMessage("OpenFile"),
      JOptionPane.YES_NO_CANCEL_OPTION);
      if (n == JOptionPane.YES_OPTION)
      {
        save();
      }
      else if (n == JOptionPane.NO_OPTION)
      {}
      else
      {
        return;
      }
    }
    JFileChooser c = new JFileChooser();
    GenFileFilter genFF = new GenFileFilter();
    c.addChoosableFileFilter(genFF);
    c.setDragEnabled(true);
    if ((dir != null) && (filename != null) && (!filename.equalsIgnoreCase("")))
    {
      lastfile = new File(dir + separator + filename);
    }
    if (lastfile != null)
    {
      c.setCurrentDirectory(lastfile);
      c.setSelectedFile(lastfile);
    }
    //    System.out.println(dir + separator + filename);
    int rVal = c.showOpenDialog(this);
    if (rVal == JFileChooser.APPROVE_OPTION)
    {
      filename = c.getSelectedFile().getName();
      dir = c.getCurrentDirectory().toString();
      File f=null;
      try
      {
        f = new File(dir + separator + filename);
        if (!f.exists())
        {
          messages.append(
          I18N.getMessage("MainFrame"),
          I18N.getMessage("FileDExist") + ": " + filename,
          ErrManager.ER_ERROR);
          return;
        }
        if (!f.canRead())
        {
          messages.append(
          I18N.getMessage("MainFrame"),
          I18N.getMessage("NoAccessFile") + ": " + filename,
          ErrManager.ER_ERROR);
          return;
        }
        InputStream input = new FileInputStream(f);
        int n = input.available();
        byte buf[] = new byte[n];
        if ((n = input.read(buf)) >= 0)
        {
          Object[] atr =
          { filename };
          messages.append(I18N.getMessage("Browser"), I18N.getFormated("FileRead", atr), ErrManager.ER_MESSAGE);
        }
        input.close();
        s_plik = new String(buf);
      }
      catch (IOException ex)
      {
        System.out.println("Load *.gen error:" + ex);
        messages.append(I18N.getMessage("Browser"), I18N.getMessage("LoadError"), ErrManager.ER_MESSAGE);
      }
      this.setTitle(Title + " - " + ((dir != null) ? (dir + separator + filename) : "") + " (" + framstick.name + ")");
      statusBar.setText(I18N.getMessage("BrowserOpen"));
      try
      {
        Browser browser = new Browser(this.s_plik, framstick);
        WeakReference wBrowser = new WeakReference(browser);
        browser = null;
        //        ((Browser)wBrowser.get()).setSize(600, 500);
        //        Point loc = getLocation();
        //        Dimension bSize = ((Browser)wBrowser.get()).getSize();
        //        Dimension tSize = this.getSize();
        //        ((Browser)wBrowser.get()).setLocation((tSize.width - bSize.width) / 2 + loc.x, (tSize.height - bSize.height) / 2 + loc.y);
        //        ((Browser)wBrowser.get()).setModal(true);
        //        ((Browser)wBrowser.get()).pack();
        statusBar.setText(" ");
        ((Browser)wBrowser.get()).show();
        if (((Browser)wBrowser.get()).accepted)
        {
          messages.clear();
          Object[] atr =
          { framstick.name };
          messages.append(I18N.getMessage("Browser"), I18N.getFormated("FramLoaded", atr), ErrManager.ER_MESSAGE);
          if (f!=null) lastfile=f;
        }
        else
          messages.append(
          I18N.getMessage("Browser"),
          I18N.getMessage("GenLoadCancel"),
          ErrManager.ER_MESSAGE);
        this.setTitle(this.Title + " - " + ((dir != null) ? (dir + separator + filename) : "") + " (" + framstick.name + ")");
        updateMessages();
        if (editor != null)
          editor.initDocument();
        enableMenu();
        wBrowser = null;
        //        System.gc();
        fileChanged = false;
      }
      catch (OutOfMemoryError e)
      {
        showMemoryDialog();
      }
    }
    if (rVal == JFileChooser.CANCEL_OPTION)
    {
      statusBar.setText(" ");
      this.setTitle(Title + " - " + ((dir != null) ? (dir + separator + filename) : "") + " (" + framstick.name + ")");
    }
    statusBar.setText(" ");
  }
  /**
   * Reads file with path = <CODE>dir/filename</CODE>.
   */
  private void readFile()
  {
    try
    {
      File f = new File(dir + separator + filename);
      if (!f.exists())
      {
        messages.append(
        I18N.getMessage("MainFrame"),
        I18N.getMessage("FileDExist") + ": " + filename,
        ErrManager.ER_ERROR);
        return;
      }
      if (!f.canRead())
      {
        messages.append(
        I18N.getMessage("MainFrame"),
        I18N.getMessage("NoAccessFile") + ": " + filename,
        ErrManager.ER_ERROR);
        return;
      }
      InputStream input = new FileInputStream(f);
      int n = input.available();
      byte buf[] = new byte[n];
      if ((n = input.read(buf)) >= 0)
      {
        Object[] atr =
        { filename };
        messages.append(I18N.getMessage("Browser"), I18N.getFormated("FileRead", atr), ErrManager.ER_MESSAGE);
      }
      input.close();
      s_plik = new String(buf);
    }
    catch (IOException ex)
    {
      System.out.println("Load *.gen error:" + ex);
      messages.append(I18N.getMessage("Browser"), I18N.getMessage("LoadError"), ErrManager.ER_MESSAGE);
    }
  }
  /**
   *
   * @param outputFile
   */
  private void writeFile(File outputFile)
  {
    String s = s_plik + "\n";
    if (outputFile.exists())
    {
      Object[] options =
      { I18N.getMessage("Overwrite"), I18N.getMessage("Append"), I18N.getMessage("Cancel")};
      int n =
      JOptionPane.showOptionDialog(
      this,
      I18N.getMessage("FileExistDoYou"),
      I18N.getMessage("FileExist"),
      JOptionPane.YES_NO_CANCEL_OPTION,
      JOptionPane.QUESTION_MESSAGE,
      null,
      options,
      options[2]);
      if (n == JOptionPane.YES_OPTION)
      {
        s = "";
      }
      else if (n == JOptionPane.NO_OPTION)
      {
        readFile();
        s = s_plik + "\n";
      }
      else if (n == JOptionPane.CANCEL_OPTION)
      {
        return;
      }
      else
      {
        return;
      }
      if (!(outputFile.canWrite()))
      {
        JOptionPane.showMessageDialog(
        this,
        I18N.getMessage("FileIsReadOnly"),
        I18N.getMessage("ReadOnlyFile"),
        JOptionPane.ERROR_MESSAGE);
        return;
      }
    }
    if ((framstick.name.compareToIgnoreCase("") == 0) || (framstick.name.compareToIgnoreCase("Noname") == 0))
      framstick.name = JOptionPane.showInputDialog(this, I18N.getMessage("EnterGenName"), I18N.getMessage("Unnamed"));
    Object[] atr =
    { filename };
    try
    {
      FileWriter out = new FileWriter(outputFile);
      out.write(s + framstick.toString());
      out.close();
      messages.append(I18N.getMessage("MainFrame"), I18N.getFormated("FileSaved", atr), ErrManager.ER_MESSAGE);
      this.setTitle(Title + " - " + outputFile + " (" + framstick.name + ")");
      fileChanged = false;
    }
    catch (IOException e2)
    {
      messages.append(I18N.getMessage("MainFrame"), I18N.getFormated("FileNotSaved", atr), ErrManager.ER_MESSAGE);
      System.out.println("ERROR:" + e2.getLocalizedMessage());
    }
  }
  /**
   * save the framstick
   */
  void save()
  {
    if (filename != null)
    {
      if (filename.compareToIgnoreCase("") == 0)
      {
        saveAs();
        return;
      }
      else
      {
        File outputFile = new File(dir + separator + filename);
        writeFile(outputFile);
        return;
      }
    }
    else
      saveAs();
  }
  /**
   * Save the framstick in new file
   */
  void saveAs()
  {
    JFileChooser c = new JFileChooser();
    GenFileFilter genFF = new GenFileFilter();
    c.setDialogTitle(I18N.getMessage("SaveAs"));
    String ver = System.getProperty("java.version");
    if (ver.startsWith("1.4"))
      c.setDragEnabled(true);
    c.addChoosableFileFilter(genFF);
    if (lastfile == null)
      lastfile = new File(dir + separator + framstick.name + ".gen");
    c.setCurrentDirectory(lastfile);
    int rVal = c.showSaveDialog(this);
    if (rVal == JFileChooser.APPROVE_OPTION)
    {
      filename = c.getSelectedFile().getName();
      if (!(filename.endsWith(".gen")))
        filename += ".gen";
      dir = c.getCurrentDirectory().toString();
      File outputFile = new File(dir + separator + filename);
      writeFile(outputFile);
      return;
    }
    if (rVal == JFileChooser.CANCEL_OPTION)
    {
      return;
    }
  }
  private void updateMessages()
  {
    for (int x = 0; x < ErrManager.msg.size(); x++)
    {
      System.out.println(((Message)ErrManager.msg.get(x)).description);
      messages.append(((Message)ErrManager.msg.get(x)).source, ((Message)ErrManager.msg.get(x)).description, ((Message)ErrManager.msg.get(x)).status);
    };
    ErrManager.msg.clear();
  }
  
  private void writeSettings()
  {
    int d = (int)(Toolkit.getDefaultToolkit().getScreenSize().height/2.5);
    settings.clear();
    settings.setProperty("fred.lastfile", (lastfile!=null) ? lastfile.getAbsolutePath() : "");
    settings.setProperty("fred.lookandfeel", currentLookAndFeel);
    settings.setProperty("fred.language", (language != null) ? language : "");
    settings.setProperty("fred.country", (country != null) ? country : "");
    settings.setProperty("fred.model.open", (model != null) ? Boolean.toString(model.isVisible()) : Boolean.toString(false));
    settings.setProperty("fred.editor.open", (editor != null) ? Boolean.toString(editor.isVisible()) : Boolean.toString(false));
    settings.setProperty("fred.maintoolbar.open", Boolean.toString(jToolBar.isVisible()));
    settings.setProperty("fred.windowtoolbar.open", Boolean.toString(jTBWindow.isVisible()));
    
    settings.setProperty("body.locX", (wBody.get() != null) ? String.valueOf(((BodyFrame)wBody.get()).getLocation().x) : String.valueOf(0));
    settings.setProperty("body.locY", (wBody.get() != null) ? String.valueOf(((BodyFrame)wBody.get()).getLocation().y) : String.valueOf(0));
    settings.setProperty("brain.locX", (wBrain.get() != null) ? String.valueOf(((BrainFrame)wBrain.get()).getLocation().x) : String.valueOf(d));
    settings.setProperty("brain.locY", (wBrain.get() != null) ? String.valueOf(((BrainFrame)wBrain.get()).getLocation().y) : String.valueOf(0));
    settings.setProperty("brain.view", (wBrain.get() != null) ? String.valueOf(((BrainFrame)wBrain.get()).view) : "1");
    settings.setProperty("messages.locX", (messages != null) ? String.valueOf(messages.getLocation().x) : String.valueOf(d));
    settings.setProperty("messages.locY", (messages != null) ? String.valueOf(messages.getLocation().y) : String.valueOf(d));
    settings.setProperty("model.locX", (model != null) ? String.valueOf(model.getLocation().x) : String.valueOf(0));
    settings.setProperty("model.locY", (model != null) ? String.valueOf(model.getLocation().y) : String.valueOf(d));
    
    settings.setProperty("brain.sizeX", (wBrain.get() != null) ? String.valueOf(((BrainFrame)wBrain.get()).getSize().width) : String.valueOf(d));
    settings.setProperty("brain.sizeY", (wBrain.get() != null) ? String.valueOf(((BrainFrame)wBrain.get()).getSize().height) : String.valueOf(d));
    settings.setProperty("body.sizeX", (wBody.get() != null) ? String.valueOf(((BodyFrame)wBody.get()).getSize().width) : String.valueOf(d));
    settings.setProperty("body.sizeY", (wBody.get() != null) ? String.valueOf(((BodyFrame)wBody.get()).getSize().height) : String.valueOf(d));
    settings.setProperty("messages.sizeX", (messages != null) ? String.valueOf(messages.getSize().width) : String.valueOf(d));
    settings.setProperty("messages.sizeY", (messages != null) ? String.valueOf(messages.getSize().height) : String.valueOf(d));
    settings.setProperty("model.sizeX", (model != null) ? String.valueOf(model.getSize().width) : String.valueOf(d));
    settings.setProperty("model.sizeY", (model != null) ? String.valueOf(model.getSize().height) : String.valueOf(d));
    
    settings.setProperty("body.wires", Boolean.toString(BWires.isSelected()));
    settings.setProperty("body.box", Boolean.toString(BBox.isSelected()));
    settings.setProperty("body.background", Boolean.toString(BBack.isSelected()));
    settings.setProperty("body.details", Boolean.toString(BDet.isSelected()));
    settings.setProperty("body.NPanel", (panel != null) ? Boolean.toString(panel.isSelected()) : "false");
    settings.setProperty("brain.network", (wBrain.get() != null) ? Boolean.toString(((BrainFrame)wBrain.get()).network.isSelected()) : Boolean.toString(false));
    settings.setProperty("body.maximized", (wBody.get() != null) ? Boolean.toString(((BodyFrame)wBody.get()).isMaximum()) : Boolean.toString(false));
    settings.setProperty("brain.maximized", (wBrain.get() != null) ? Boolean.toString(((BrainFrame)wBrain.get()).isMaximum()) : Boolean.toString(false));
    settings.setProperty("messages.maximized", (messages != null) ? Boolean.toString(messages.isMaximum()) : Boolean.toString(false));
    settings.setProperty("model.maximized", (model != null) ? Boolean.toString(model.isMaximum()) : Boolean.toString(false));
    settings.setProperty("editor.maximized", (editor != null) ? Boolean.toString(editor.isMaximum()) : Boolean.toString(false));
    
    settings.setProperty("fred.locX", String.valueOf(Math.max(0,this.getLocation().x)));
    settings.setProperty("fred.locY", String.valueOf(Math.max(0,this.getLocation().y)));
    settings.setProperty("fred.sizeX", String.valueOf(this.getSize().width));
    settings.setProperty("fred.sizeY", String.valueOf(this.getSize().height));
    
    FileOutputStream out = null;
    File file = new File(settingsfilename);
    System.out.println("Saving "+file);
    try
    {
      out = new FileOutputStream(file);
    }
    catch (FileNotFoundException e1)
    {
      messages.append(I18N.getMessage("MainFrame"), I18N.getMessage("SettingsNotFound"), ErrManager.ER_HINT);
      System.out.println("Error:settings file was not found: " + e1.getLocalizedMessage());
    }
    try
    {
      settings.store(
      out,
      I18N.getMessage("FredSettings")
      + "\t\n# "
      + I18N.getMessage("PleaseNotModify"));
      out.close();
    }
    catch (IOException e3)
    {
      System.out.println("Error while storing Fred settings: " + e3);
    }
  }
  /**
   * <p>Method actionPerformed.</P>
   * Doing all action connected with all menus and buttons.
   * @param ActionEvent e
   */
  public void actionPerformed(ActionEvent e)
  {
    //MenuFileNew
    if (e.getActionCommand().compareToIgnoreCase("New") == 0)
    {
      if (fileChanged)
      {
        int n =
        JOptionPane.showConfirmDialog(
        this,
        I18N.getMessage("FileNSavedDo"),
        I18N.getMessage("New") + " " + I18N.getMessage("File"),
        JOptionPane.YES_NO_CANCEL_OPTION);
        if (n == JOptionPane.YES_OPTION)
        {
          save();
        }
        else if (n == JOptionPane.NO_OPTION)
        {
          ;
        }
        else
        {
          return;
        }
      }
      statusBar.setText(I18N.getMessage("MakingFram"));
      if (framstick == null)
        framstick = new Framstick();
      
      framstick.author = "Unknown";
      framstick.genotype = "//0\np:\n";
      framstick.info = "None";
      framstick.name = "Noname";
      framstick.rebuildModel();
      framstick.fireMarkChangedEvent();
      if (wBody.get() != null)
        ((BodyFrame)wBody.get()).setVisible(true);
      this.setTitle(this.Title + " - " + ((dir != null) ? (dir + separator + filename) : "") + " (" + framstick.name + ")");
      enableMenu();
      statusBar.setText(" ");
      messages.append(
      I18N.getMessage("MainFrame"),
      I18N.getMessage("NewFramCreated"),
      ErrManager.ER_MESSAGE);
    }
    //Exit
    else if (e.getActionCommand().compareToIgnoreCase("Exit") == 0)
    {
      if (fileChanged)
      {
        int n = JOptionPane.showConfirmDialog( this,
        I18N.getMessage("NotSavedSave"),
        I18N.getMessage("Closing"),
        JOptionPane.YES_NO_CANCEL_OPTION);
        if (n == JOptionPane.YES_OPTION)  save();
        if (n == JOptionPane.CANCEL_OPTION)  return;
      }
      writeSettings();
      System.exit(0);
    }
    //Help | About action performed
    else if (e.getActionCommand().compareToIgnoreCase("About...") == 0)
    {
      MainFrame_AboutBox dlg = new MainFrame_AboutBox(this);
      Dimension dlgSize = dlg.getPreferredSize();
      Dimension frmSize = this.getSize();
      Point loc = this.getLocation();
      dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
      dlg.setModal(true);
      dlg.pack();
      dlg.show();
    }
    //
    else if (e.getActionCommand().compareToIgnoreCase("File...") == 0)
      jMenuFileOpenFile_actionPerformed();
    else if (e.getActionCommand().compareToIgnoreCase("AddBrain") == 0)
    {
      try
      {
        createBrain();
        ((BrainFrame)wBrain.get()).show();
      }
      catch (OutOfMemoryError e1)
      {
        showMemoryDialog();
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("Brain") == 0)
    {
      if (wBrain.get() == null)
      {
        statusBar.setText(I18N.getMessage("ShowBrain"));
        createBrain();
        ((BrainFrame)wBrain.get()).show();
      }
      else if (((BrainFrame)wBrain.get()).isVisible())
      {
        if (brainV.size() > 1)
          ((BrainFrame)wBrain.get()).dispose();
        else
          ((BrainFrame)wBrain.get()).setVisible(false);
      }
      else
      {
        if (((BrainFrame)wBrain.get()).isClosed())
        {
          statusBar.setText(I18N.getMessage("ShowBrain"));
          createBrain();
          ((BrainFrame)wBrain.get()).show();
        }
        else
        {
          ((BrainFrame)wBrain.get()).show();
          ((BrainFrame)wBrain.get()).setView(Integer.parseInt(settings.getProperty("brain.view", "1")));
        }
      }
      if (wBrain.get() != null)
      {
        jTBWindowBrain.setSelected(((BrainFrame)wBrain.get()).isVisible());
        jMenuWindowBrain.setSelected(((BrainFrame)wBrain.get()).isVisible());
      }
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("Messages") == 0)
    {
      if (messages == null)
      {
        statusBar.setText(I18N.getMessage("ShowMsg"));
        createMessages();
        messages.show();
      }
      else if (messages.isVisible())
        messages.setVisible(false);
      else
      {
        if (messages.isClosed())
        {
          statusBar.setText(I18N.getMessage("ShowMsg"));
          createMessages();
          messages.show();
        }
        else
          messages.show();
      }
      jTBWindowMessages.setSelected(messages.isVisible());
      jMenuWindowMessages.setSelected(messages.isVisible());
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("Model") == 0)
    {
      if (model == null)
      {
        statusBar.setText(I18N.getMessage("ShowModel"));
        createModel();
        model.show();
      }
      else if (model.isVisible())
        model.setVisible(false);
      else
      {
        if (model.isClosed())
        {
          statusBar.setText(I18N.getMessage("ShowModel"));
          createModel();
          model.show();
        }
        else
          model.show();
      }
      jTBWindowModel.setSelected(model.isVisible());
      jMenuWindowModel.setSelected(model.isVisible());
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("Editor") == 0)
    {
      if (editor == null)
      {
        statusBar.setText(I18N.getMessage("ShowModel"));
        createEditor();
        editor.show();
      }
      else if (editor.isVisible())
        editor.setVisible(false);
      else
      {
        if (editor.isClosed())
        {
          statusBar.setText(I18N.getMessage("ShowModel"));
          createEditor();
          editor.show();
        }
        else
          editor.show();
      }
      jTBWindowEditor.setSelected(editor.isVisible());
      jMenuWindowEditor.setSelected(editor.isVisible());
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("AddBody") == 0)
    {
      try
      {
        createBody();
        ((BodyFrame)wBody.get()).show();
      }
      catch (OutOfMemoryError e1)
      {
        showMemoryDialog();
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("Body") == 0)
    {
      if (wBody.get() == null)
      {
        statusBar.setText(I18N.getMessage("ShowBody"));
        createBody();
        ((BodyFrame)wBody.get()).show();
      }
      else if (((BodyFrame)wBody.get()).isVisible())
      {
        if (bodyV.size() > 1)
          ((BodyFrame)wBody.get()).dispose();
        else
          ((BodyFrame)wBody.get()).setVisible(false);
      }
      else
      {
        if (((BodyFrame)wBody.get()).isClosed())
        {
          statusBar.setText(I18N.getMessage("ShowBody"));
          createBody();
          ((BodyFrame)wBody.get()).show();
        }
        else
        {
          ((BodyFrame)wBody.get()).show();
        }
      }
      if (wBody.get() != null)
      {
        jTBWindowBody.setSelected(((BodyFrame)wBody.get()).isVisible());
        jMenuWindowBody.setSelected(((BodyFrame)wBody.get()).isVisible());
      }
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("Windows toolbar") == 0)
      jTBWindow.setVisible(!jTBWindow.isVisible());
    else if (e.getActionCommand().compareToIgnoreCase("Basic toolbar") == 0)
      jToolBar.setVisible(!this.jToolBar.isVisible());
    else if (e.getActionCommand().compareToIgnoreCase("Help") == 0)
    {
      if (help == null)
      {
        statusBar.setText(I18N.getMessage("ShowHelp"));
        help = new Help();
        this.jDesktop.add(help);
        help.show();
      }
      if (help.isClosed())
      {
        statusBar.setText(I18N.getMessage("ShowHelp"));
        this.jDesktop.remove(help);
        help = new Help();
        this.jDesktop.add(help);
        help.show();
      }
      else
        help.show();
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("Internet") == 0)
    {
      if (fileChanged)
      {
        int n =
        JOptionPane.showConfirmDialog(
        this,
        I18N.getMessage("FileNSavedDo"),
        I18N.getMessage("New") + " " + I18N.getMessage("File"),
        JOptionPane.YES_NO_CANCEL_OPTION);
        if (n == JOptionPane.YES_OPTION)  save();  else if (n == JOptionPane.CANCEL_OPTION)  return;
      }
      statusBar.setText(I18N.getMessage("ShowInternetD"));
      if (framstick == null)
        framstick = new Framstick();
      URLGens uc = new URLGens(framstick);
      WeakReference wuc = new WeakReference(uc);
      uc = null;
      ((URLGens)wuc.get()).setModal(true);
      ((URLGens)wuc.get()).show();
      wuc = null;
      fileChanged = false;
      
      
      this.setTitle(this.Title + " - (Unnamed)");
      enableMenu();
      //      System.gc();
      statusBar.setText(" ");
    }
    else if (e.getActionCommand().compareToIgnoreCase("SaveAs") == 0)
    {
      saveAs();
    }
    else if (e.getActionCommand().compareToIgnoreCase("metal") == 0)
    {
      currentLookAndFeel = metal;
      updateLookAndFeel();
    }
    else if (e.getActionCommand().compareToIgnoreCase("mac") == 0)
    {
      currentLookAndFeel = mac;
      updateLookAndFeel();
    }
    else if (e.getActionCommand().compareToIgnoreCase("motif") == 0)
    {
      currentLookAndFeel = motif;
      updateLookAndFeel();
    }
    else if (e.getActionCommand().compareToIgnoreCase("windows") == 0)
    {
      currentLookAndFeel = windows;
      updateLookAndFeel();
    }
    else if (e.getActionCommand().compareToIgnoreCase("ZoomIN") == 0)
    {
      //      System.out.println(">>ZoomIN");
      switch (activeWindow)
      {
        case 1 :
          ((BodyFrame)wBody.get()).jTBZoomIn_Action.actionPerformed(null);
          break;
        case 2 :
          ((BrainFrame)wBrain.get()).jZoomIn_Action.actionPerformed(null);
          break;
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("ZoomOUT") == 0)
    {
      //      System.out.println(">>ZoomOUT");
      switch (activeWindow)
      {
        case 1 :
          ((BodyFrame)wBody.get()).jTBZoomOut_Action.actionPerformed(null);
          break;
        case 2 :
          ((BrainFrame)wBrain.get()).jZoomOut_Action.actionPerformed(null);
          break;
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("Props") == 0)
    {
      //      System.out.println(">>Properties");
      switch (activeWindow)
      {
        case 1 :
          wProp = ((BodyFrame)wBody.get()).showProp();
          if (wProp.get() != null)
          {
            propsV.addElement(wProp);
            jDesktop.add(((Prop)wProp.get()));
            ((Prop)wProp.get()).show();
          }
          break;
        case 2 :
          wProp = ((BrainFrame)wBrain.get()).showProp();
          if (wProp.get() != null)
          {
            propsV.addElement(wProp);
            jDesktop.add(((Prop)wProp.get()));
            ((Prop)wProp.get()).show();
          }
          break;
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("Delete") == 0)
    {
      //      System.out.println(">>delete");
      switch (activeWindow)
      {
        case 1 :
          ((BodyFrame)wBody.get()).jTBDelete_Action.actionPerformed(null);
          ;
          break;
        case 2 :
          ((BrainFrame)wBrain.get()).jDelN_Action.actionPerformed(null);
          break;
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("panel") == 0)
    {
      if (wBody.get() != null)
        ((BodyFrame)wBody.get()).neuroPanel.setVisible(this.panel.isSelected());
      else
        panel.setSelected(false);
    }
    else if (e.getActionCommand().compareToIgnoreCase("Save") == 0)
    {
      save();
    }
    else if (e.getActionCommand().compareToIgnoreCase("Info") == 0)
    {
      if (framInfo == null)
      {
        framInfo = new FramInfo(framstick);
        framInfo.setSize(300, 300);
        framInfo.setLocation(50, 50);
      }
      framInfo.show();
    }
    else if (e.getActionCommand().compareToIgnoreCase("Wires") == 0)
    {
      if (wBody.get() != null)
        ((BodyFrame)wBody.get()).setWires();
      else
        BWires.setSelected(false);
    }
    else if (e.getActionCommand().compareToIgnoreCase("Box") == 0)
    {
      if (wBody.get() != null)
        ((BodyFrame)wBody.get()).setBox();
      else
        BBox.setSelected(true);
    }
    else if (e.getActionCommand().compareToIgnoreCase("Background") == 0)
    {
      if (wBody.get() != null)
        ((BodyFrame)wBody.get()).setBackground();
      else
        BBack.setSelected(false);
    }
    else if (e.getActionCommand().compareToIgnoreCase("Details") == 0)
    {
      if (wBody.get() != null)
        ((BodyFrame)wBody.get()).setDetails();
      else
        BDet.setSelected(false);
    }
    else if (e.getActionCommand().compareToIgnoreCase("Undo") == 0)
    {
      if (editor != null)
        if (editor.isSelected())
        {
          editor.undoAction.actionPerformed(null);
          return;
        }
      if (framstick.undo())
      {
        framstick.fireSignificantEditEvent();
      }
    }
    else if (e.getActionCommand().compareToIgnoreCase("Redo") == 0)
    {
      if (editor != null)
        if (editor.isSelected())
        {
          editor.redoAction.actionPerformed(null);
          return;
        }
      if (framstick.redo())
      {
        framstick.fireSignificantEditEvent();
      }
    }
  }
  /**
   *
   */
  private void showMemoryDialog()
  {
    JOptionPane.showMessageDialog(
    this,
    "Cannot do this operation becouse of out of memory.\nPlease close some windows and try again.",
    "Out of memory error",
    JOptionPane.ERROR_MESSAGE);
  }
  /**
   * Updates all buttons and menus connected with any internal windows.
   */
  public void internalFrameActivated(InternalFrameEvent e)
  {
    updateMessages();
    if (e.getSource().getClass().isInstance(((BodyFrame)wBody.get())))
    {
      //jMenuEditUndo.setAction(body.jTBUndo_Action);
      //jMenuEditRedo.setAction(body.jTBRedo_Action);
      //jTBUndo.setAction(body.jTBUndo_Action);
      jTBUndo.setEnabled(true);
      jTBUndo.setText("");
      //jTBRedo.setAction(body.jTBRedo_Action);
      jTBRedo.setEnabled(true);
      jTBRedo.setText("");
      wBody = new WeakReference(e.getSource());
      BBox.setSelected(((BodyFrame)wBody.get()).boxSet);
      BBack.setSelected(((BodyFrame)wBody.get()).backSet);
      BDet.setSelected(((BodyFrame)wBody.get()).detSet);
      BWires.setSelected(((BodyFrame)wBody.get()).wiresSet);
      activeWindow = 1;
    }
    else if (e.getSource().getClass().isInstance(model))
    {
      //jMenuEditUndo.setAction(model.undo);
      //jMenuEditRedo.setAction(model.redo);
      //jTBUndo.setAction(model.undo);
      jTBUndo.setEnabled(true);
      jTBUndo.setText("");
      //jTBRedo.setAction(model.redo);
      jTBRedo.setEnabled(true);
      jTBRedo.setText("");
      activeWindow = 4;
    }
    else if (e.getSource().getClass().isInstance(((BrainFrame)wBrain.get())))
    {
      //jMenuEditUndo.setAction(brain.jUndo_Action);
      //jMenuEditRedo.setAction(brain.jRedo_Action);
      //jTBUndo.setAction(brain.jUndo_Action);
      jTBUndo.setEnabled(true);
      jTBUndo.setText("");
      //jTBRedo.setAction(brain.jRedo_Action);
      jTBRedo.setEnabled(true);
      jTBRedo.setText("");
      activeWindow = 2;
    }
    else if (e.getSource().equals(editor))
    {
      //jMenuEditUndo.setAction(editor.undoAction);
      //jMenuEditRedo.setAction(editor.redoAction);
      //jTBUndo.setAction(editor.undoAction);
      jTBUndo.setEnabled(editor.undoAction.isEnabled());
      jTBUndo.setIcon(undoIcon);
      //jTBRedo.setAction(editor.redoAction);
      jTBRedo.setEnabled(editor.redoAction.isEnabled());
      jTBRedo.setIcon(redoIcon);
      jMenuEditUndo.setText(I18N.getMessage("Undo"));
      jMenuEditUndo.setIcon(undoIcon);
      jMenuEditRedo.setText(I18N.getMessage("Redo"));
      jMenuEditRedo.setIcon(redoIcon);
      jMenuEditCopy.setAction(editor.getActionByName(DefaultEditorKit.copyAction));
      jMenuEditCopy.setText(I18N.getMessage("Copy"));
      jMenuEditCopy.setIcon(copyIcon);
      jMenuEditCopy.setEnabled(true);
      jMenuEditPaste.setAction(editor.getActionByName(DefaultEditorKit.pasteAction));
      jMenuEditPaste.setText(I18N.getMessage("Paste"));
      jMenuEditPaste.setIcon(pasteIcon);
      jMenuEditPaste.setEnabled(true);
      jMenuEditCut.setAction(editor.getActionByName(DefaultEditorKit.cutAction));
      jMenuEditCut.setText(I18N.getMessage("Cut"));
      jMenuEditCut.setIcon(cutIcon);
      jMenuEditCut.setEnabled(true);
      activeWindow = 3;
    }
    //System.out.println(activeWindow + "Focus:" + e.getSource().toString());
  }
  /**
   * Updates all buttons and menus connected with any internal windows.
   */
  public void internalFrameOpened(InternalFrameEvent e)
  {
    if (e.getSource().equals(((BodyFrame)wBody.get())))
    {
      jTBWindowBody.setSelected(true);
      jMenuWindowBody.setSelected(true);
    }
    else if (e.getSource().equals(((BrainFrame)wBrain.get())))
    {
      jTBWindowBrain.setSelected(true);
      jMenuWindowBrain.setSelected(true);
    }
    else if (e.getSource().equals(editor))
    {
      jTBWindowEditor.setSelected(true);
      jMenuWindowEditor.setSelected(true);
    }
    else if (e.getSource().equals(messages))
    {
      jTBWindowMessages.setSelected(true);
      jMenuWindowMessages.setSelected(true);
    }
    else if (e.getSource().equals(model))
    {
      jTBWindowModel.setSelected(true);
      jMenuWindowModel.setSelected(true);
    }
  }
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void internalFrameClosing(InternalFrameEvent e)
  {}
  /**
   * Setting variables connected with internal windows to point to another, not closed window.
   */
  public void internalFrameClosed(InternalFrameEvent e)
  {
    if (e.getSource().equals(((BodyFrame)wBody.get())))
    {
      if (bodyV.size() > 0)
      {
        bodyV.removeElementAt(bodyV.size() - 1);
      }
      jDesktop.remove(((BodyFrame)wBody.get()));
      ((BodyFrame)wBody.get()).removeInternalFrameListener(this);
      body = null;
      //System.runFinalization();
      if (bodyV.size() == 0)
      {
        jTBWindowBody.setSelected(false);
        jMenuWindowBody.setSelected(false);
      }
      else if (bodyV.size() > 0)
      {
        wBody = ((WeakReference)bodyV.elementAt(bodyV.size() - 1));
        //body = (BodyFrame)wBody.get();
      }
    }
    else if (e.getSource().equals(((BrainFrame)wBrain.get())))
    {
      if (brainV.size() > 0)
        brainV.remove(brainV.size() - 1);
      jDesktop.remove(((BrainFrame)wBrain.get()));
      brain = null;
      //System.runFinalization();
      if (brainV.size() == 0)
      {
        jTBWindowBrain.setSelected(false);
        jMenuWindowBrain.setSelected(false);
      }
      else if (brainV.size() > 0)
        wBrain = (WeakReference)brainV.elementAt(brainV.size() - 1);
    }
    else if (e.getSource().equals(editor))
    {
      jTBWindowEditor.setSelected(false);
      jMenuWindowEditor.setSelected(false);
      jDesktop.remove(editor);
      editor = null;
      //System.runFinalization();
    }
    else if (e.getSource().equals(messages))
    {
      jTBWindowMessages.setSelected(false);
      jMenuWindowMessages.setSelected(false);
      jDesktop.remove(messages);
      messages = null;
      //System.runFinalization();
    }
    else if (e.getSource().equals(model))
    {
      jTBWindowModel.setSelected(false);
      jMenuWindowModel.setSelected(false);
      jDesktop.remove(model);
      model = null;
      //System.runFinalization();
    }
    //    System.gc();
  }
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void internalFrameIconified(InternalFrameEvent e)
  {}
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void internalFrameDeiconified(InternalFrameEvent e)
  {}
  /**
   * Turn off all menus and buttons connected with internal windows.
   */
  public void internalFrameDeactivated(InternalFrameEvent e)
  {
    jMenuEditUndo.setEnabled(false);
    jMenuEditRedo.setEnabled(false);
    jTBUndo.setEnabled(false);
    jTBRedo.setEnabled(false);
    jMenuEditCopy.setEnabled(false);
    jMenuEditCut.setEnabled(false);
    jMenuEditPaste.setEnabled(false);
    jTBWindowMessages.setSelected(messages.isVisible());
    jMenuWindowMessages.setSelected(messages.isVisible());
    updateMessages();
  }
  /**
   * updating messages window, and setting <CODE>fileChanged</CODE> variable to true.
   */
  public void significantEdit(FramstickEvent framEvent)
  {
    fileChanged = true;
    updateMessages();
  }
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void markChanged(FramstickEvent framEvent)
  {}
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void mouseClicked(MouseEvent e)
  {}
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void mousePressed(MouseEvent e)
  {}
  /**
   * If it was right mouse button, shows the Properties dialog.
   */
  public void mouseReleased(MouseEvent e)
  {
    if (e.getModifiers() == MouseEvent.BUTTON3_MASK)
    {
      //System.out.print("-->prawy klaw:");
      String[] s = null;
      if (wBody.get() != null)
      {
        if (e.getSource().equals(((BodyFrame)wBody.get()).tazo))
        {
          s = ((BodyFrame)wBody.get()).tazo.showProperties(e.getX(), e.getY());
        }
      }
      if (wBrain.get() != null)
      {
        //System.out.println("Brain=");
        if ((e.getSource().getClass().isInstance(((BrainFrame)wBrain.get()).neuro))
        || (e.getSource().getClass().isInstance(((BrainFrame)wBrain.get()).names))
        || (e.getSource().getClass().isInstance(((BrainFrame)wBrain.get()).symbol)))
        {
          s = ((BrainFrame)wBrain.get()).showProperties(e.getX(), e.getY());
          //System.out.println(s);
        }
      }
      //Marcin
      if (s == null)
      {
        return;
      }
      Prop p = new Prop(framstick, s[1], Integer.parseInt(s[0]));
      wProp = new WeakReference(p);
      p = null;
      propsV.addElement(wProp);
      ((Prop)wProp.get()).setLocation(e.getX() + 50, e.getY() - 50);
      ((Prop)wProp.get()).pack();
      jDesktop.add(((Prop)wProp.get()));
      ((Prop)wProp.get()).show();
    }
  }
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void mouseEntered(MouseEvent e)
  {}
  /**
   * doing nothing, but needed becouse of interface implemented.
   */
  public void mouseExited(MouseEvent e)
  {}
}