package fred;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelListener;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.*;
import java.awt.font.*;
import java.text.*;
import java.awt.font.LineBreakMeasurer;
import javax.swing.JPanel;

import jGDK.Framstick;
import jGDK.Library;
import jGDK.Record;
import jGDK.FramstickListener;
import jGDK.FramstickEvent;
import jGDK.I18N;

//import java.awt.Button;

/**
 * Class responsible for network visualisation in names mode.
 */
public class Names extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener
{
  private FramstickListener myFL = new MyFramstickListener();
  static Library library;
  static int[] standardNeuron =
  {1,4,70,50,50,50,20,80,20,20,50,50};
  
  Framstick fram;
  float zoom = 1;
  int oldHeight = 0;	//stare rozmiary okna
  int oldWidth = 0;
  Dimension d;		//zmienna do rozmiarow okna
  boolean sortNeurons = false;	//czy sorujemy neurony
  boolean addingConnection = false; //czy jestesmy na etapie dodawania polaczenia
  public int markedNeuron = -1;	//neuronik zaznaczony :)
  String description0 = "";
  String description1 = "";
  String description2 = "";//do konstrukcji opisu
  boolean shiftDown = false;//czy wcisnieto klawisz Ctrl
  boolean controlDown = false;
  
  /**
   * Imports information about neural network, sets the position fof each neuron and the zoom value, to fit the scene size to the screen.
   */
  Names(Framstick model)
  {
    fram=model;
    fram.addFramstickListener(myFL);
    setNeuronsPos();							//poczatkowe ustawienie
    d = getSize();
    setZoomValue();								//dopasowanie do okna
    this.addMouseListener(this);
    this.addMouseMotionListener(this);
    this.addMouseWheelListener(this);
  }
  
  public void paint(Graphics g)
  {
    update(g);
  }
  
  public void update(Graphics g)
  {
    d = getSize();								//pobiera rozmiar appletu
    g.setColor(new Color(0xaaaaaa));
    g.fillRect(0,0,d.width,d.height);		//tlo i czyszczenie
    if (d.width!=oldWidth || d.height!=oldHeight)//jesli dawne rozmiary sa
    {											//inne niz aktualne
      oldWidth = d.width;						//to zapamietuje rozmiar apletu
      oldHeight = d.height;
      setZoomValue();
    }
    Font courier = new Font("Courier", Font.PLAIN, (int)(Math.floor(16*zoom)));
    g.setFont(courier);
    //Font f = new Font("Times New Roman", Font.BOLD, 14);
    //		int f = 12;
    //		g.setFontHeight(f);
    g.setColor(Color.black);				//siec rysuje na czarno
    drawNetwork(g);
    Font courier2 = new Font("Courier", Font.PLAIN, 12);
    g.setFont(courier2);
    g.setColor(new Color(0x000000ff));		//wypelnia na niebiesko
    g.fillRect(0,d.height-32,d.width,d.height);//prostokat w dole ekranu
    g.setColor(new Color(0x00ffffff));		//a na bialo wypisuje
    g.drawString(description0,10,d.height-22);//opis zaznaczonego neuronu
    g.drawString(description1,10,d.height-12);
    g.drawString(description2,10,d.height-2);
    g.setColor(Color.black);				//font znowu czarny
  }
  
  public void drawNeuron(Graphics g, float x, float y, String typ)
  {//rysuje symbol neuronu na podstawie definicji z tablicy (odpowiedni indeks-type)
    //na poczatek rameczka dla naszego neuronu
    g.drawLine((int)((10+15+x*100)*zoom),(int)((10+15+y*100)*zoom),(int)((10+85+x*100)*zoom),(int)((10+15+y*100)*zoom));
    g.drawLine((int)((10+15+x*100)*zoom),(int)((10+85+y*100)*zoom),(int)((10+85+x*100)*zoom),(int)((10+85+y*100)*zoom));
    g.drawLine((int)((10+15+x*100)*zoom),(int)((10+15+y*100)*zoom),(int)((10+15+x*100)*zoom),(int)((10+85+y*100)*zoom));
    g.drawLine((int)((10+85+x*100)*zoom),(int)((10+15+y*100)*zoom),(int)((10+85+x*100)*zoom),(int)((10+85+y*100)*zoom));
    
    //highly non-optimal code - computed each time but works for all known neuron types - mac
    Object o=Framstick.library.getRecord("neuro", typ).getInfoAbout("name");
    if (o!=null) typ=o.toString(); else typ="a neuron"; //typ contains neuron name now
    
//simple version, single-line text:
/*    
    Font f=new Font("Courier", Font.PLAIN, (int)(Math.floor(zoom*130/typ.length()))); //width estimate
    g.setFont(f);
    Rectangle2D size=g.getFontMetrics().getStringBounds(typ,g);
    g.drawString(typ, (int)((x*100+60)*zoom-size.getWidth()/2),(int)((y*100+60)*zoom+size.getHeight()/4));
*/    
    
//elaborate version, splits names into multi-line: 
    Graphics2D graphics2D = (Graphics2D) g;
    float drawPosX,drawPosY=-1,formatWidth = zoom*70; //box width
    java.util.Hashtable map = new java.util.Hashtable();
    map.put(TextAttribute.SIZE, new Float(zoom*10));
    AttributedString as = new AttributedString(typ,map);
    AttributedCharacterIterator paragraph = as.getIterator();
    LineBreakMeasurer lineMeasurer = new LineBreakMeasurer(paragraph,new FontRenderContext(null, false, false));
    lineMeasurer.setPosition(paragraph.getBeginIndex());
    while (lineMeasurer.getPosition() < paragraph.getEndIndex())
    {
      TextLayout layout = lineMeasurer.nextLayout(formatWidth);
      if (drawPosY==-1) drawPosY=(10+15+  30.0f*lineMeasurer.getPosition()/paragraph.getEndIndex()  +y*100)*zoom;
      drawPosY += layout.getAscent();
      drawPosX=(10+15+x*100)*zoom + (formatWidth-(float)layout.getBounds().getWidth())/2; //centered
      layout.draw(graphics2D, drawPosX, drawPosY);
      drawPosY += layout.getDescent() + layout.getLeading();
    }
    
    
    
    
    //previous code with just a few specific neuron types. uughh! :[
/*    if(typ.equals("@"))//OK
    {
      g.drawString("rotation",(int)((x*100+35)*zoom),(int)((y*100+55)*zoom));
      g.drawString("muscle ",(int)((x*100+35)*zoom),(int)((y*100+70)*zoom));
    }
    else if(typ.equals("Sin"))//OK
    {
      g.drawString(" sinus",(int)((x*100+34)*zoom),(int)((y*100+55)*zoom));
      g.drawString("generator",(int)((x*100+26)*zoom),(int)((y*100+70)*zoom));
    }
    else if(typ.equals("Fuzzy"))//OK
    {
      g.drawString("  fuzzy",(int)((x*100+34)*zoom),(int)((y*100+55)*zoom));
      g.drawString(" system",(int)((x*100+34)*zoom),(int)((y*100+70)*zoom));
    }
    else if(typ.equals("G"))//OK
    {
      Font courier = new Font("Courier", Font.PLAIN, (int)(Math.floor(15*zoom)));
      g.setFont(courier);
      g.drawString("gyroscope",(int)((x*100+26)*zoom),(int)((y*100+65)*zoom));
      Font courier2 = new Font("Courier", Font.PLAIN, (int)(Math.floor(16*zoom)));
      g.setFont(courier2);
    }
    else if(typ.equals("T"))//OK
    {
      g.drawString("touch",(int)((x*100+40)*zoom),(int)((y*100+65)*zoom));
    }
    else if(typ.equals("S"))//OK
    {
      g.drawString("smell",(int)((x*100+40)*zoom),(int)((y*100+65)*zoom));
    }
    else if(typ.equals("*"))//OK
    {
      g.drawString("const",(int)((x*100+40)*zoom),(int)((y*100+65)*zoom));
    }
    else if(typ.equals("|"))//OK
    {
      g.drawString(" bend",(int)((x*100+35)*zoom),(int)((y*100+55)*zoom));
      g.drawString("muscle ",(int)((x*100+35)*zoom),(int)((y*100+70)*zoom));
    }
    else//OK
    {
      g.drawString("neuron",(int)((x*100+35)*zoom),(int)((y*100+65)*zoom));
    }*/
  }
  
  public void drawConnection(Graphics g, int i)
  {
    float x1 = ((Float)fram.getNeuroProp(((Integer)fram.getConnProp(i,"n")).intValue(),"x")).floatValue();//wspolrzedne obu neuronow
    float x2 = ((Float)fram.getNeuroProp(((Integer)fram.getConnProp(i,"i")).intValue(),"x")).floatValue();
    float y1 = ((Float)fram.getNeuroProp(((Integer)fram.getConnProp(i,"n")).intValue(),"y")).floatValue();
    float y2 = ((Float)fram.getNeuroProp(((Integer)fram.getConnProp(i,"i")).intValue(),"y")).floatValue();
    //najpierw zatyczki wyjsciowe i wejsciowe - zalezne od typu neuronu
    //najpierw wyjscie (nadawca sygnalu)
    g.drawLine((int)((10+85+x2*100)*zoom),(int)((10+50+y2*100)*zoom),(int)((10+95+x2*100)*zoom),(int)((10+50+y2*100)*zoom));
    //teraz wejscia
    g.drawLine((int)((10+5+x1*100)*zoom),(int)((10+50+y1*100)*zoom),(int)((10+15+x1*100)*zoom),(int)((10+50+y1*100)*zoom));
    //bezposredni poprzednik - to koniec :)
    if (x1==x2+1 && y1==y2)
    {
      g.drawLine((int)((15+(x1*100))*zoom),(int)((10+50+y1*100)*zoom),(int)((95+x2*100)*zoom),(int)((10+50+y1*100)*zoom));
      return;
    }
    //pierwszy pionowy
    if(y1>=y2) g.drawLine((int)((10+95+(x2*100))*zoom),(int)((60+y2*100)*zoom),(int)((10+95+(x2*100))*zoom),(int)((105+y2*100)*zoom));
    else g.drawLine((int)((10+95+(x2*100))*zoom),(int)((60+y2*100)*zoom),(int)((10+95+(x2*100))*zoom),(int)((15+y2*100)*zoom));
    //poziomy
    if(y1>=y2) g.drawLine((int)((10+95+(x2*100))*zoom),(int)((105+y2*100)*zoom),(int)((15+(x1*100))*zoom),(int)((105+y2*100)*zoom));
    else g.drawLine((int)((10+95+(x2*100))*zoom),(int)((15+y2*100)*zoom),(int)((15+(x1*100))*zoom),(int)((15+y2*100)*zoom));
    //drugi pionowy
    if(y1>=y2) g.drawLine((int)((15+(x1*100))*zoom),(int)((60+y1*100)*zoom),(int)((15+(x1*100))*zoom),(int)((105+y2*100)*zoom));
    else g.drawLine((int)((15+(x1*100))*zoom),(int)((60+y1*100)*zoom),(int)((15+(x1*100))*zoom),(int)((15+y2*100)*zoom));
  }
  
  public void drawNetwork(Graphics g)
  {
    if(markedNeuron>=fram.getNeuroCount()) markedNeuron = -1;//zebu nie bylo nulla, jak zaznaczono neuron u numerze wiekszy niz jest a mamy aktualizacje (cos usunieto w tazo)
    if(sortNeurons) setAllNeurons(); else setNeuronsPos();//zalezy jak chcemy rozmiescic - update
    //algorytm probuje rysowac na ujemnych (poczatkowo dzialal dla 3D) - trza to naprawic
    float minX = Float.POSITIVE_INFINITY;
    float minY = Float.POSITIVE_INFINITY;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {//obliczamy min X a Y po prostu odwracamy(obraz do gory nogami w porownaniu z neuro)
      if(((Float)fram.getNeuroProp(i,"x")).floatValue()< minX) minX = ((Float)fram.getNeuroProp(i,"x")).floatValue();
      if(((Float)fram.getNeuroProp(i,"y")).floatValue()< minY) minY = ((Float)fram.getNeuroProp(i,"y")).floatValue();
    }
    if(minX<0) for(int i=0; i<fram.getNeuroCount(); i++) fram.setNeuroProp(i,"x", new Float(((Float)fram.getNeuroProp(i,"x")).floatValue()-minX));
    if(minY<0) for(int i=0; i<fram.getNeuroCount(); i++) fram.setNeuroProp(i,"y", new Float(((Float)fram.getNeuroProp(i,"y")).floatValue()-minY));
    for(int i=0; i<fram.getNeuroCount(); i++)
    {//rysujemy neurony
      //w zaleznosci od rodzaju zaznaczenia niebieskie lub zielone
      if(fram.elementWeakSelected("n",i)) g.setColor(new Color(40, 150, 255));
      if(fram.elementStrongSelected("n",i)) g.setColor(Color.green);
      drawNeuron(g, ((Float)fram.getNeuroProp(i,"x")).floatValue(), ((Float)fram.getNeuroProp(i,"y")).floatValue(),  fram.getNeuroProp(i,"d").toString());//[wiersz][kolumna]
      g.setColor(Color.black);//na koniec czarny kolor
    }
    for(int i=0; i<fram.getConnCount(); i++)
    {//a potem polaczenia
      drawConnection(g, i);
    }
    //a na wierzchu zaznaczone :) (i neurony i polaczenia w odpowiednich kolorkach)
    if(markedNeuron==-1) return;
    for(int i=0; i<fram.getConnCount(); i++)
    {//a potem polaczenia
      //zasada jw. wyjscia zielone, wejscia czerwone
      if(fram.elementStrongSelected("n",((Integer)fram.getConnProp(i,"n")).intValue())) g.setColor(Color.red);
      else if(fram.elementStrongSelected("n",((Integer)fram.getConnProp(i,"i")).intValue())) g.setColor(Color.green);
      else continue;//jak nie zaznaczone to nie rysujemy ponownie
      drawConnection(g, i);
    }
    g.setColor(Color.black);//ponownie domyslny kolor
/*		g.setColor(Color.red);
                for(int i=0; i<fram.getConnCount(); i++)
                {
                        if(((Integer)fram.getConnProp(i,"n")).intValue()==markedNeuron) drawConnection(g,i);
                }
                g.setColor(Color.green);
                //System.out.println(fram.getNeuroProp(markedNeuron,"x"));
                //System.out.println(fram.getNeuroProp(markedNeuron,"y"));
                //System.out.println(fram.getNeuroProp(markedNeuron,"d"));
                drawNeuron(g, ((Float)fram.getNeuroProp(markedNeuron,"x")).floatValue(), ((Float)fram.getNeuroProp(markedNeuron,"y")).floatValue(), fram.getNeuroProp(markedNeuron,"d").toString());//[wiersz][kolumna]
                for(int i=0; i<fram.getConnCount(); i++)
                {
                        if(((Integer)fram.getConnProp(i,"i")).intValue()==markedNeuron) drawConnection(g,i);
                }
                g.setColor(Color.black);
 */}
  
  // rozmieszczenie w kwadracie o boku dlugosci a=sqrt(N)
  
  public void setNeuronsPos()
  {
    int a = ((int)(Math.sqrt(fram.getNeuroCount())-0.0001))+1;
    int numer = 0;
    for(int i=0; i<a; i++)
      for(int j=0; j<a; j++)
      {
        if (numer==fram.getNeuroCount()) return;//OK
        fram.setNeuroProp(numer,"x",new Float(i));
        fram.setNeuroProp(numer,"y",new Float(j));
        numer++;
      }
  }
  
  public void setZoomValue()
  {//aby pomiescic siec w oknie na samym poczatku
    float maxNeuronCoord = 0;
    int minWindowCoord = 0;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {//skalowanie to max. rozmiar okna (pomniejszony o opis) / max wsp. symbolu neuronu
      if(((Float)fram.getNeuroProp(i,"x")).floatValue()*100>maxNeuronCoord) maxNeuronCoord = ((Float)fram.getNeuroProp(i,"x")).floatValue()*100;
      if(((Float)fram.getNeuroProp(i,"y")).floatValue()*100>maxNeuronCoord) maxNeuronCoord = ((Float)fram.getNeuroProp(i,"y")).floatValue()*100;
    }
    if(d.width>d.height-35) minWindowCoord = d.height-35;
    else minWindowCoord = d.width;
    zoom = (minWindowCoord/*-35*/)/(maxNeuronCoord+110+10);
  }
  
  public void zoomIn()
  {
    zoom *=2;
    repaint(0);
  }
  
  public void zoomOut()
  {
    zoom /=2;
    repaint(0);
  }
  
  public void changePlacingMethod()
  {//zmiana metody sortowania neuronow
    if(sortNeurons)
    {
      setNeuronsPos();
      sortNeurons = false;
    }
    else
    {
      setAllNeurons();
      sortNeurons = true;
    }
    //algorytm probuje rysowac na ujemnych (poczatkowo dzialal dla 3D) - trza to naprawic
    float minX = Float.POSITIVE_INFINITY;
    float minY = Float.POSITIVE_INFINITY;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {//obliczamy min X a Y po prostu odwracamy(obraz do gory nogami w porownaniu z neuro)
      if(((Float)fram.getNeuroProp(i,"x")).floatValue()< minX) minX = ((Float)fram.getNeuroProp(i,"x")).floatValue();
      if(((Float)fram.getNeuroProp(i,"y")).floatValue()< minY) minY = ((Float)fram.getNeuroProp(i,"y")).floatValue();
    }
    if(minX<0) for(int i=0; i<fram.getNeuroCount(); i++) fram.setNeuroProp(i,"x", new Float(((Float)fram.getNeuroProp(i,"x")).floatValue()-minX));
    if(minY<0) for(int i=0; i<fram.getNeuroCount(); i++) fram.setNeuroProp(i,"y", new Float(((Float)fram.getNeuroProp(i,"y")).floatValue()-minY));
    setZoomValue();//po przesunieciu X obliczmay new zoom :)
    repaint(0);
  }
  
  public void markNeuronAt(int x, int y)
  {//zmienia kolorki neuronu zaznaczonego mysza
    //zeby mozliwe bylo zaznaczanie kilku neuronow jednoczesnie
    if(!shiftDown && !controlDown)
    {
      for(int i=0; i<fram.getNeuroCount(); i++) fram.unmarkNeuron(i);
    }
    if(sortNeurons) setAllNeurons(); else setNeuronsPos();//zalezy jak chcemy rozmiescic - update
    ///algorytm probuje rysowac na ujemnych (poczatkowo dzialal dla 3D) - trza to naprawic
    float minX = Float.POSITIVE_INFINITY;
    float minY = Float.POSITIVE_INFINITY;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {//obliczamy min X a Y po prostu odwracamy(obraz do gory nogami w porownaniu z neuro)
      if(((Float)fram.getNeuroProp(i,"x")).floatValue()< minX) minX = ((Float)fram.getNeuroProp(i,"x")).floatValue();
      if(((Float)fram.getNeuroProp(i,"y")).floatValue()< minY) minY = ((Float)fram.getNeuroProp(i,"y")).floatValue();
    }
    if(minX<0) for(int i=0; i<fram.getNeuroCount(); i++) fram.setNeuroProp(i,"x", new Float(((Float)fram.getNeuroProp(i,"x")).floatValue()-minX));
    if(minY<0) for(int i=0; i<fram.getNeuroCount(); i++) fram.setNeuroProp(i,"y", new Float(((Float)fram.getNeuroProp(i,"y")).floatValue()-minY));
    //setZoomValue();
    int posX = (int)(((x/zoom)-10)/100);
    int posY = (int)(((y/zoom)-10)/100);
    for(int i=0; i<fram.getNeuroCount(); i++)
      if((int)((Float)fram.getNeuroProp(i,"x")).floatValue()==posX && (int)((Float)fram.getNeuroProp(i,"y")).floatValue()==posY)
      {
        if(fram.elementStrongSelected("n",i) && controlDown)
        {
          fram.unmarkNeuron(i);
          fram.fireSignificantEditEvent();
          return;
        }
        markedNeuron = i;
        fram.markNeuron(i);//zaznaczamy neuron w modelu
        createDescription(i);
        fram.fireSignificantEditEvent();
        return;
      }
    fram.fireSignificantEditEvent();
  }
  
  public void createDescription(int id)//tworzy opis slowny zaznaczonego neuronu
  {
    description0 = "Neuron " + id + " (" + fram.getNeuroProp(id,"d") + ") ";//paramerty neuronu
    description1 = "In : ";					//wejscia
    description2 = "Out: ";					//oraz wyjscia
    if (fram.neuroPropExists(id,"p")) description0 = description0 + "- Part " + fram.getNeuroProp(id,"p") + " :";
    if (fram.neuroPropExists(id,"j")) description0 = description0 + "- Joint " + fram.getNeuroProp(id,"j") + " :";
    for(int i=0; i<fram.getConnCount(); i++)//i jakie ma polaczenia
    {
      if (id==((Integer)fram.getConnProp(i,"n")).intValue())//do, czyli wejscia
      {
        description1 = description1 + fram.getConnProp(i,"i") + "(w=" + fram.getConnProp(i,"w") + "), ";
      }
      if (id==((Integer)fram.getConnProp(i,"i")).intValue())//od, czyli wyjscia
      {
        description2 = description2 + fram.getConnProp(i,"n") + "(w=" + fram.getConnProp(i,"w") + "), ";
      }	//opisy sie odpowiednio poszerzaja
    }
  }
  
  public void mousePressed(MouseEvent evt)//wcisniecie klawisza myszy
  {
    if(evt.getModifiers()==MouseEvent.BUTTON3_MASK) return;
    //above - jak chcemy propertisy to nic innego nie robimy
    if(evt.isShiftDown()) shiftDown=true;
    else shiftDown=false;
    if(evt.isControlDown()) controlDown=true;
    else controlDown=false;
    if(addingConnection) addConnectionProcedure(evt.getPoint().x,evt.getPoint().y);
    else markNeuronAt(evt.getPoint().x,evt.getPoint().y);//dla zaznaczenia
    repaint(0);
  }
  
  public void mouseReleased(MouseEvent evt)
  {}
  public void mouseDragged(MouseEvent evt)
  {}
  public void mouseClicked(MouseEvent evt)
  {}
  public void mouseEntered(MouseEvent evt)
  {}
  public void mouseExited(MouseEvent evt)
  {}
  public void mouseMoved(MouseEvent evt)
  {}
  
  public void mouseWheelMoved(MouseWheelEvent evt)
  {
    if(evt.getWheelRotation()==-1) zoomOut();
    else zoomIn();
  }
  
  //nowe metody edycyjne
  
  public void symbolAddNeuron(String type)
  {
    fram.addNeuron(type);//domyslnie dodajemy standardowy neuron
    if(sortNeurons)//koniecznosc: jak posortowane
    {
      setAllNeurons();//to umieszczamy wspolrzedne
      sortNeurons = false;//sztucznie zmieniamy sposob rozmieszczenia (i tak bedzie true)
      changePlacingMethod();//i rozmieszczamy neurony
    }//wszystko to po to, by nie bylo nulli
    else setNeuronsPos();//jak nie posortowane to jest latwiej
    setZoomValue();//zblizenie odpowiednie
    description0 = I18N.getMessage("neuronAdded");
    description1 = "";
    description2 = "";//i odpowiedni komunikat
    fram.fireSignificantEditEvent();
  }
  
  public void symbolDelNeuron()//kasowanie neuronu
  {
    boolean niema = true;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {
      if(fram.elementStrongSelected("n",i))
      {
        niema = false;
        break;
      }
    }
    if(niema)
    {
      description0=I18N.getMessage("markDel");
      description1="";
      description2="";
      return;
    }
    {
      //comment fram.removeNeuron(markedNeuron);//usuwamy neuron
      for(int i=fram.getNeuroCount()-1; i>=0; i--)
      {
        if(fram.elementStrongSelected("n",i) )//|| fram.elementWeakSelected("n",i))
        {
          fram.unmarkNeuron(i);//just in case - bo przemianowywanie
          fram.removeNeuron(i);
        }
      }
      //for(int i=0; i<fram.getNeuroCount(); i++)
      //{
      //	if(fram.elementStrongSelected("n",i)) fram.unmarkNeuron(i);
      //}
      if(sortNeurons)//patrz metoda symbolAddNeuron() line 408
      {
        setAllNeurons();
        sortNeurons = false;
        changePlacingMethod();
      }
      else setNeuronsPos();
      setZoomValue();
      markedNeuron=-1;//zaznaczamy w symbol, ze go nie ma
      description0 = I18N.getMessage("neuronRemoved");
      description1 = "";//i stosowny komunikat
      description2 = "";
      fram.fireSignificantEditEvent();
    }
  }
  
  public void symbolAddConnect()//wejscie w tryb dodawania
  {
    if (markedNeuron==-1)//pierwszy z elementow musi byc zaznaczony
    {
      description0=I18N.getMessage("notMarked");
      description1="";//i user ma to wiedziec
      description1="";
    }
    else
    {
      addingConnection = true;// jak zaznaczono, mozna dodac drugi
      description0=I18N.getMessage("mark2");
      description1="";//polecamy to zrobic :)
      description2="";
    }
  }
  
  public void addConnectionProcedure(int x, int y)//tu wlasciwe dodanie
  {
    if (markedNeuron==-1)//po wybraniu funkcji mogli odznaczyc pierwszy neuron
    {
      description0=I18N.getMessage("notMarked");
      description1="";
      description2="";
      addingConnection=false;//wychodzimy z trybu dodawania
      return;
    }
    int posX = (int)(((x/zoom)-10)/100);//identyfikacja drugiego neuronu
    int posY = (int)(((y/zoom)-10)/100);//na podstawie wspolrzednych panelu
    int secondNeuron = -1;
    for(int i=0; i<fram.getNeuroCount(); i++)
      if((int)((Float)fram.getNeuroProp(i,"x")).floatValue()==posX && (int)((Float)fram.getNeuroProp(i,"y")).floatValue()==posY)
      {
        secondNeuron = i;
        break;
      }
    if (secondNeuron!=-1)//musi byc drugi neuron
    {
      if(fram.addConnection(secondNeuron,markedNeuron))
      {//jesli udane utworzenie to message
        description0=I18N.getMessage("connAdded");
        description1="";
        description2="";
        addingConnection=false;//koniec trybu
        fram.fireSignificantEditEvent();
        if(sortNeurons)//patrz metoda symbolAddNeuron() line 408
        {
          setAllNeurons();
          sortNeurons = false;
          changePlacingMethod();
        }
        else setNeuronsPos();
        setZoomValue();
      }
      else
      {//a jesli nie wydalo
        description0=I18N.getMessage("cantAdd");
        description1="";
        description2="";
        addingConnection=false;
      }
    }
    else//jak drugie klikniecie nie wskazalo neuronu to tez zwara
    {
      description0=I18N.getMessage("mark1st");
      description1=I18N.getMessage("tryAgain");
      description2="";
      addingConnection=false;
      return;
    }
  }
  
  public void symbolUndo()//operacja undo
  {
    if (fram.undo())//jesli sie powiodla
    {
      if(sortNeurons) //patrz metoda symbolAddNeuron() line 408
      {
        setAllNeurons();
        sortNeurons = false;
        changePlacingMethod();
      }
      else setNeuronsPos();
      setZoomValue();
      markedNeuron=-1;//to odznaczamy neuron (just in case, zeby nulli nie bylo)
      description0 = I18N.getMessage("undo");
      description1 = "";
      description2 = "";
      fram.fireSignificantEditEvent();
    }
  }
  
  public void symbolRedo()//operacja redo
  {
    if (fram.redo())//postepujemy identycznie jak w przypadku undo
    {
      if(sortNeurons)//patrz metoda symbolAddNeuron() line 408
      {
        setAllNeurons();
        sortNeurons = false;
        changePlacingMethod();
      }
      else setNeuronsPos();
      setZoomValue();
      markedNeuron=-1;
      description0 = I18N.getMessage("redo");
      description1 = "";
      description2 = "";
      fram.fireSignificantEditEvent();
    }
  }
  
  public String[] showProperties(int x, int y)// trial method
  {
    int number = -1;
    boolean znaleziono = false;
    int posX = (int)(((x/zoom)-10)/100);
    int posY = (int)(((y/zoom)-10)/100);
    for(int i=0; i<fram.getNeuroCount(); i++)
      if((int)((Float)fram.getNeuroProp(i,"x")).floatValue()==posX && (int)((Float)fram.getNeuroProp(i,"y")).floatValue()==posY)
      {
        number = i;
        znaleziono = true;
        break;
      }
    if(number==-1) return null;
    String[] s =
    {String.valueOf(number), "n"};
    return s;
  }
  
  protected class MyFramstickListener implements FramstickListener
  {
    public void significantEdit(FramstickEvent framEvent)
    {
      //System.out.println("Names: edit event !");
      if(sortNeurons) //patrz metoda symbolAddNeuron() line 408
      {
        setAllNeurons();
        sortNeurons = false;
        changePlacingMethod();
      }
      else setNeuronsPos();
      setZoomValue();
      repaint(0);
    }
    
    public void markChanged(FramstickEvent framEvent)
    {
      //System.out.println("Names: mark event !");
      markedNeuron = -1;
      description0 = "";			//3 linie do opisania zaznaczonego neuronu
      description1 = "";
      description2 = "";
      if(sortNeurons) //patrz metoda symbolAddNeuron() line 408
      {
        setAllNeurons();
        sortNeurons = false;
        changePlacingMethod();
      }
      else setNeuronsPos();
      setZoomValue();
      repaint(0);
    }
  }
  //--------------------------------------------------------------------------------------
  //------------------------all the stuff required for sorting----------------------------
  //--------------------------------------------------------------------------------------
  
  public void setAllNeurons()//ladne, posortowane
  {
    if(fram.getNeuroCount()==0) return;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {
      fram.setNeuroProp(i,"x",new Float(0));//new Float(Float.POSITIVE_INFINITY));
      fram.setNeuroProp(i,"y",new Float(0));//new Float(Float.POSITIVE_INFINITY));
      fram.setNeuroProp(i,"blok",new Integer(Integer.MAX_VALUE));
    }
    int liczbaNeuronow = fram.getNeuroCount();
    int el;
    for(el=0; el<liczbaNeuronow; el++) ustawElement(el);
    int blok = -1;
    for(el=0; el<fram.getNeuroCount(); el++)
    {
      if(((Integer)fram.getNeuroProp(el,"blok")).intValue()==el)
      {
        blok=el;
        el++;
        break;
      }
    }
    if(blok==-1) blok=0;
    for(;el<liczbaNeuronow;el++)
    {
      if(blok!=((Integer)fram.getNeuroProp(el,"blok")).intValue())
      {
        int numer = ((Integer)fram.getNeuroProp(el,"blok")).intValue();
        float blokminx = Float.POSITIVE_INFINITY;
        float numerminx= Float.POSITIVE_INFINITY;
        float numerminy = Float.POSITIVE_INFINITY;
        float blokmaxy = Float.NEGATIVE_INFINITY;
        for(int i=0; i<fram.getNeuroCount(); i++)
        {
          if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==blok)
          {
            if(blokminx>((Float)fram.getNeuroProp(i,"x")).floatValue()) blokminx=((Float)fram.getNeuroProp(i,"x")).floatValue();
            if(blokmaxy<((Float)fram.getNeuroProp(i,"y")).floatValue()) blokmaxy=((Float)fram.getNeuroProp(i,"y")).floatValue();
          }
          if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==numer)
          {
            if(numerminx>((Float)fram.getNeuroProp(i,"x")).floatValue()) numerminx=((Float)fram.getNeuroProp(i,"x")).floatValue();
            if(numerminy>((Float)fram.getNeuroProp(i,"y")).floatValue()) numerminy=((Float)fram.getNeuroProp(i,"y")).floatValue();
          }
        }
        dolaczBlok(blok,numer,blokminx-numerminx,blokmaxy-numerminy+1);
      }
    }
  }
  
  boolean moznaDolaczyc(int /***/b, int /***/b2, float dx, float dy)
  {
    if(b==b2)return false;
    float bmaxx= Float.NEGATIVE_INFINITY;
    float bmaxy= Float.NEGATIVE_INFINITY;
    float b2maxx= Float.NEGATIVE_INFINITY;
    float b2maxy = Float.NEGATIVE_INFINITY;
    float bminx= Float.POSITIVE_INFINITY;
    float b2minx = Float.POSITIVE_INFINITY;
    float bminy = Float.POSITIVE_INFINITY;
    float b2miny = Float.POSITIVE_INFINITY;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {
      if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==b)
      {
        if(((Float)fram.getNeuroProp(i,"x")).floatValue()<bminx) bminx=((Float)fram.getNeuroProp(i,"x")).floatValue();
        if(((Float)fram.getNeuroProp(i,"y")).floatValue()<bminy) bminy=((Float)fram.getNeuroProp(i,"y")).floatValue();
        if(((Float)fram.getNeuroProp(i,"x")).floatValue()>bmaxx) bmaxx=((Float)fram.getNeuroProp(i,"x")).floatValue();
        if(((Float)fram.getNeuroProp(i,"y")).floatValue()>bmaxy) bmaxy=((Float)fram.getNeuroProp(i,"y")).floatValue();
      }
      if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==b2)
      {
        if(((Float)fram.getNeuroProp(i,"x")).floatValue()<b2minx) b2minx=((Float)fram.getNeuroProp(i,"x")).floatValue();
        if(((Float)fram.getNeuroProp(i,"y")).floatValue()<b2miny) b2miny=((Float)fram.getNeuroProp(i,"y")).floatValue();
        if(((Float)fram.getNeuroProp(i,"x")).floatValue()>b2maxx) b2maxx=((Float)fram.getNeuroProp(i,"x")).floatValue();
        if(((Float)fram.getNeuroProp(i,"y")).floatValue()>b2maxy) b2maxy=((Float)fram.getNeuroProp(i,"y")).floatValue();
      }
    }
    float x1,x2,y1,y2;
    x1=Math.max(0,b2minx-bminx+dx);
    x2=Math.min(bmaxx-bminx,-bminx+dx+b2maxx);
    if(x1>x2) return true;
    y1=Math.max(0,b2miny-bminy+dy);
    y2=Math.min(bmaxy-bminy,-bminy+dy+b2maxy);
    if(y1>y2) return true;
    int x,y;
    dx+=b2minx-bminx;
    dy+=b2miny-bminy;
    int w = (int)bmaxx - (int)bminx + 1;
    int h = (int)bmaxy - (int)bminy + 1;
    float[] mapa = new float[w*h];
    for(int i=0; i<w*h; i++) mapa[i]=0;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {
      if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==b)
        mapa[w*(int)(((Float)fram.getNeuroProp(i,"y")).floatValue()-bminy)+(int)(((Float)fram.getNeuroProp(i,"x")).floatValue()-bminx)]=1;
    }
    int w2 = (int)b2maxx - (int)b2minx + 1;
    int h2 = (int)b2maxy - (int)b2miny + 1;
    float[] mapa2 = new float[w2*h2];
    for(int i=0; i<w2*h2; i++) mapa2[i]=0;
    for(int i=0; i<fram.getNeuroCount(); i++)
    {
      if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==b2)
        mapa2[w2*(int)(((Float)fram.getNeuroProp(i,"y")).floatValue()-b2miny)+(int)(((Float)fram.getNeuroProp(i,"x")).floatValue()-b2minx)]=1;
    }
    for(y=(int)y1; y<=y2; y++)
    {
      for(x=(int)x1; x<=x2; x++)
      {
        if(mapa[w*y+x]==1 && mapa2[w2*(y-(int)dy)+(x-(int)dx)]==1) return false;
        
      }
    }
    return true;
  }
  
  boolean dolaczBlok(int b, int b2, float dx, float dy)
  {
    if(!moznaDolaczyc(b,b2,dx,dy)) return false;
    int i,e;
    for(i=0; i<fram.getNeuroCount(); i++)
    {
      if(((Integer)fram.getNeuroProp(i,"blok")).intValue()==b2)
      {
        fram.setNeuroProp(i,"blok",new Integer(b));
        fram.setNeuroProp(i,"x",new Float(((Float)fram.getNeuroProp(i,"x")).floatValue()+dx));
        fram.setNeuroProp(i,"y",new Float(((Float)fram.getNeuroProp(i,"y")).floatValue()+dy));
      }
    }
    return true;
  }
  
  void polaczJakoWejscie(int e, int e2)
  {
    int b,b2;
    b=((Integer)fram.getNeuroProp(e,"blok")).intValue();
    if(((Integer)fram.getNeuroProp(e2,"blok")).intValue()==Integer.MAX_VALUE)
      fram.setNeuroProp(e2,"blok",new Integer(e2));
    b2=((Integer)fram.getNeuroProp(e2,"blok")).intValue();
    if(b==b2) return;
    float dx,dy;
    dx=((Float)fram.getNeuroProp(e,"x")).floatValue()-((Float)fram.getNeuroProp(e2,"x")).floatValue();
    dy=((Float)fram.getNeuroProp(e,"y")).floatValue()-((Float)fram.getNeuroProp(e2,"y")).floatValue();
    if(dolaczBlok(b,b2,dx-1,dy)) return;
    int proba;
    for(proba=1;;proba++)
    {
      if(dolaczBlok(b,b2,dx-1,dy-proba)) return;
      if(dolaczBlok(b,b2,dx-1,dy+proba)) return;
    }
  }
  
  void ustawElement(int e)
  {
    if(((Integer)fram.getNeuroProp(e,"blok")).intValue()!=Integer.MAX_VALUE) return;
    fram.setNeuroProp(e,"blok",new Integer(e));
    int we;
    for(int i=0; i<fram.getConnCount(); i++)
    {
      if(((Integer)fram.getConnProp(i,"n")).intValue()!=e) continue;
      int e2 = ((Integer)fram.getConnProp(i,"i")).intValue();
      if((e2<0) || (e==e2)) continue;
      ustawElement(e2);
      polaczJakoWejscie(e,e2);
    }
  }
}