
/**
 * <p>Title: F0Automaton.java</p>
 * @author Cezary Dowhan
 * @version 2.0
 */

package jGDK;

import java.io.*;

public class F0Automaton
{
  public static final int ID_ERROR	= -1;	//	syntax error
  public static final int ID_NONE		=  0;	//	auxiliary token
  public static final int ID_EOL		=  1;	//	the end of line
  public static final int ID_EOF		=  2;	//	the end of file
  public static final int ID_CLASS	=  3;	//	p: ...
  public static final int ID_VARIABLE	=  4;	//	x= ...
  public static final int ID_WORD		=  6;	//	ascd, |, *, abc_g78, etc.
  public static final int ID_QUOTE	=  7;	//	"abc\tdef"
  public static final int ID_FORMAT	=  8;	//	//number or /*number*/
  
  protected	StreamTokenizer st;
  private		Reader buffer;
  
  public  int	lineNo;			// the number of readed lines
  private int offset;
  
  public	Token tokenMaster;
  public  Token tokenSlave;
  private Token currentToken;
  
  // constructor ----------------------------------------------------------
  
  /**	Na wejsciu oczekuje genotypu w postci lancucha znakow */
  
  public F0Automaton(String string)
  {
    buffer	= new StringReader(string);
    st		= new StreamTokenizer(buffer);
    
    initTokenizerSettings();
    
    tokenMaster = new Token();
    tokenSlave	= new Token();
    
    lineNo = 1;
  }
  //the end of initial section --------------------------------------
  
  /**	Standardowe ustawienia StreamTokenizera. */
  
  protected void initTokenizerSettings()
  {
    st.resetSyntax();
    st.whitespaceChars('\t',' ');
    st.wordChars('a','z');
    st.wordChars('A','Z');
    st.wordChars('0','9');
    st.wordChars('_','_');
    st.wordChars('-','-');
    st.wordChars('.','.');
    st.wordChars('@','@');
    st.wordChars('|','|');
    st.quoteChar('"');
    st.eolIsSignificant(true);
    addTokenizerSettings();
  }
  
  protected void addTokenizerSettings()
  {
    ;
  }
  
  // the end of protected method: initTokenizerSettings() --------------------------
  
  /**	Rozpoznaje typowe dla formatu f0 tokeny. */
  
  public int nextToken()
  {
    tokenMaster.type = ID_NONE;
    tokenSlave.type  = ID_NONE;
    
    tokenMaster.data = null;
    tokenSlave.data  = null;
    
    currentToken=tokenMaster;
    String possible = "/WEQ";
    
    // oznacza, ze jestesmy po znaku '=' wiec kolejne tokeny beda sklejane
    // az do EOL,EOF lub przecinka (zeby umozliwic d=costam zamiast d="costam")
    boolean afterEQ=false;
    
    try
    {
      while (true)
      {
        switch (st.nextToken())
        {
          case ':':
            if (possible.indexOf(':') != -1)
            {
              currentToken.type = ID_CLASS;
              offset = 0;
              afterEQ=false;
              return tokenMaster.type;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case ',' :
            if (possible.indexOf(',') != -1)
            {
              if (tokenMaster.type == ID_NONE) continue;
              offset++;
              return tokenMaster.type;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case '#': //skip comment - dont know if this is well implemented!
            if (tokenMaster.type == ID_NONE)
            {
              skipLine();
              tokenMaster.type=ID_EOL;
              offset = 0;
              lineNo++;
              return ID_EOL;
            }
            //------------------------------------------
          case StreamTokenizer.TT_EOL:
            if (possible.indexOf('E') != -1)
            {
              if (tokenMaster.type == ID_NONE)
              {
                tokenMaster.type=ID_EOL;
                offset = 0;
                lineNo++;
                return ID_EOL;
              }
              st.pushBack();
              return tokenMaster.type;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case StreamTokenizer.TT_EOF:
            if (possible.indexOf('E') != -1)
            {
              if (tokenMaster.type == ID_NONE)
              {
                tokenMaster.type=ID_EOF;
                return ID_EOF;
              }
              st.pushBack();
              return tokenMaster.type;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case '=':
            if (possible.indexOf('=') != -1)
            {
              currentToken = tokenSlave;
              tokenMaster.type = ID_VARIABLE;
              offset	= 0;
              possible="WQ";
              afterEQ=true;
              continue;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case StreamTokenizer.TT_WORD:
            if (possible.indexOf('W') != -1)
            {
              if (tokenMaster.type == ID_VARIABLE)
                possible=",E";
              else
                possible=":=,E";
              
              currentToken.type = ID_WORD;
              currentToken.data = new String(st.sval);
              currentToken.offset=offset;
              
              if (afterEQ)
              {
                boolean more=true;
                while(more)
                {
                  int t;
                  switch(t=st.nextToken())
                  {
                    case StreamTokenizer.TT_WORD:
                      currentToken.data=currentToken.data+st.sval;
                      currentToken.type = ID_QUOTE;
                      break;
                    case StreamTokenizer.TT_NUMBER:
                      currentToken.data=currentToken.data+st.nval;
                      currentToken.type = ID_QUOTE;
                      break;
                    case ':':
                    case '=':
                      currentToken.data=currentToken.data+(char)st.ttype;
                      currentToken.type = ID_QUOTE;
                      break;
                    default:
                      more=false;
                      st.pushBack();
                      break;
                  }
                }
                afterEQ=false;
              }
              continue;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case '"':
            if (possible.indexOf('Q') != -1)
            {
              currentToken.type = ID_QUOTE;
              currentToken.data = new String(st.sval);
              currentToken.offset=offset;
              possible=",E";
              afterEQ=false;
              continue;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          case '/':
            if (possible.indexOf('/') != -1)
            {
              int character = st.nextToken();
              if (character == '/' || character == '*')
              {
                if (st.nextToken() == StreamTokenizer.TT_WORD)
                {
                  tokenMaster.data = new String(st.sval);
                  if (character == '/')
                  {
                    skipLine();
                    tokenMaster.type = ID_FORMAT;
                    return ID_FORMAT;
                  }
                  
                  if (character == '*')
                  {
                    do
                    {
                      if (st.nextToken() == '*')
                      {
                        if (st.nextToken() == '/')
                        {
                          tokenMaster.type = ID_FORMAT;
                          return ID_FORMAT;
                        }
                      }
                    }
                    while (st.ttype!=StreamTokenizer.TT_EOL && st.ttype != StreamTokenizer.TT_EOF );
                  }
                }
              }
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
            //------------------------------------------
          default:
            if (possible.indexOf('W') != -1)
            {
              currentToken.type = ID_WORD;
              currentToken.data = String.valueOf((char) st.ttype);
              currentToken.offset=offset;
              possible=":,E";
              continue;
            }
            tokenMaster.type = ID_ERROR;
            return ID_ERROR;
        }
      }
    }
    catch (IOException ioEx)
    {
      tokenMaster.type = ID_ERROR;
      return ID_ERROR;
    }
  }
  // the end of public method: nextToken() -----------------------------------------
  
  /**	Powoduje pobranie sekcji bez rozbijania na tokeny. */
  
  public String getSection()
  {
    StringBuffer sectionContents = new StringBuffer();
    
    st.resetSyntax();
    st.whitespaceChars('\n','\n');
    st.whitespaceChars('\r','\r');
    st.eolIsSignificant(true);
    
    try
    {
      if (st.nextToken() == '~')
      {
        while (st.nextToken() != '~' && st.ttype != StreamTokenizer.TT_EOF)
        {
          if (st.ttype == '\n') lineNo++;
          if (st.ttype == '\\') st.nextToken();
          sectionContents.append((char) st.ttype);
        }
      }
      else
      {
        lineNo++;
        while (st.ttype != StreamTokenizer.TT_EOF &&
        st.ttype != StreamTokenizer.TT_EOL)
        {
          if (st.ttype == '\\') st.nextToken();
          sectionContents.append((char) st.ttype);
          st.nextToken();
        }
      }
      initTokenizerSettings();
      return sectionContents.toString().trim();
    }
    catch (IOException ioEx)
    { return ""; }
  }
  // the end of private method: getSection() ----------------------------------------
  
  /**	Pomija biezaca linie */
  
  protected void skipLine()
  {
    try
    {
      while (st.nextToken()!=StreamTokenizer.TT_EOF &&
      st.ttype!=StreamTokenizer.TT_EOL)
      { ; }
      lineNo++;
    }
    catch (IOException ex)
    { ; }
  }
  
  /**
   *
   */
  
  protected Integer parseInt(String string)
  {
    try
    { return Integer.valueOf(string); }
    catch (NumberFormatException ex)
    { return null; }
  }
  
  /**
   *
   */
  
  protected Float parseFloat(String string)
  {
    try
    { return Float.valueOf(string);	}
    catch (NumberFormatException ex)
    { return null; }
  }
}