/*
 * Created on 2003-06-21
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package jGDK;

import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * <p>Title: <CODE>I18N</CODE></p>
 * <p>Description: Internationalization class. Used to get localized message from <CODE>ResourceBoundle</CODE></p>
 * @author Damian Jarz?bski
 * @see ResourceBoundle
 */
public class I18N
{
  
  private static ResourceBundle msgBoundle;
  private static MessageFormat formatter = new MessageFormat("");
  final static String msgfilename="jGDK/messages";
  
  /**
   * Constructor of the internalization class. This constructor reads the language file
   *  and then we can get localized message throw the <CODE>I18N</CODE> class.
   * @param language
   * @param country
   */
  
  public I18N(String language, String country) throws MissingResourceException
  {
    Locale currentLocale = null;
    if ((language == null) || (language.equalsIgnoreCase("")))
    {
      currentLocale = Locale.getDefault();
      System.out.print("Setting language... default is detected as "+ currentLocale.getLanguage()+"_"+currentLocale.getCountry()+" - ");
    }
    else
    {
      language.toLowerCase();
      
      if ((country == null) || (country.compareToIgnoreCase("") == 0))
      {
        currentLocale = new Locale(language);
      }
      else
      {
        country.toUpperCase();
        currentLocale = new Locale(language, country);
      }
    }
    System.out.println(currentLocale.getDisplayName()+".");
    
    try
    {
      msgBoundle = ResourceBundle.getBundle(msgfilename, currentLocale);
    }
    catch (MissingResourceException e)
    {
      System.out.println(e.getLocalizedMessage());
      System.out.println("Trying to use English_US language messages...");
      msgBoundle = ResourceBundle.getBundle(msgfilename, new Locale("en","US"));
    }
    
    formatter.setLocale(currentLocale);
  }
  /**
   *
   * @param locale
   */
  public static void setLocale(Locale l)
  {
    msgBoundle = ResourceBundle.getBundle(msgfilename, l);
    formatter.setLocale(l);
  }
  
  public static String getMessage(String key)
  {
    String s=key;
    if (msgBoundle != null)
    {
      try
      {
        s = msgBoundle.getString(key);
      }
      catch (MissingResourceException e)
      {
        System.err.println("I18N translation missing for: "+key);
      }
    }
    return s;
  }
  /**
   * Method returning is given <B>key</B> exist or not.
   * @param key
   * @return <B>true</B> or <B>false</B>
   */
  public static boolean isExisting(String key)
  {
    try
    {
      msgBoundle.getString(key);
    }
    catch (MissingResourceException e)
    {
      return false;
    }
    return true;
  }
  /**
   *
   * @return
   */
  public static Vector getAvailableLocales()
  {
    System.out.println("Available locales:");
    Locale list[] = DateFormat.getAvailableLocales();
    Vector alist = new Vector();
    ResourceBundle mb;
    String s;
    for (int i = 0; i < list.length; i++)
    {
      mb = null;
      boolean ok=true;
      try
      {
        mb = ResourceBundle.getBundle(msgfilename, list[i]);
      }
      catch (MissingResourceException e)
      {
        ok=false;
      }
      if (ok)
      {
        Locale l=mb.getLocale();
        for(int j=0;j<alist.size();j++) if (alist.get(j)==l) ok=false; //this resource was used for another locale
        if (ok)
        {
          alist.add(l);
          System.out.println(l.getDisplayName());
        }
      }
    }
    return alist;
  }
  /**
   * Method returning compound message.
   * @param key
   * @param arguments Arguments of this message
   * @return
   */
  public static String getFormated(String key, Object[] arguments)
  {
    formatter.applyPattern(msgBoundle.getString(key));
    return formatter.format(arguments);
  }
  public static void main(String[] args)
  {
    I18N in = new I18N("", "");
    Vector v = I18N.getAvailableLocales();
    System.out.println("Result:" + v.toString());
  }
}
