package jGDK;

import java.util.Hashtable;
import java.util.Vector;

/**
  * <p>Title: Model.java</p>  
  * Main class of jGDK(java Genome Development Kit).
  * Allows you to manipulate with all Framstick's objects.
  * Supports Parts,Joints,Neurons and Connections adding and removing
  * Four vectors vParts,vJoints,vNeurons,vConnections
  * contains basic information about each group of objects.
  * @author Piotr Jankowski
  * @version 1.0
  */
public class Model
{
	public Vector vParts;
	public Vector vNeurons;
	public Vector vJoints;
	public Vector vConnections;
	
	/** 
  	* Constructor of Model class,initializing vectors of objects
	*/
	Model()
	{
		vParts=new Vector();
		vNeurons=new Vector();
		vJoints=new Vector();
		vConnections=new Vector();

	};

	/** 
  	* Constructor of Model class,initializing vectors of objects with objects from another model
	*/
	Model(Model model)
	{
		vParts		= new Vector();
		vNeurons	= new Vector();
		vJoints		= new Vector();
		vConnections = new Vector();

		for (int i=0; i<model.vParts.size(); i++) {
			Part part = (Part) model.vParts.get(i);
			vParts.add(new Part(part));
		}

		for (int i=0; i<model.vJoints.size(); i++) {
			Joint joint = (Joint) model.vJoints.get(i);
			vJoints.add(new Joint(joint));
		}

		for (int i=0; i<model.vNeurons.size(); i++) {
			Neuron neuron = (Neuron) model.vNeurons.get(i);
			vNeurons.add(new Neuron(neuron));
		}


		for (int i=0; i<model.vConnections.size(); i++) {
			Connection conn = (Connection) model.vConnections.get(i);
			vConnections.add(new Connection(conn));
		}
	};

//--------------------------------------------------------------------------------
	/** 
  	* Adds Part to vParts vector.
  	* @param hPart Hashtable with Part's properties
	*/
	public boolean addPart(final Hashtable hPart)
	{
		Part p=new Part(hPart);
		p.setOrientation();
		vParts.addElement(p);
		p=null;
		return true;
	}
//--------------------------------------------------------------------------------
	/** 
  	* Removes Part from vParts vector.
  	* If any joint or neuron is attached to this Part it is removed.
  	* @param i number of Part to remove
	*/
	public void removePart(int i)
	{	
		
	 		vParts.removeElementAt(i);
   			//deleting Joints attached to this Part
			for (int k=vJoints.size()-1; k>-1;k--)
      		{
      			if  (i == ((Integer)((Joint)vJoints.elementAt(k)).properties.get("p1")).intValue()||
      			i == ((Integer)((Joint)vJoints.elementAt(k)).properties.get("p2")).intValue())
      			{
      				removeJoint(k);
				
					Object[] inon={""+i,""+k};
			  
      		/*M*/	ErrManager.send(I18N.getMessage("modeModel"),ErrManager.ER_WARNING, I18N.getFormated("modeDurROPj",inon));
      		
      			};
      		}
			//deleting Neurons attached to this Part
	  		for (int k=vNeurons.size()-1; k>-1;k--)
      		{
      			int r;
      			if (((Neuron)vNeurons.elementAt(k)).properties.containsKey("p"))
      	 		{
      	 			r=((Integer)((Neuron)vNeurons.elementAt(k)).properties.get("p")).intValue();
      	  			if  (i == r) 
      	  			{
      	  				removeNeuron(k);
						Object[] inon={""+i,""+k};
      	  		/*M*/	ErrManager.send(I18N.getMessage("modeModel"),ErrManager.ER_WARNING, I18N.getFormated("modeDurROPn",inon));
      	  			};
 				}     
       		}
       
       		//changing numeration of p1&p2 in Joints definition(only p1 or p2 which value is greater than i)
			for (int k=0; k<vJoints.size(); k++)
      		{
      			int r;
      			r=((Integer)((Joint)vJoints.elementAt(k)).properties.get("p1")).intValue();
      			if  (i < r)
      				{ 
      					((Joint)vJoints.elementAt(k)).properties.put("p1",new Integer(r-1));
      				};
      			
      			r=((Integer)((Joint)vJoints.elementAt(k)).properties.get("p2")).intValue();
      			if  (i < r)
      				{ 
      					((Joint)vJoints.elementAt(k)).properties.put("p2",new Integer(r-1));
      				};
      		}
      
      		//changing numeration of p in Neurons definition(only p which value is greater than i)
			for (int k=0; k<vNeurons.size(); k++)
      		{
      			int r;
      			if (((Neuron)vNeurons.elementAt(k)).properties.containsKey("p"))
      			{
      				r=((Integer)((Neuron)vNeurons.elementAt(k)).properties.get("p")).intValue();
      				if  (i < r)
      				{ 
      					((Neuron)vNeurons.elementAt(k)).properties.put("p",new Integer(r-1));
      				};
      			}
      		}	
      		
	  		return;	
      	
		
	}
//--------------------------------------------------------------------------------	
	/** 
  	* Adds Joint to vJoints vector.
  	* @param hJoint Hashtable with Joint's properties
	*/
	public boolean addJoint(final Hashtable hJoint)
	{	
	 	int p1,p2;
	 	p1=-1;p2=-1;
	 
	 	//checking Part#p1 and Part#p2
	 	if (hJoint.containsKey("p1"))
	 	p1 = ((Integer)hJoint.get("p1")).intValue();
	 	
	 	if (hJoint.containsKey("p2"))
	 	p2 = ((Integer)hJoint.get("p2")).intValue();
	 
	 	if (p1==-1&&p2==-1) 
	 	{	
	 		Object[] inon={"p1, p2"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modePsmbd",inon));
	 		return false;
	 	};
	 
	 	
	 	if (p1==-1) 
	 	{	
	 		Object[] inon={"p1"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modePmbd",inon));
	 		return false;
	 	};

	 	if (p2==-1) 
	 	{	
	 		Object[] inon={"p2"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modePmbd",inon));
	 		return false;
	 	};

	 	if (p1==p2) 
	 	{	
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, "p1 "+I18N.getMessage("modeEq")+" p2.");
	 		return false;
	 	};

	
	 	//checking values
	 	if (p1>=this.vParts.size()||p1<0) 
	 	{
	 		Object[] inon={"p1"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeVinc",inon));
	 		return false;
	 }
	 
	 if (p2>=this.vParts.size()||p2<0) 
	 	{
	 		Object[] inon={"p2"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeVinc",inon));
	 		return false;
	 }
	 
	
     	//checking if this Joint doesn't already exists.
  		for(int i=0;i<vJoints.size(); i++)
    	{ 
    		int vP1 = ((Integer)((Joint)vJoints.elementAt(i)).properties.get("p1")).intValue();
      		int vP2 = ((Integer)((Joint)vJoints.elementAt(i)).properties.get("p2")).intValue();
	  		if ( ((p1==vP1)&&(p2==vP2)) || ((p1==vP2)&&(p2==vP1)) ) 
	  		{	
	  				
	 			Object[] inon={""+p1,""+p2};
	  	/*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeJbpe",inon));
	  			return false;
	  		}
    	}
    
     	Joint j=new Joint(hJoint);
    
	 	vJoints.addElement(j);
	
	 	Part newP2Part = j.calcPosition(vParts);
		
	 	float dxTest=0;
	 	if (j.properties.containsKey("dx")) dxTest=((Float)j.properties.get("dx")).floatValue();
	
	 	if (dxTest!=0) 
	 	{   
	 		((Part)vParts.elementAt(p1)).parent=true;
	 		vParts.setElementAt(newP2Part,p2);
	 		((Part)vParts.elementAt(p2)).child=true;
 		}
 		j=null;
 		newP2Part=null;
	 	return true;
   }
//--------------------------------------------------------------------------------
	/** 
  	* Marks Part(strong selection) and Joints attached to this Part(weak selection) . 
  	* @param nr number of Part to mark.
  	*/
	public void markPart(int nr)
	{
		((Element)vParts.elementAt(nr)).stSelElement();
		int vJ1,vJ2,no=0;

		for(int i=0;i<vJoints.size();i++)
		{
		 vJ1 = ((Integer)((Joint)vJoints.elementAt(i)).properties.get("p1")).intValue();
      		 vJ2 = ((Integer)((Joint)vJoints.elementAt(i)).properties.get("p2")).intValue();

			if (vJ1==nr||vJ2==nr)
			{
				((Element)vJoints.elementAt(i)).weSelElement();
			}
		}
		
		for(int i=0; i<vNeurons.size();i++)
		{
			if(((Neuron)vNeurons.elementAt(i)).properties.containsKey("p"))
			{
				no=((Integer)((Neuron)vNeurons.elementAt(i)).properties.get("p")).intValue();			
				if(no==nr) ((Element)vNeurons.elementAt(i)).weSelElement();
			}
		}
	}
//------------------------------------------------------------------
	/** 
  	* Unmarks Part and Joints attached to this Part . 
  	* @param nr number of Part to unmark.
  	*/
	public void unmarkPart(int nr)
	{
		((Element)vParts.elementAt(nr)).stDesElement();
		int vJ1,vJ2,no=0;

		for(int i=0;i<vJoints.size();i++)
		{
		 vJ1 = ((Integer)((Joint)vJoints.elementAt(i)).properties.get("p1")).intValue();
      		 vJ2 = ((Integer)((Joint)vJoints.elementAt(i)).properties.get("p2")).intValue();

			if (vJ1==nr||vJ2==nr)
			{
				((Element)vJoints.elementAt(i)).weDesElement();
			}
		}

		for(int i=0; i<vNeurons.size();i++)
		{
			if(((Neuron)vNeurons.elementAt(i)).properties.containsKey("p"))
			{
				no=((Integer)((Neuron)vNeurons.elementAt(i)).properties.get("p")).intValue();			
				if(no==nr) ((Element)vNeurons.elementAt(i)).weDesElement();
			}
		}
	}
//--------------------------------------------------------------------------------
	/** 
  	* Marks Joint(strong selection). 
  	* @param nr number of Joint to mark.
  	*/
	public void markJoint(int nr)
	{
		((Element)vJoints.elementAt(nr)).stSelElement();
		int no = 0;
		for(int i=0; i<vNeurons.size();i++)
		{
			if(((Neuron)vNeurons.elementAt(i)).properties.containsKey("j"))
			{
				no=((Integer)((Neuron)vNeurons.elementAt(i)).properties.get("j")).intValue();			
				if(no==nr) ((Element)vNeurons.elementAt(i)).weSelElement();
			}
		}
	}

//------------------------------------------------------------------
	/** 
  	* Unmarks Joint. 
  	* @param nr number of Joint to unmark.
  	*/

	public void unmarkJoint(int nr)
	{
		((Element)vJoints.elementAt(nr)).stDesElement();
		int no = 0;
		for(int i=0; i<vNeurons.size();i++)
		{
			if(((Neuron)vNeurons.elementAt(i)).properties.containsKey("j"))
			{
				no=((Integer)((Neuron)vNeurons.elementAt(i)).properties.get("j")).intValue();			
				if(no==nr) ((Element)vNeurons.elementAt(i)).weDesElement();
			}
		}
	}
//------------------------------------------------------------------
	/** 
  	* Removes Joint from vJoints vector.
  	* If any neuron is attached to this Joint it is removed.
  	* @param i number of Joint to remove
	*/
	public void removeJoint(int i)
	{
	 		vJoints.removeElementAt(i);
	 	
	  		//deleting Neurons attached to this Joint
	  		for (int k=vNeurons.size()-1; k>-1;k--)
      		{
      			int r;
      			if (((Neuron)vNeurons.elementAt(k)).properties.containsKey("j"))
      	 		{
      	 			r=((Integer)((Neuron)vNeurons.elementAt(k)).properties.get("j")).intValue();
      	  			if  (i == r) 
      	  			{
      	  				removeNeuron(k);
      	  				Object[] inon={""+i,""+k};
      	  		/*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_WARNING, I18N.getFormated("modeDurROJ",inon));
      	  			};
 		 		}     
       		} 
			
			//changing numeration of j in Neurons definition(only j which value is greater than i)	 
	  		for (int k=0; k<vNeurons.size(); k++)
      		{
      			int r;
      			if (((Neuron)vNeurons.elementAt(k)).properties.containsKey("j"))
      			{
      				r=((Integer)((Neuron)vNeurons.elementAt(k)).properties.get("j")).intValue();
      				if  (i < r)
      				{ 
      					((Neuron)vNeurons.elementAt(k)).properties.put("j",new Integer(r-1));
      				};
      			}
      		}
	 	
	 	
	 		return;
	 }

//--------------------------------------------------------------------------------
	/** 
  	* Adds Neuron to vNeurons vector.
  	* @param hNeuron Hashtable with Neuron's properties
	*/
	public boolean addNeuron(final Hashtable hNeuron)
   	{
   		//checking  if p and j are not both defined 
   		if (hNeuron.containsKey("j")&&hNeuron.containsKey("p")) return false;
   	    //checking if p has correct value
    	if (hNeuron.containsKey("p"))
    	{
    		int pN = ((Integer)hNeuron.get("p")).intValue();
    		if (!(pN>=0&&pN<vParts.size()))
    		{
    			Object[] inon={"p"};
		/*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeVinc",inon));    			
				return false;
    		}
		}
    
    	//checking if j has correct value
    	if (hNeuron.containsKey("j"))
    	{
    		int jN = ((Integer)hNeuron.get("j")).intValue();
    		if (!(jN>=0&&jN<vJoints.size()))
    		{	
    			Object[] inon={"j"};
		/*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeVinc",inon));    			
				return false;
    		}
		}
   	
   		Neuron n=new Neuron(hNeuron);
   		vNeurons.addElement(n);
   		n=null;
   		return true;
   }
//--------------------------------------------------------------------------------	
	/** 
  	* Removes Neuron from vNeurons vector.
  	* If any connection is attached to this Neuron it is removed.
  	* @param i number of Neuron to remove
	*/
	public void removeNeuron(int i)
	{				
		
	 		vNeurons.removeElementAt(i);
	  	//deleting Connections attached to this Neuron
	  	for (int k=vConnections.size()-1; k>-1;k--)
      	{
      		if  (i == ((Integer)((Connection)vConnections.elementAt(k)).properties.get("n")).intValue()||
      		i == ((Integer)((Connection)vConnections.elementAt(k)).properties.get("i")).intValue())
      			{
      				removeConnection(k);
      				
     				Object[] inon={""+i,""+k};
			/*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_WARNING, I18N.getFormated("modeDurRON",inon));
      				
      			};
      	}
 		
 		//changing numeration of n and i in Connection definition(only n or i which value is greater than i)
 		for (int k=0; k<vConnections.size(); k++)
      	{
      		int r;
      		r=((Integer)((Connection)vConnections.elementAt(k)).properties.get("i")).intValue();
      		if  (i < r)
      		{ 
      			((Connection)vConnections.elementAt(k)).properties.put("i",new Integer(r-1));
      			
      		};
      			
      		r=((Integer)((Connection)vConnections.elementAt(k)).properties.get("n")).intValue();
      		if  (i < r)
      		{ 
      			((Connection)vConnections.elementAt(k)).properties.put("n",new Integer(r-1));
      			
      		};
      	} 
	 	return;	
	 
	} 	
//-------------------------------------------------------------------------------
	/** 
  	* Marks Neuron(strong selection) and Connections attached to this Neuron(weak selection) . 
  	* @param nr number of Neuron to mark.
  	*/
	public void markNeuron(int nr)
	{

	
		((Element)vNeurons.elementAt(nr)).stSelElement();
		int vN,vI=0;

		for(int i=0;i<vConnections.size();i++)
		{
		 vN = ((Integer)((Connection)vConnections.elementAt(i)).properties.get("n")).intValue();
      	 vI = ((Integer)((Connection)vConnections.elementAt(i)).properties.get("i")).intValue();

			if (vN==nr||vI==nr)
			{
				((Element)vConnections.elementAt(i)).weSelElement();
			}
		}
	}
//-------------------------------------------------------------------------------
	/** 
  	* Unmarks Neuron and Connections attached to this Neuron. 
  	* @param nr number of Neuron to unmark.
  	*/
	public void unmarkNeuron(int nr)
	{

	
		((Element)vNeurons.elementAt(nr)).stDesElement();
		int vN,vI=0;

		for(int i=0;i<vConnections.size();i++)
		{
		 vN = ((Integer)((Connection)vConnections.elementAt(i)).properties.get("n")).intValue();
      	 vI = ((Integer)((Connection)vConnections.elementAt(i)).properties.get("i")).intValue();

			if (vN==nr||vI==nr)
			{
				((Element)vConnections.elementAt(i)).weDesElement();
			}
		}
	}

   
//--------------------------------------------------------------------------------	

	/** 
  	* adds Connection to vConnections vector.
  	* @param hConnection Hashtable with Connection's properties
	*/
	public boolean addConnection(final Hashtable hConnection)
	{
		int n,i;
	 	n=-1;i=-1;
	 
	 	//checkig parent & child exist
	 	if (hConnection.containsKey("n"))
	 	n = ((Integer)hConnection.get("n")).intValue();
	 	if (hConnection.containsKey("i"))
	 	i = ((Integer)hConnection.get("i")).intValue();
	 	
	 	if (n==-1&&i==-1) 
	 	{
	 		Object[] inon={"n, i"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modePsmbd",inon));
	 		return false;
	 	}

	 	
	 	if (n==-1) 
	 	{
	 		Object[] inon={"n"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modePmbd",inon));
	 		return false;
	 	}
	 
	 	if (i==-1) 
	  	{
	 		Object[] inon={"i"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modePmbd",inon));
	 		return false;
	 	}
	 	
	 	
	 	
	
	 	//checking values
	 	if (n>=this.vNeurons.size()||n<0) 
	 	{
	 		Object[] inon={"n"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeVinc",inon));
	 		return false;
	 	}

	 	if (i>=this.vNeurons.size()||i<0) 
	 	{
			Object[] inon={"i"};
	 /*M*/	ErrManager.send(I18N.getMessage("modeModel"), ErrManager.ER_ERROR, I18N.getFormated("modeVinc",inon));
	  		return false;
	 	}


	   	Connection c=new Connection(hConnection);
	 	vConnections.addElement(c);
	 	c=null;
	 	return true;
   	}

//--------------------------------------------------------------------------------	
	/** 
  	* Removes Connection from vConnections vector.
  	* @param i number of Connection to remove
	*/
	public void removeConnection(int i)
	{
			
	 		vConnections.removeElementAt(i);
	 	 	return;
	 	
	}
//--------------------------------------------------------------------------------
	/** 
  	* Marks Connection(strong selection). 
  	* @param nr number of Connection to mark.
  	*/
	public void markConnection(int nr)
	{
		((Element)vConnections.elementAt(nr)).stSelElement();

	}

//------------------------------------------------------------------
	/** 
  	* Unmarks Connection(strong selection). 
  	* @param nr number of Connection to unmark.
  	*/
	public void unmarkConnection(int nr)
	{
		((Element)vConnections.elementAt(nr)).stDesElement();
	}
//--------------------------------------------------------------------------------	
	public String toString()
  	{
    	String ret = new String("");
		for (int i=0; i<vParts.size(); i++)
      	ret += i+vParts.elementAt(i).toString() + "\n";

    	for (int i=0; i<vJoints.size(); i++)
      	ret += i+vJoints.elementAt(i).toString() + "\n";

		for (int i=0; i<vNeurons.size(); i++)
      	ret += i+vNeurons.elementAt(i).toString() + "\n";
	
		for (int i=0; i<vConnections.size(); i++)
      	ret += i+vConnections.elementAt(i).toString() + "\n";

    	return ret;
  	}
//--------------------------------------------------------------------------------	
	/** 
  	* Recounts all positions in 3d due to Delta Joints. 
  	*/
	void recountCo()
	{			
				for (int k=0; k<vParts.size(); k++)
      			{
      				((Part)vParts.elementAt(k)).child=false;
      				((Part)vParts.elementAt(k)).parent=false;
      				((Part)vParts.elementAt(k)).setOrientation();
      			}
      				 
		
    	for (int i=0; i<vJoints.size(); i++)
    	{
			Joint j=(Joint)vJoints.elementAt(i);
    		Part newP2Part = j.calcPosition(vParts);

		
		 	float dxTest=0;
	 		if (j.properties.containsKey("dx")) dxTest=((Float)j.properties.get("dx")).floatValue();
	
		 	if (dxTest!=0) 
	 		{   
	 						
	 			int p1 = ((Integer)j.properties.get("p1")).intValue();
	 			int p2 = ((Integer)j.properties.get("p2")).intValue();
	 		
	 			
	 			((Part)vParts.elementAt(p1)).parent=true;
	 			vParts.setElementAt(newP2Part,p2);
	 			((Part)vParts.elementAt(p2)).child=true;
 			}
			      	
      	}
		
	}

};